/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

/*
   Purpose: Releasing all ressources taken by a s_tpl structure
*/
void                 Free_struct_template(s_tpl *template, _Bool lock_gtk)
{
  unsigned int       a;

  ASSERT( !template, "template = 0" );

  if( template->OperationTable ) {
    for( a = 0; a < template->OperationCount ; ++a )
      Free_struct_operation( template->OperationTable[a], lock_gtk  );
    free( template->OperationTable );
  }

  if( lock_gtk ) { LOCK_GTK(); }
  if( template->MainBox ) gtk_widget_destroy( template->MainBox );
  if( template->Iter.stamp ) gtk_list_store_remove( app->Gui.ListTemplates, &template->Iter );
  if( template->MenuItem ) gtk_widget_destroy( template->MenuItem );
  if( lock_gtk ) { UNLOCK_GTK(); }

  free( (char*) template->HeaderOptions );
  free( (char*) template->Name );
  free( template );
}

/*
   Purpose: Releasing all ressources taken by a search structure
*/
void                 Free_struct_search(s_so *search, _Bool lock_gtk)
{
  unsigned int       a;

  ASSERT( !search, "search = 0" );

  free( search->Url );
  free( search->CacheFile );
  if( search->OperationTable ) {
    for( a = 0; a < search->OperationCount ; ++a )
      Free_struct_operation( search->OperationTable[a], lock_gtk );
    free( search->OperationTable );
  }
  free( search );
}

/*
   Purpose: Releasing all ressources taken by a s_op structure
*/
void                 Free_struct_operation(s_op *operation, _Bool lock_gtk)
{
  ASSERT( !operation, "operation = 0" );

  if( operation->GtkTable ) {
    if( lock_gtk ) { LOCK_GTK(); }
    /* here GTK should destroy all hbox child, else ... memleak */
    gtk_widget_destroy( operation->GtkTable[0] );
    if( lock_gtk ) { UNLOCK_GTK(); }
    free( operation->GtkTable );
  }
  free( (char*) operation->Pattern );
  free( operation );
}

/*
   Purpose: Releasing all ressources taken by a s_wtr structure
*/
void                 Free_struct_watcher(s_wtr *watcher, _Bool free_ptr, _Bool lock_gtk)
{
  unsigned int       a;

  ASSERT( !watcher, "watcher = 0" );

  free( watcher->ImgPath );
  free( watcher->Name );
  free( watcher->LocalArg );
  free( watcher->LocalExec );
  free( watcher->LocalVersion );
  free( watcher->OfficialURL );
  free( watcher->OfficialHeader );
  free( watcher->OfficialVersion );
  free( watcher->TemplateName );
  free( watcher->Trigger_on_mismatch );
  free( watcher->Trigger_on_error );

  if( watcher->OfficialOperationTable ) {
    for( a = 0; a < watcher->OfficialOperationCount ; ++a )
      Free_struct_operation( watcher->OfficialOperationTable[a], lock_gtk );
    free( watcher->OfficialOperationTable );
  }

  if( watcher->LocalOperationTable ) {
    for( a = 0; a < watcher->LocalOperationCount ; ++a )
      Free_struct_operation( watcher->LocalOperationTable[a], lock_gtk );
    free( watcher->LocalOperationTable );
  }

  if( lock_gtk ) { LOCK_GTK(); }

  if( watcher->DrawArea ) {
    if( !( watcher->Flags & WTR_FLG_DISABLE ))
      gtk_container_remove( GTK_CONTAINER(app->Gui.Main[MW_TABLE_MAIN]), watcher->DrawArea );
    gtk_widget_destroy( watcher->DrawArea );
  }

  if( watcher->OriginPixbuf )
    g_object_unref( watcher->OriginPixbuf ); //#will this destroy pix ?
  if( watcher->OriginPixbuf != watcher->CurrentPixbuf )
    g_object_unref( watcher->CurrentPixbuf ); //#will this destroy pix ?

  if( watcher->InfosWindow ) gtk_widget_destroy( watcher->InfosWindow );
  if( watcher->Iter.stamp ) gtk_list_store_remove( app->Gui.ListWatchers, &watcher->Iter );

  if( lock_gtk ) { UNLOCK_GTK(); }

  if( free_ptr )
    free( watcher );
}
