/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'launchpadctrl'.
 *
 * 'launchpadctrl' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'launchpadctrl' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'launchpadctrl'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

/* Printing help */
static void         Usage()
{
  printf( "\nUsage: %s [OPTIONS] [-l <file>]\nOPTIONS:\n", APP_SH_NAME );
  printf( " -C, --no-colors\t%s\n", app->Strs[9] ); /* "Disable colors" */
  printf( " -w\t\t\t%s\n", app->Strs[258] ); /* "Export english table in the language file format" */
  printf( " -h, --help\t\t%s\n", app->Strs[4] ); /* "Print this help" */
  printf( " -l, --lang-file\t%s\n", app->Strs[8] ); /* "Load a language file" */
  printf( " -p, --phone\t\t(%s) %s\n", app->Strs[172], app->Strs[224] ); /* "Work in Progress" "Adapatation to smartphone" */
  printf( " -v, --version\t\t%s\n", app->Strs[6] ); /* "Print current version and quit" */
  printf( " -V, --verbose\t\t%s\n", app->Strs[145] ); /* "Activate verbose mode" */
  exit( EXIT_FAILURE );
}

/* Manage the '-l' '--lang-file' option */
static int       Option_l(char **av, unsigned int len)
{
  const char     *param;
  int            ret = 1;

  if( len ) {
    switch( av[0][len] ) {
    case '=': param = &av[0][len + 1]; ret = 0; break;
    case '\0': param = av[1]; break;
    default:
      ERR_APP( "%s : '%s'", app->Strs[5], av[0] ); /* "Unknown option" */
      Usage();
    }
  }
  else
    param = av[1];

  if( ! param || !( *param )) {
    ERR_APP( "%s : '%s'", app->Strs[7], av[0] ); /* "Argument is missing for option" */
    Usage();
  }

  Load_file_lang( param );
  return( ret );
}

/* Manage the '-w' option */
static void         Option_w()
{
  unsigned int   a;

  for( a = 0; a < APP_NB_STRINGS; ++a )
    printf( "%u:%s\n", a, English[a] );
  exit( EXIT_SUCCESS );
}

/* Manage the '-v --version' option */
static void         Option_v()
{
  printf( "%s\n", APP_VERSION );
  printf( "GTK %u.%u.%u (compiled with)\n", GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION );
  printf( "GTK %u.%u.%u (actual)\n", gtk_major_version, gtk_minor_version, gtk_micro_version );
#ifdef LIB_HANDY
  printf( "HANDY %u.%u.%u (compiled with)\n", HDY_MAJOR_VERSION, HDY_MINOR_VERSION, HDY_MICRO_VERSION );
#endif
  exit( EXIT_SUCCESS );

}

/* Retrieving options */
void                Get_options(int ac, char **av)
{
  int               a, b;

  for( a = 1; a < ac; ++a ) {
    if( av[a][0] == '-' ) {
      if( av[a][1] == '-' ) {
	if( ! strcmp( &av[a][2], "help" ))
	  Usage();
        else if( ! strcmp( &av[a][2], "no-colors" ))
	  app->Flags |= APP_FLG_NO_COLORS;
        else if( ! strncmp(&av[a][2], "lang-file", 9 )) {
	  a += Option_l( &av[a], 11 );
	}
        else if( ! strcmp(&av[a][2], "verbose" ))
	  app->Flags |= APP_FLG_VERBOSE;
        else if( ! strcmp(&av[a][2], "phone" ))
	  app->Flags |= APP_FLG_PHONE;
        else if( ! strcmp(&av[a][2], "version" ))
	  Option_v();
	else {
	  ERR_APP( "%s : '--%s'", app->Strs[5], &av[a][2] ); /* "Unknown option" */
	  Usage();
	}
      }
      else if( av[a][1] == '\0' ) {
	  ERR_APP( "%s : '-'", app->Strs[5] ); /* "Unknown option" */
	  Usage();
      }
      else { /* if( av[a][1] != '-' ) */
	for( b = 1; b && av[a][b]; ++b ) {
	  switch( av[a][b] ) {
	  case 'C': app->Flags |= APP_FLG_NO_COLORS; break;
	  case 'l': Option_l( &av[a], 0 ); b = -1; break;
	  case 'p': app->Flags |= APP_FLG_PHONE; break;
	  case 'V': app->Flags |= APP_FLG_VERBOSE; break;
	  case 'v': Option_v();
	  case 'w': Option_w();

	  default : ERR_APP( "%s : '%c'", app->Strs[5], av[a][b] ); /* "Unknown option" */
	  case 'h': Usage();
	  }
	}
      }
    }
  }
}
