/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

static const char  *section_name = "preference";

/*
  Purpose: Handeling key name 'main_coords'
*/
static void              key_main_coords(s_file *file, s_src *source, s_src *value)
{
  unsigned int           x, y, err;

  if(( err =  Convert_source_to_number( file, value, &x ))) {
    Pop_error( file, section_name, "main_coords", app->Strs[err] );
    return;
  }

  if( value->Pos >= value->Length )
    goto key_main_coords_FAIL;

  switch( value->Mem[value->Pos] ) {
  case ',':  case '*':  case ' ':
  case '-':  case '.':  case ':':
    break;
  default:
    goto key_main_coords_FAIL;
  }
  ++value->Pos;

  if(( err =  Convert_source_to_number( file, value, &y )))
    goto key_main_coords_FAIL;

  if( app->Flags & APP_FLG_INIT ) {
    app->Gui.MainX = x;
    app->Gui.MainY = y;
    app->Gui.SaveX = x;
    app->Gui.SaveY = y;
  }

  return;

 key_main_coords_FAIL:
  Pop_error( file, section_name, "main_coords", app->Strs[162] ); /* "Format of value must be x,y (where ',' can also be ' ', '.', ':', '*', or '-'" */
}

/*
  Purpose: Handeling key name 'max_width'
*/
static void              key_max_width(s_file *file, s_src *source, s_src *value)
{
  unsigned int           nb, err;

  if(( err =  Convert_source_to_number( file, value, &nb ))) {
    Pop_error( file, section_name, "max_width", app->Strs[err] );
    return;
  }

  if( nb > MAX_COUNT_WIDTH ) {
    Pop_error( file, section_name, "max_width", app->Strs[82] ); /* "Number is out of parameter range" */
    return;
  }

  app->Gui.MaxXWatcher = nb;
}

/*
  Purpose: Handeling key name 'max_height'
*/
static void              key_max_height(s_file *file, s_src *source, s_src *value)
{
  unsigned int           nb, err;

  if(( err =  Convert_source_to_number( file, value, &nb ))) {
    Pop_error( file, section_name, "max_height", app->Strs[err] );
    return;
  }

  if( nb > MAX_COUNT_WIDTH ) {
    Pop_error( file, section_name, "max_height", app->Strs[82] ); /* "Number is out of parameter range" */
    return;
  }

  app->Gui.MaxYWatcher = nb;
  return;
}

/*
  Purpose: Handeling key name 'vertical_expansion'
*/
static void              key_vertical_expansion(s_file *file, s_src *source, s_src *value)
{
  unsigned int           err;
  _Bool                  state;

  if(( err =  Convert_source_to_boolean( file, value, &state )) > 1 ) {
    Pop_error( file, section_name, "vertical_expansion", app->Strs[err] );
    return;
  }

  if( state )
    app->Gui.Flags |= GUI_FLG_VERTICAL_EXPAND;
  else
    app->Gui.Flags &= ~( GUI_FLG_VERTICAL_EXPAND );
}

/*
  Purpose: Handeling key name 'lock_position'
*/
static void              key_lock_position(s_file *file, s_src *source, s_src *value)
{
  unsigned int           err;
  _Bool                  state;

  if(( err =  Convert_source_to_boolean( file, value, &state )) > 1 ) {
    Pop_error( file, section_name, "lock_position", app->Strs[err] );
    return;
  }

  if( state )
    app->Gui.Flags |= GUI_FLG_LOCK_MAIN;
  else
    app->Gui.Flags &= ~( GUI_FLG_LOCK_MAIN );
}

/*
  Purpose: Handeling key name 'always_on'
*/
void                     key_always_on(s_file *file, s_src *source, s_src *value)
{
  if( value->Length == 3 && ! strncasecmp( value->Mem, "top", 3 ))
    app->Gui.Flags |= GUI_FLG_ON_TOP;
  else if( value->Length == 6 && ! strncasecmp( value->Mem, "bottom", 6 ))
    app->Gui.Flags &= ~( GUI_FLG_ON_TOP );
  else
    Pop_error( file, section_name, "always_on", app->Strs[83] ); /* "Unknwon value for parameter" */
}

/*
  Purpose: Converting hexadecimal to decimal
*/
static _Bool             hexa_to_decimal(char c, guint16 *value)
{
  *value *= 16;

  if( c < '0' )
    return( 1 );
  if( c > '9' ) {
    if( c < 'A' )
      return( 1 );
    if( c > 'Z' ) {
      if( c < 'a'  || c > 'z' )
	return( 1 );
      *value += c - 'a' + 10;
    }
    else
      *value += c - 'A' + 10;
  }
  else
    *value += c - '0';

  return( 0 );
}

/*
  Purpose: Handeling key name 'bg_color'
*/
static void              key_bg_color(s_file *file, s_src *source, s_src *value)
{
  guint16                red, blue, green, opacity;

  if( value->Length < 8 ) {
    Pop_error( file, section_name, "bg_color", app->Strs[161] ); /* "Format of value must be RRGGBBAA in hexadecimal" */
    return;
  }
  red = 0;
  if( hexa_to_decimal( value->Mem[0], &red )
      || hexa_to_decimal( value->Mem[1], &red ))
    return;

  green = 0;
  if( hexa_to_decimal( value->Mem[2], &green )
      || hexa_to_decimal( value->Mem[3], &green ))
    return;

  blue = 0;
  if( hexa_to_decimal( value->Mem[4], &blue )
      || hexa_to_decimal( value->Mem[5], &blue ))
    return;

  opacity = 0;
  if( hexa_to_decimal( value->Mem[6], &opacity )
      || hexa_to_decimal( value->Mem[7], &opacity ))
    return;

#if GTK_MAJOR_VERSION >= 3
  app->Gui.MainColor.red = red / 256.0;
  app->Gui.MainColor.green = green / 256.0;
  app->Gui.MainColor.blue = blue / 256.0;
  app->Gui.MainColor.alpha = opacity / 256.0;
#else
  app->Gui.MainColor.red = ( red << 8 ) + red;
  app->Gui.MainColor.green = ( green << 8 ) + green;
  app->Gui.MainColor.blue = ( blue << 8 ) + blue;
  app->Gui.MainOpacity = opacity / 256.0;
#endif
}

/*
  Purpose: Trying to create a folder with non existing sub-folders
  - return 0 on success
*/
_Bool                    Try_create_folder(char *folder, unsigned int length)
{
  static unsigned char   secu = 0;
  struct stat            st;
  _Bool                  ret;

  if( ! length || secu > 100 )
    return( 1 );

  if( mkdir( folder, S_IRWXU )) {
    if( errno == ENOENT ) { /* A directory component in pathname does not exist or is a dangling symbolic link */
      for( --length; length > 1; --length )
	if( folder[length] == '/' ) {
	  folder[length] = '\0';
	  ++secu;
	  ret = Try_create_folder( folder, length );
	  --secu;
	  folder[length] = '/';
	  if( ! ret )
	    break;
	  return( 1 );
	}
      if( mkdir( folder, S_IRWXU ))
	return( 1 );
      return( 0 );
    }

    if( errno == EEXIST ) { /* pathname already exists */
      stat( folder, &st );
      if( ! S_ISDIR( st.st_mode )) {
	errno = EEXIST;
	return( 1 );
      }
      return( 0 );
    }
    return( 1 );
  }
  return( 0 );
}

/*
  Purpose: Handeling key name 'cache_folder'
*/
static void              key_cache_folder(s_file *file, s_src *source, s_src *value)
{
  const char             *home = 0;
  char                   *path;
  unsigned int           home_length = 0;

  if( value->Mem[0] == '~' ) {
    home = app->UserHome;
    home_length = app->UserHomeLength;
  }

  if( value->Mem[0] != '/' && ! home ) {
    Pop_error( file, section_name, "cache_folder", app->Strs[154] ); /* "An absolute path is required" */
    return;
  }

  MALLOC( path, home_length + value->Length + (home_length == 0) );
  if( home_length ) {
    memcpy( path, home, home_length );
    --value->Length; /* won't copy '~' */
    memcpy( &path[home_length], &value->Mem[1], value->Length );
    path[home_length + value->Length] = '\0';
  }
  else {
    memcpy( path, value->Mem, value->Length );
    path[value->Length] = '\0';
  }

  if( Try_create_folder( path, home_length + value->Length )) {
    Pop_error( file, section_name, "cache_folder", strerror( errno )); /* "Unable to create folder" */
    free( path );
    return;
  }

  free( app->CacheFolder );
  app->CacheFolder = path;
}

/*
  Purpose: Handeling key name 'phone'
*/
static void              key_phone(s_file *file, s_src *source, s_src *value)
{
  unsigned int           err;
  _Bool                  state;

  if(( err =  Convert_source_to_boolean( file, value, &state )) > 1 ) {
    Pop_error( file, section_name, "phone", app->Strs[err] );
    return;
  }

  if( state )
    app->Flags |= APP_FLG_PHONE | APP_FLG_FORCE_PHONE;
  else
    app->Flags &= ~( APP_FLG_PHONE );
}

/*
  Purpose: Handeling key name 'size_icon'
*/
static void              key_size_icon(s_file *file, s_src *source, s_src *value)
{
  unsigned int           nb, err;

  if(( err =  Convert_source_to_number( file, value, &nb ))) {
    Pop_error( file, section_name, "size_icon", app->Strs[err] );
    return;
  }

  if( nb < 26 || nb > 128 ) {
    Pop_error( file, section_name, "size_icon", app->Strs[82] ); /* "Number is out of parameter range" */
    return;
  }

  app->Gui.WatcherSizeX = nb;
  app->Gui.WatcherSizeY = nb;
}

/*
  Purpose: Handeling key name 'systray'
*/
static void              key_systray(s_file *file, s_src *source, s_src *value)
{
  unsigned int           err;
  _Bool                  state;

  if(( err =  Convert_source_to_boolean( file, value, &state )) > 1 ) {
    Pop_error( file, section_name, "disable", app->Strs[err] );
    return;
  }

  if(( app->Flags & APP_FLG_PHONE ) || ! state )
    app->Flags &= ~( APP_FLG_SYSTRAY );
  else
    app->Flags |= APP_FLG_SYSTRAY;
}

/*
  Purpose: Handeling key name 'infosview_expansion'
*/
static void              key_infosview_expansion(s_file *file, s_src *source, s_src *value)
{
  app->Gui.Flags &= ~( GUI_FLG_INFOS_MASK );
  if( value->Length == 3 && ! strncasecmp( value->Mem, "top", 3 ))
    app->Gui.Flags |= GUI_FLG_INFOS_TO_TOP;
  else if( value->Length == 4 && ! strncasecmp( value->Mem, "left", 4 ))
    app->Gui.Flags |= GUI_FLG_INFOS_TO_LEFT;
  else if( value->Length == 5 && ! strncasecmp( value->Mem, "right", 5 ))
    app->Gui.Flags |= GUI_FLG_INFOS_TO_RIGHT;
  else if( value->Length == 6 && ! strncasecmp( value->Mem, "bottom", 6 ))
    /* nothing to do */ ;
  else
    Pop_error( file, section_name, "infosview_expansion", app->Strs[83] ); /* "Unknwon value for parameter" */
}

/*
  Purpose: Contains all key names to configure preferences
*/
static const s_pkn     preferences_key_name[] = {
  { "always_on"          , key_always_on          , 9 },
  { "bg_color"           , key_bg_color           , 8 },
  { "cache_folder"       , key_cache_folder       , 12 },
  { "infosview_expansion", key_infosview_expansion, 19 },
  { "lock_position"      , key_lock_position      , 13 },
  { "main_coords"        , key_main_coords        , 11 },
  { "max_height"         , key_max_height         , 10 },
  { "max_width"          , key_max_width          , 9 },
  { "phone"              , key_phone              , 5 },
  { "size_icon"          , key_size_icon          , 9 },
  { "systray"            , key_systray            , 7 },
  { "vertical_expansion" , key_vertical_expansion , 18 },
};
#define COUNT_PREF_KEY_NAME     (sizeof(preferences_key_name) / sizeof(*preferences_key_name))

/*
  Purpose: Getting preferences
*/
_Bool                   Get_preferences(s_file *file, s_src *source)
{
  char                  *str;
  const s_pkn           *item;
  s_src                 value;
  unsigned int          a, len, err;
  char                  c;

  for( ++source->Pos ; source->Pos < source->Length && source->Mem[source->Pos] != '\n'; ++source->Pos );
  if( source->Pos >= source->Length )
    return( 0 );
  ++file->Line;

  for( ++source->Pos; source->Pos < source->Length && ( source->Mem[source->Pos] == ' ' || source->Mem[source->Pos] == '\t' || source->Mem[source->Pos] == '\n'); ++source->Pos )
    if( source->Mem[source->Pos] == '\n' )
      ++file->Line;

  for( ; source->Pos < source->Length; ++source->Pos ) {
    c = source->Mem[source->Pos];
    if( c == '\n' ) {
      ++file->Line;
      for( ++source->Pos ; source->Pos < source->Length && ( source->Mem[source->Pos] == ' ' || source->Mem[source->Pos] == '\t' || source->Mem[source->Pos] == '\n'); ++source->Pos )
	if( source->Mem[source->Pos] == '\n' )
	  ++file->Line;
      --source->Pos;
      continue;
    }
    if( c == '[' ) { /* Other section found */
      --source->Pos;
      return( 0 );
    }
    if( c == '#' ) { /* Skipping comments */
      for( ++source->Pos; source->Pos < source->Length && source->Mem[source->Pos] != '\n'; ++source->Pos );
      --source->Pos;
      continue;
    }

    for( a = 0; a < COUNT_PREF_KEY_NAME; ++a ) {
      item = &preferences_key_name[a];
      if( c == item->Name[0] ) {
	if( source->Pos + item->Length <= source->Length && !strncmp( item->Name, &source->Mem[source->Pos], item->Length )) {
	  source->Pos += item->Length;
	  /* key_name found */
	  break;
	}
      }
    }

    if( a == COUNT_PREF_KEY_NAME && !(( item = Get_obsolete_preferences_properties( file, source )))) {
      /* key_name not found */
      len = source->Pos;
      for( ; source->Pos < source->Length; ++source->Pos )
	if( source->Mem[source->Pos] == ' ' ||  source->Mem[source->Pos] == '\t'
	    || source->Mem[source->Pos] == '\n' || source->Mem[source->Pos] == app->ConfigSeparator )
	  break;
      len = source->Pos - len;
      MALLOC( str, len + 1 );
      if( len )
	memcpy( str, &source->Mem[source->Pos - len], len);
      str[len] = '\0';
      Pop_error( file, section_name, str, app->Strs[78] ); /* "Unknown key name" */
      free( str );
      for( ++source->Pos ; source->Pos < source->Length && source->Mem[source->Pos] != '\n'; ++source->Pos );
      ++file->Line;
    }
    else {
      if(( err = Get_next_value( file, source, &value ))) {
	if( err == 2 )
	  Pop_error( file, section_name, item->Name, app->Strs[84] ); /* "Missing closing character" */
	else
	  Pop_error( file, section_name, item->Name, app->Strs[79] ); /* "Missing value" */
	return( 1 );
      }

      ASSERT( !item->Func, "Function for '%s' key, has not been set", item->Name );
      item->Func( file, source, &value );
    }
    for( ; source->Pos < source->Length && source->Mem[source->Pos] != '\n'; ++source->Pos );
    ++file->Line;
  }

  return( 0 );
}
