/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

/*
  Purpose: Handeling key name 'official_after_contains'
  Obsolete since: 1.9
*/
static void             key_official_after_contains(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                  *operation = 0;
  char                  *pattern;
  unsigned int          a;

  for( a = 0; a < watcher->OfficialOperationCount; ++a ) {
    operation = watcher->OfficialOperationTable[a];
    if( operation->Type == OP_TYPE_AFTER_LINE )
      break;
  }

  MALLOC( pattern, value->Length + 1 );
  memcpy( pattern, value->Mem, value->Length );
  pattern[value->Length] = '\0';

  if( a == watcher->OfficialOperationCount ) {
    operation = New_operation_after_line( pattern );
    Add_official_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Pattern = pattern;
    gtk_entry_set_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_AFTER_LINE_PATTERN]), pattern );
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'official_cut_pos'
  Obsolete since: 1.9
*/
static void             key_official_cut_pos(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                  *operation;
  unsigned int          a, err, pos;

  if(( err =  Convert_source_to_number( file, value, &pos ))) {
    Pop_error( file, watcher->Name, "official_cut_pos", app->Strs[err] );
    return;
  }

  if( ! pos || pos > 65535 ) {
    Pop_error( file, watcher->Name, "official_cut_pos", app->Strs[107] ); /* "Number is out of bounds" */
    return;
  }

  for( a = 0; a < watcher->OfficialOperationCount; ++a ) {
    operation = watcher->OfficialOperationTable[a];
    if( operation->Type == OP_TYPE_IN_FIELD )
      break;
  }

  if( a == watcher->OfficialOperationCount ) {
    for( a = 0; a < watcher->OfficialOperationCount; ++a ) {
      operation = watcher->OfficialOperationTable[a];
      if( operation->Type == OP_TYPE_IN_SECTION )
	break;
    }
    if( a == watcher->OfficialOperationCount ) {
      operation = New_operation_in_section( 1, '<', '>' );
      Add_official_operation_to_watcher( watcher, operation );
    }
    operation = New_operation_in_field( 1, ' ' );
    Add_official_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Number = pos;
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_FIELD_NB]), pos );
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'official_cut_with'
  Obsolete since: 1.9
*/
static void             key_official_cut_with(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                  *operation = 0;
  unsigned int          a;

  if( value->Mem[0] < ' ' || value->Mem[0] > '~' ) {
    Pop_error( file, watcher->Name, "official_cut_with", app->Strs[121] ); /* "The character specified to cut the word is not permitted" */
    return;
  }

  for( a = 0; a < watcher->OfficialOperationCount; ++a ) {
    operation = watcher->OfficialOperationTable[a];
    if( operation->Type == OP_TYPE_IN_FIELD )
      break;
  }

  if( a == watcher->OfficialOperationCount ) {
    for( a = 0; a < watcher->OfficialOperationCount; ++a ) {
      operation = watcher->OfficialOperationTable[a];
      if( operation->Type == OP_TYPE_IN_SECTION )
	break;
    }
    if( a == watcher->OfficialOperationCount ) {
      operation = New_operation_in_section( 1, '<', '>' );
      Add_official_operation_to_watcher( watcher, operation );
    }
    operation = New_operation_in_field( 1, value->Mem[0] );
    Add_official_operation_to_watcher( watcher, operation );
  }
  else {
    char delim[2];
    delim[0] = value->Mem[0];
    delim[1] = '\0';
    gtk_entry_set_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_FIELD_CHAR]), delim );
    operation->Delimiter[0] = value->Mem[0];
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'official_line_contains'
  Obsolete since: 1.9
*/
static void             key_official_line_contains(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                  *operation = 0;
  char                  *pattern;
  unsigned int          a;

  for( a = 0; a < watcher->OfficialOperationCount; ++a ) {
    operation = watcher->OfficialOperationTable[a];
    if( operation->Type == OP_TYPE_IN_LINE )
      break;
  }

  MALLOC( pattern, value->Length + 1 );
  memcpy( pattern, value->Mem, value->Length );
  pattern[value->Length] = '\0';

  if( a == watcher->OfficialOperationCount ) {
    operation = New_operation_in_line( 1, 1, pattern );
    Add_official_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Activated = 1;
    operation->Pattern = pattern;

    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(operation->GtkTable[OP_WIDGET_IN_LINE_CHECK]), 1 );
    gtk_entry_set_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_LINE_PATTERN]), pattern );
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'official_field'
  Obsolete since: 1.9
*/
static void             key_official_field(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                  *operation = 0;
  unsigned int          a, err, pos;

  if(( err =  Convert_source_to_number( file, value, &pos ))) {
    Pop_error( file, watcher->Name, "official_field", app->Strs[err] );
    return;
  }

  if( ! pos || pos > 65535 ) {
    Pop_error( file, watcher->Name, "official_field", app->Strs[107] ); /* "Number is out of bounds" */
    return;
  }

  for( a = 0; a < watcher->OfficialOperationCount; ++a ) {
    operation = watcher->OfficialOperationTable[a];
    if( operation->Type == OP_TYPE_IN_SECTION )
      break;
  }

  if( a == watcher->OfficialOperationCount ) {
    operation = New_operation_in_section( pos, '<', '>' );
    Add_official_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Number = pos;
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'official_line'
*/
static void             key_official_line(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                  *operation;
  unsigned int          a, err, nb_line;

  if(( err =  Convert_source_to_number( file, value, &nb_line ))) {
    Pop_error( file, watcher->Name, "official_line", app->Strs[err] );
    return;
  }

  if( ! nb_line || nb_line > 65535 ) {
    Pop_error( file, watcher->Name, "official_line", app->Strs[107] ); /* "Number is out of bounds" */
    return;
  }

  for( a = 0; a < watcher->OfficialOperationCount; ++a ) {
    operation = watcher->OfficialOperationTable[a];
    if( operation->Type == OP_TYPE_IN_LINE )
      break;
  }

  if( a == watcher->OfficialOperationCount ) {
    operation = New_operation_in_line( nb_line, 0, 0 );
    Add_official_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Number = nb_line;
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_LINE_NB]), nb_line );
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'local_line'
  Obsolete since: 2.0
*/
static void             key_local_line(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                  *operation;
  unsigned int          a, err, nb_line;

  if(( err =  Convert_source_to_number( file, value, &nb_line ))) {
    Pop_error( file, watcher->Name, "local_line", app->Strs[err] );
    return;
  }

  if( ! nb_line || nb_line > 65535 ) {
    Pop_error( file, watcher->Name, "local_line", app->Strs[107] ); /* "Number is out of bounds" */
    return;
  }

  for( a = 0; a < watcher->LocalOperationCount; ++a ) {
    operation = watcher->LocalOperationTable[a];
    if( operation->Type == OP_TYPE_IN_LINE )
      break;
  }

  if( a == watcher->LocalOperationCount ) {
    operation = New_operation_in_line( nb_line, 0, 0 );
    Add_local_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Number = nb_line;
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_LINE_NB]), nb_line );
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'local_contains'
*/
static void              key_local_contains(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                   *operation;
  char                   *pattern;
  unsigned int           a;

  MALLOC( pattern, value->Length + 1 ); /* + '\0' */
  memcpy( pattern, value->Mem, value->Length );
  pattern[value->Length] = '\0';

  for( a = 0; a < watcher->LocalOperationCount; ++a ) {
    operation = watcher->LocalOperationTable[a];
    if( operation->Type == OP_TYPE_IN_LINE )
      break;
  }

  if( a == watcher->LocalOperationCount ) {
    operation = New_operation_in_line( 1, 1, pattern );
    Add_local_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Pattern = pattern;
    operation->Activated = 1;
    gtk_entry_set_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_LINE_PATTERN]), pattern );
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'local_cut_pos'
  Obsolete since: 2.0
*/
static void              key_local_cut_pos(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                   *operation;
  unsigned int           a, err, pos;

  if(( err =  Convert_source_to_number( file, value, &pos ))) {
    Pop_error( file, watcher->Name, "local_cut_pos", app->Strs[err] );
    return;
  }

  if( ! pos || pos > 255 ) {
    Pop_error( file, watcher->Name, "local_cut_pos", app->Strs[107] ); /* "Number is out of bounds" */
    return;
  }

  for( a = 0; a < watcher->LocalOperationCount; ++a ) {
    operation = watcher->LocalOperationTable[a];
    if( operation->Type == OP_TYPE_IN_FIELD )
      break;
  }

  if( a == watcher->LocalOperationCount ) {
    operation = New_operation_in_field( pos, ' ' );
    Add_official_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Number = pos;
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_FIELD_NB]), pos );
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'local_cut_with'
  Obsolete since: 2.0
*/
static void              key_local_cut_with(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                   *operation;
  unsigned int           a;

  if( value->Mem[0] < ' ' || value->Mem[0] > '~' ) {
    Pop_error( file, watcher->Name, "local_cut_with", app->Strs[121] ); /* "The character specified to cut the word is not permitted" */
    return;
  }

  for( a = 0; a < watcher->LocalOperationCount; ++a ) {
    operation = watcher->LocalOperationTable[a];
    if( operation->Type == OP_TYPE_IN_FIELD )
      break;
  }

  if( a == watcher->LocalOperationCount ) {
    operation = New_operation_in_field( 1, value->Mem[0] );
    Add_local_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Delimiter[0] = value->Mem[0];
    operation->Delimiter[1] = '\0';
    gtk_entry_set_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_FIELD_CHAR]), operation->Delimiter );
  }
  operation->Reorder = 1;
}

/*
  Purpose: Handeling key name 'local_word'
  Obsolete since: 2.0
*/
static void             key_local_word(s_file *file, s_src *source, s_wtr *watcher, s_src *value)
{
  s_op                  *operation;
  unsigned int          a, err, nb_word;

  if(( err =  Convert_source_to_number( file, value, &nb_word ))) {
    Pop_error( file, watcher->Name, "local_word", app->Strs[err] );
    return;
  }

  if( ! nb_word || nb_word > 65535 ) {
    Pop_error( file, watcher->Name, "local_word", app->Strs[107] ); /* "Number is out of bounds" */
    return;
  }

  for( a = 0; a < watcher->LocalOperationCount; ++a ) {
    operation = watcher->LocalOperationTable[a];
    if( operation->Type == OP_TYPE_IN_WORD )
      break;
  }

  if( a == watcher->LocalOperationCount ) {
    operation = New_operation_in_word( nb_word, 0 );
    Add_local_operation_to_watcher( watcher, operation );
  }
  else {
    operation->Number = nb_word;
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_LINE_NB]), nb_word );
  }
  operation->Reorder = 1;
}

/*
  Purpose: Contains search key names to configure a watcher
*/
static const s_wkn      watcher_key_name[] = {
  { "official_after_contains", key_official_after_contains, 23 },
  { "official_cut_pos"       , key_official_cut_pos       , 16 },
  { "official_cut_with"      , key_official_cut_with      , 17 },
  { "official_field"         , key_official_field         , 14 },
  { "official_line_contains" , key_official_line_contains , 22 },
  { "official_line"          , key_official_line          , 13 },
  { "local_cut_pos"          , key_local_cut_pos          , 13 },
  { "local_cut_with"         , key_local_cut_with         , 14 },
  { "local_line_contains"    , key_local_contains         , 19 },
  { "local_line"             , key_local_line             , 10 },
  { "local_word"             , key_local_word             , 10 }
};
#define COUNT_KEY_NAME    (sizeof(watcher_key_name) / sizeof(*watcher_key_name))

/*
  Purpose: Getting the properties of the watcher from the source
*/
const s_wkn             *Get_obsolete_watchers_properties(s_file *file, s_src *source, s_wtr *watcher)
{
  const s_wkn           *item;
  unsigned int          a;
  char                  c;

  c = source->Mem[source->Pos];
  for( a = 0; a < COUNT_KEY_NAME; ++a ) {
    item = &watcher_key_name[a];
    if( c == item->Name[0] ) {
      if( source->Pos + item->Length <= source->Length && !strncmp( item->Name, &source->Mem[source->Pos], item->Length )) {
	WARN_APP( "%s: '%s': %s", watcher->Name, item->Name, app->Strs[237] ); /* "Key name is obsolete and will be replace" */
	return( item );
      }
    }
  }
  return( 0 );
}

/*
  Purpose: Contains all key names to configure preferences
*/
static const s_pkn     preferences_key_name[] = {
  { "always_on"          , key_always_on          , 9 }
};
#define COUNT_PREF_NAME     (sizeof(preferences_key_name) / sizeof(*preferences_key_name))

/*
  Purpose: Getting the properties of the watcher from the source
*/
const s_pkn             *Get_obsolete_preferences_properties(s_file *file, s_src *source)
{
  const s_pkn           *item;
  unsigned int          a;
  char                  c;

  c = source->Mem[source->Pos];
  for( a = 0; a < COUNT_PREF_NAME; ++a ) {
    item = &preferences_key_name[a];
    if( c == item->Name[0] ) {
      if( source->Pos + item->Length <= source->Length && !strncmp( item->Name, &source->Mem[source->Pos], item->Length )) {
	WARN_APP( "[preferences]: '%s': %s", item->Name, app->Strs[237] ); /* "Key name is obsolete and will be replace" */
	return( item );
      }
    }
  }
  return( 0 );
}
