/*
 *  'UpdatesWarner' alerts you about system failures on your distant linux hosts
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

static void         Callback_hide_about()
{
  gtk_widget_hide( app->About[ABW_WIN_MAIN] );
}

void                Create_window_about()
{
  GtkWidget         *win, *frame, *vbox, *vbox_main, *item;
  GError            *err;
  char              *str, gtk_version[55];

  /* Window */
#ifdef LIB_HANDY
  win = hdy_window_new();
#else
  win = gtk_window_new( GTK_WINDOW_TOPLEVEL );
#endif
  g_signal_connect( G_OBJECT(win), "delete_event", G_CALLBACK(Callback_on_delete), 0 );
    g_signal_connect( G_OBJECT(win), "destroy", G_CALLBACK(Callback_on_destroy), 0 );
  gtk_window_set_title( GTK_WINDOW(win), app->Strs[205] ); /* "About..." */
  gtk_window_set_icon_from_file( GTK_WINDOW(win), DEFAULT_APP_RESOURCES"/icon_about.png", &err );
  /* gtk_window_set_default_size( GTK_WINDOW(win), 300, 150 ); */
  /* gtk_window_set_resizable( GTK_WINDOW(win), FALSE ); */
  app->About[ABW_WIN_MAIN] = win;

  vbox_main = APP_GTK_VBOX( FALSE, 0 );

#ifdef LIB_HANDY
  gtk_container_add( GTK_CONTAINER(win), vbox_main );

  GtkWidget *bar = hdy_header_bar_new();
  g_object_set( bar, "show_close_button", TRUE,
                "title", app->Strs[205], /* "About..." */
                "can-focus", FALSE,
                NULL );
  gtk_container_add( GTK_CONTAINER(vbox_main), bar );
  frame = hdy_preferences_group_new();
#else
  Set_window_properties( win, vbox_main );
  frame = gtk_frame_new( app->Strs[205] ); /* "About..." */
#endif
  gtk_container_add( GTK_CONTAINER(vbox_main), frame );

  /* Text */
  vbox = APP_GTK_VBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(frame), vbox );

  item = gtk_label_new( APP_NAME " " APP_VERSION );
  gtk_container_add( GTK_CONTAINER(vbox), item );

  snprintf( gtk_version, 55, "GTK version: %u.%u.%u (compiled) / %u.%u.%u (actual)\n",
	    GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
	    gtk_major_version, gtk_minor_version, gtk_micro_version );
  item = gtk_label_new( gtk_version );
  gtk_container_add( GTK_CONTAINER(vbox), item );

  item = gtk_label_new( app->Strs[207] ); /* " Last version of UpdatesWarner can be found in the following URL :" */
  gtk_container_add( GTK_CONTAINER(vbox), item );

  item = gtk_link_button_new( "https://videca.fr/updates-warner/" );
  gtk_container_add( GTK_CONTAINER(vbox), item );

  item = gtk_label_new( "\n\nCopyright (C) 2017-2021 Vincent Deca." );
  gtk_container_add( GTK_CONTAINER(vbox), item );

  item = gtk_label_new( " known as Bigcake@ubuntu-fr.org (user@forum not email)" );
  gtk_container_add( GTK_CONTAINER(vbox), item );

  str = " 'UpdatesWarner' is free software: you can redistribute it and/or modify it under the terms\n of the GNU General Public License as published by the Free Software Foundation, \n either version 3 of the License, or any later version.";
  item = gtk_label_new( str );
  gtk_container_add( GTK_CONTAINER(vbox), item );

  /* Button */
  item = gtk_button_new_with_mnemonic( app->Strs[206] ); /* "_Close" */
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_hide_about), 0 );
  gtk_container_add( GTK_CONTAINER(vbox_main), item );

  gtk_widget_show_all( vbox_main );
}
