/*
 *  'UpdatesWarner' alerts you about system failures on your distant linux hosts
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

static void         Callback_hide_error()
{
  gtk_label_set_label( GTK_LABEL(app->Error[ERW_LBL_MESSAGE]), "" );
  gtk_widget_hide( app->Error[ERW_WIN_MAIN] );
}

void                Create_window_error()
{
  GtkWidget         *frame, *win, *hbox, *vbox_main, *item;
  GError            *err;

  /* Window */
#ifdef LIB_HANDY
  if( app->Flags & APP_FLG_PHONE )
    win = hdy_window_new();
  else
#endif
    win = gtk_window_new( GTK_WINDOW_TOPLEVEL );
  g_signal_connect( G_OBJECT(win), "delete_event", G_CALLBACK(Callback_on_delete), 0 );
  g_signal_connect( G_OBJECT(win), "destroy", G_CALLBACK(Callback_on_destroy), 0 );

  gtk_window_set_title( GTK_WINDOW(win), app->Strs[216] ); /* "Error..." */
  gtk_window_set_icon_from_file( GTK_WINDOW(win), DEFAULT_APP_RESOURCES"/icon_error.png", &err );
  app->Error[ERW_WIN_MAIN] = win;

  vbox_main = APP_GTK_VBOX( FALSE, 0 );

#ifdef LIB_HANDY
  GtkWidget *leaf = hdy_leaflet_new();
  gtk_container_add( GTK_CONTAINER(win), leaf );
  gtk_widget_show_all( leaf );

  gtk_container_add( GTK_CONTAINER(leaf), vbox_main );

  if( app->Flags & APP_FLG_PHONE ) {
    GtkWidget *bar = hdy_header_bar_new();
    g_object_set( bar, "show_close_button", TRUE,
		  "title", app->Strs[216], /* "Error..." */
		  "can-focus", FALSE,
		  NULL );
    gtk_container_add( GTK_CONTAINER(vbox_main), bar );
  }
  else {
    gint width, height;
    SCREEN_WIDTH_HEIGHT( width, height );
    if( width > 500 ) width = 500;
    else if( width > 360 ) width = 360;
    height = 30;
    gtk_window_set_default_size( GTK_WINDOW(win), width, height );
    gtk_window_set_resizable( GTK_WINDOW(win), FALSE );
  }

  frame = hdy_preferences_group_new();
#else
  Set_window_properties( win, vbox_main );
  frame = gtk_frame_new( app->Strs[0] ); /* "Error" */
#endif

  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(frame), hbox );

  /* img */
  item = gtk_image_new_from_file( DEFAULT_APP_RESOURCES"/icon_error.png" );
  if( app->Flags & APP_FLG_PHONE )
    gtk_container_add( GTK_CONTAINER(vbox_main), item );
  else
    gtk_container_add( GTK_CONTAINER(hbox), item );

  gtk_container_add( GTK_CONTAINER(vbox_main), frame );

  /* Text */
  item = gtk_label_new( "" );
  gtk_label_set_line_wrap( GTK_LABEL(item), 1 );
  gtk_box_pack_start( GTK_BOX(hbox), item, TRUE, TRUE, 0);
  app->Error[ERW_LBL_MESSAGE] = item;

  /* Button */
  item = gtk_button_new_with_mnemonic( app->Strs[217] ); /* "_Okay" */
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_hide_error), 0 );
  gtk_container_add( GTK_CONTAINER(vbox_main), item );

  gtk_widget_show_all( vbox_main );
  gtk_widget_hide( win );
}
