/*
 *  'UpdatesWarner' alerts you about system failures on your distant linux hosts
 *  Copyright (C) 2014  Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

void                Show_window(GtkWidget *widget, gpointer data)
{
  gtk_widget_show( GTK_WIDGET(data) );
}

gboolean            Callback_on_delete(GtkWidget *widget, gpointer window)
{
  if( app->Flags & APP_FLG_SYSTRAY )
    gtk_widget_hide( widget );
  else
    Quit_app();
  return( TRUE );
}

void                Callback_on_destroy(GtkWidget *widget, gpointer window)
{
  printf( "ON DESTROY %p\n", widget );
}

static gboolean     Callback_on_motion(GtkWidget *widget, GdkEvent *event, gpointer window)
{
  if( !( app->Gui.Flags & GUI_FLG_LOCK_MAIN )) {
    app->Gui.MainX = event->button.x_root - app->Gui.MoveX;
    app->Gui.MainY = event->button.y_root - app->Gui.MoveY;
    gtk_window_move( GTK_WINDOW(widget), app->Gui.MainX, app->Gui.MainY );
  }
  return( TRUE );
}

static void         *th_Force_window_position(void *param)
{
  GtkWidget         *widget = param;
  gint              a, x, y;

  /*
    Sometimes the window manager moves the window for good reasons
    But not all window managers move the windows back where they belong
    So this ugly procedure exist with pthread_create() and usleep()
    Because... well it works pretty fine without screwing up window managers
    usleep() is used to prevent trying to move thousand of times per second
  */
  usleep( 500000 ); /* should be more than enough */
  LOCK_GTK();
  gtk_window_get_position( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), (gint*) &x, (gint*) &y );
  UNLOCK_GTK();
  for( a = 0; a < 5 && ( x != app->Gui.MainX || y != app->Gui.MainY ); ++a ) {
    app->Gui.Flags |= GUI_FLG_MOVE_FORCED;
    LOCK_GTK();
    gtk_window_move( GTK_WINDOW(widget), app->Gui.MainX, app->Gui.MainY );
    UNLOCK_GTK();
    usleep( 500000 ); /* should be more than enough */
    LOCK_GTK();
    gtk_window_get_position( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), (gint*) &x, (gint*) &y );
    UNLOCK_GTK();
  }
  app->Gui.Flags &= ~( GUI_FLG_FORCE_MOVE );

  return( 0 );
}

static gboolean     Callback_on_configure(GtkWidget *widget, GdkEvent *event, gpointer window)
{
  gint              x, y, curr_w, curr_h;

  if( app->Flags & APP_FLG_PHONE ) {
    app->Gui.MainX = event->configure.x;
    app->Gui.MainY = event->configure.y;
  }
  else {
    if( !( app->Gui.Flags & ( GUI_FLG_MOVING | GUI_FLG_MOVE_FORCED ))) {
      gtk_window_get_position( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), (gint*) &x, (gint*) &y );
      SCREEN_WIDTH_HEIGHT( curr_w, curr_h );
      if(( x != app->Gui.MainX && ( app->Gui.MainX + event->configure.width ) < curr_w )
	 || ( y != app->Gui.MainY && ( app->Gui.MainY + event->configure.height ) < curr_h )) {
	pthread_t tmp_id;
	if( !( app->Gui.Flags & GUI_FLG_FORCE_MOVE )) {
	  app->Gui.Flags |= GUI_FLG_FORCE_MOVE;
	  pthread_create( &tmp_id, &app->Attr, th_Force_window_position, widget );
	}
      }
    }
    else if( app->Gui.Flags & GUI_FLG_MOVE_FORCED )
      app->Gui.Flags &= ~( GUI_FLG_MOVE_FORCED  );
    else {
      app->Gui.MainX = event->configure.x;
      app->Gui.MainY = event->configure.y;
      if( app->Gui.HoveredWatcher )
	Update_info_position( app->Gui.HoveredWatcher );
    }
  }

  return( FALSE );
}

/*
  Purpose: Searching the watcher from the coordinates
*/
static s_wtr        *Search_watcher_from_coords(int x, int y)
{
  s_wtr             *watcher = 0;
  unsigned int      a, pos, count;

  if(( app->Gui.Flags & GUI_FLG_VERTICAL_EXPAND ) && !( app->Flags & APP_FLG_PHONE ))
    pos = y / ( app->Gui.WatcherSizeY + 2 ) + ( x / ( app->Gui.WatcherSizeX + 2 )) * app->Gui.MaxYWatcher;
  else
    pos = x / ( app->Gui.WatcherSizeX + 2 ) + ( y / ( app->Gui.WatcherSizeY + 2 )) * app->Gui.MaxXWatcher;

  count = 0;
  for( a = 0; a < app->Gui.WatcherCount; ++a ) {
    if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE )) {
      if( count == pos) {
	watcher = app->Gui.WatcherTable[a];
	app->Gui.HoveredWatcher = watcher;
	app->Gui.SelectedWatcher = watcher;
	break;
      }
      ++count;
    }
  }
  return( watcher );
}

/*
  Purpose: Showing the right-lick menu at the click position
*/
static void         Show_main_menu(GdkEvent *event)
{
  s_wtr             *watcher = 0;

  if( app->Gui.HoveredWatcher )
    watcher = app->Gui.HoveredWatcher;
  else
    watcher = Search_watcher_from_coords( event->button.x, event->button.y );

  if( watcher ) {
    if( watcher->Flags & WTR_FLG_LOCAL_AUTO ) {
      gtk_menu_item_set_label( GTK_MENU_ITEM(app->Gui.Main[MW_ROW_LOCAL_UPDATE]), app->Strs[136] ); /* "Update local version" */
      gtk_widget_set_tooltip_text( app->Gui.Main[MW_ROW_LAUNCH], watcher->LocalExec );
      gtk_widget_show( app->Gui.Main[MW_ROW_LOCAL_UPDATE] );
      gtk_widget_show( app->Gui.Main[MW_ROW_LAUNCH] );
      gtk_widget_show( app->Gui.Main[MW_ROW_GOTO_WEB] );
    }
    else {
      if( watcher->Flags & WTR_FLG_VERSION_MISMATCH ) {
	gtk_menu_item_set_label( GTK_MENU_ITEM(app->Gui.Main[MW_ROW_LOCAL_UPDATE]), app->Strs[98] ); /* "Update manual version" */
	gtk_widget_show( app->Gui.Main[MW_ROW_LOCAL_UPDATE] );
      }
      else
	gtk_widget_hide( app->Gui.Main[MW_ROW_LOCAL_UPDATE] );
      gtk_widget_hide( app->Gui.Main[MW_ROW_LAUNCH] );
    }

    gtk_menu_item_set_label( GTK_MENU_ITEM(app->Gui.Main[MW_ROW_NAME]), watcher->Name );
#ifndef LIB_HANDY
    GtkWidget *img;
    img = gtk_image_new_from_pixbuf( watcher->CurrentPixbuf );
    gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(app->Gui.Main[MW_ROW_NAME]), img );
#endif
    gtk_widget_show( app->Gui.Main[MW_ROW_NAME] );
    gtk_widget_show( app->Gui.Main[MW_ROW_OFFI_UPDATE] );
    gtk_widget_hide( app->Gui.Main[MW_ROW_ALL_OFFI_UPDATE] );

    gtk_widget_set_tooltip_text( app->Gui.Main[MW_ROW_GOTO_WEB], watcher->OfficialURL );
    gtk_widget_show( app->Gui.Main[MW_ROW_GOTO_WEB] );

    gtk_widget_show( app->Gui.Main[MW_ROW_EDIT] );
  }
  else {
    gtk_widget_hide( app->Gui.Main[MW_ROW_NAME] );
    gtk_widget_hide( app->Gui.Main[MW_ROW_LOCAL_UPDATE] );
    gtk_widget_hide( app->Gui.Main[MW_ROW_OFFI_UPDATE] );
    gtk_widget_hide( app->Gui.Main[MW_ROW_LAUNCH] );
    gtk_widget_hide( app->Gui.Main[MW_ROW_GOTO_WEB] );
    gtk_widget_hide( app->Gui.Main[MW_ROW_EDIT] );
    gtk_widget_show( app->Gui.Main[MW_ROW_ALL_OFFI_UPDATE] );
  }
#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 22
  gtk_menu_popup_at_pointer( GTK_MENU(app->Gui.Main[MW_MENU_WATCHER]), event );
#else
  gtk_menu_popup( GTK_MENU(app->Gui.Main[MW_MENU_WATCHER]), NULL, NULL, NULL, NULL, (event != NULL) ? event->button.button : 0, gdk_event_get_time( (GdkEvent*) event ) );
#endif
}

static gboolean     Callback_on_press(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  s_wtr             *watcher = 0;

  if( event->button.button == 1 ) {
    gettimeofday( &app->Gui.ClickTimestamp, NULL );

    if( app->Gui.HoveredWatcher )
      gtk_tree_selection_select_iter( app->Gui.SelectionWatcher, &app->Gui.HoveredWatcher->Iter );
    else
      gtk_tree_selection_unselect_all( app->Gui.SelectionWatcher );

    if( app->Flags & APP_FLG_PHONE ) {
      if( app->Gui.HoveredWatcher )
	watcher = app->Gui.HoveredWatcher;
      else
	watcher = Search_watcher_from_coords( event->button.x, event->button.y );

      if( watcher ) {
	Update_info_position( watcher );
	gtk_widget_show( watcher->InfosWindow );
      }
    }
    else {
      if( !( app->Gui.Flags & GUI_FLG_LOCK_MAIN )) {
	app->Gui.MoveX = event->button.x;
	app->Gui.MoveY = event->button.y;
	app->Gui.Flags |= GUI_FLG_MOVING;
      }
    }
  }
  else if( event->button.button == 3 ) {
    app->Gui.SelectedWatcher = app->Gui.HoveredWatcher;
    if( app->Gui.HoveredWatcher )
      gtk_tree_selection_select_iter( app->Gui.SelectionWatcher, &app->Gui.HoveredWatcher->Iter );
    else
      gtk_tree_selection_unselect_all( app->Gui.SelectionWatcher );
    Show_main_menu( event );
  }
  return( TRUE );
}

static gboolean     Callback_on_release(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  s_wtr             *watcher = 0;
  struct timeval    now;
  unsigned int      ms;

  if( event->button.button == 1 ) {
    gettimeofday( &now, NULL );
    if( now.tv_usec > app->Gui.ClickTimestamp.tv_usec )
      ms = ( now.tv_usec - app->Gui.ClickTimestamp.tv_usec ) / 1000 + ( now.tv_sec - app->Gui.ClickTimestamp.tv_sec ) * 1000;
    else
      ms = ( 1000000 - app->Gui.ClickTimestamp.tv_usec + now.tv_usec ) / 1000 + ( now.tv_sec - app->Gui.ClickTimestamp.tv_sec - 1 ) * 1000;

    if( app->Gui.HoveredWatcher )
      watcher = app->Gui.HoveredWatcher;
    else
      watcher = Search_watcher_from_coords( event->button.x, event->button.y );

    app->Gui.SelectedWatcher = watcher;
    if( watcher )
      gtk_tree_selection_select_iter( app->Gui.SelectionWatcher, &watcher->Iter );
    else
      gtk_tree_selection_unselect_all( app->Gui.SelectionWatcher );

    if( app->Flags & APP_FLG_PHONE ) {
      if( ms > 300 )
	Show_main_menu( event );
    }
    else {
      app->Gui.Flags &= ~( GUI_FLG_MOVING );
      if( app->Gui.MainX != app->Gui.SaveX || app->Gui.MainY != app->Gui.SaveY ) {
	app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_PREFERENCES;
	gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
      }
    }
  }
  return( TRUE );
}


/*
  Purpose: Drawing something visible in the window when there is no watcher
*/
static void         Draw_empty_background(cairo_t *cr)
{
  cairo_set_line_width( cr, 3 );
  if( app->Flags & APP_FLG_NO_ALPHA )
    cairo_set_source_rgb( cr, 1.0, 0.0, 0.0 );
  else
    cairo_set_source_rgba( cr, 1.0, 0.0, 0.0, 1.0 );
  cairo_rectangle( cr, 0, 0, app->Gui.WatcherSizeX, app->Gui.WatcherSizeY );
  cairo_stroke( cr );

  if( app->Flags & APP_FLG_NO_ALPHA )
    cairo_set_source_rgb( cr, 0.0, 1.0, 0.0 );
  else
    cairo_set_source_rgba( cr, 0.0, 1.0, 0.0, 1 );
  cairo_rectangle( cr, 3, 3, app->Gui.WatcherSizeX - 6, app->Gui.WatcherSizeY - 6 );
  cairo_stroke( cr );

  if( app->Flags & APP_FLG_NO_ALPHA )
    cairo_set_source_rgb( cr, 0.0, 0.0, 1.0 );
  else
    cairo_set_source_rgba( cr, 0.0, 0.0, 1.0, 1 );
  cairo_rectangle( cr, 6, 6, app->Gui.WatcherSizeX - 12, app->Gui.WatcherSizeY - 12 );
  cairo_stroke( cr );
}

#if GTK_MAJOR_VERSION < 3
gboolean            Callback_draw_main(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  cairo_t           *cr;

  if( app->Flags & APP_FLG_NO_COMPOSITE ) {
    if( gdk_screen_is_composited( gtk_widget_get_screen( widget )))
      app->Flags &= ~( APP_FLG_NO_COMPOSITE );
    else
      app->Flags |= APP_FLG_NO_COMPOSITE;
    return( TRUE );
  }

  cr = gdk_cairo_create( widget->window );
  if( app->Flags & APP_FLG_NO_ALPHA )
    cairo_set_source_rgb( cr, app->Gui.MainColor.red / 65536.0, app->Gui.MainColor.green / 65536.0, app->Gui.MainColor.blue / 65536.0 );
  else
    cairo_set_source_rgba( cr, app->Gui.MainColor.red / 65536.0, app->Gui.MainColor.green / 65536.0, app->Gui.MainColor.blue / 65536.0, app->Gui.MainOpacity );

  cairo_set_operator( cr, CAIRO_OPERATOR_SOURCE );
  cairo_paint( cr );

  if( app->Gui.Flags & GUI_FLG_NO_WATCHER )
    Draw_empty_background( cr );

  cairo_destroy( cr );
  return( TRUE );
}
#else
gboolean            Callback_draw_main(GtkWidget *widget, cairo_t *cr, gpointer data)
{
  GdkWindow         *win;
  _Bool             destroy = ( cr == 0 );

  if( destroy ) {
    win = gtk_widget_get_window( widget );
    if( ! win ) {
      SNO( 1, "Main window: gtk_widget_get_window() failed" );
      return( FALSE );
    }

    cr = gdk_cairo_create( win );
  }
  else
    cairo_save (cr);

  if( app->Flags & APP_FLG_NO_ALPHA )
    cairo_set_source_rgb( cr, app->Gui.MainColor.red, app->Gui.MainColor.green, app->Gui.MainColor.blue );
  else
    cairo_set_source_rgba( cr, app->Gui.MainColor.red, app->Gui.MainColor.green, app->Gui.MainColor.blue, app->Gui.MainColor.alpha );

  /* draw the background */
  cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
  cairo_paint (cr);

  if( app->Gui.Flags & GUI_FLG_NO_WATCHER )
    Draw_empty_background( cr );

  if( destroy )
    cairo_destroy( cr );
  else
    cairo_restore (cr);

  return FALSE;
}
#endif

void                Set_window_properties(GtkWidget *win, GtkWidget *main_box)
{
  GtkWidget         *item;
  gint              width, height;

  if( app->Flags & APP_FLG_PHONE ) {
    SCREEN_WIDTH_HEIGHT( width, height );
    gtk_window_set_default_size( GTK_WINDOW(win), width, height );
    gtk_window_set_decorated( GTK_WINDOW(win), 0 );
    item = gtk_scrolled_window_new( NULL, NULL );
    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(item), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
    gtk_container_add( GTK_CONTAINER(win), item );
    gtk_widget_show_all( item );
    if( main_box )
      gtk_scrolled_window_add_with_viewport( GTK_SCROLLED_WINDOW(item), main_box );
  }
  else {
    gtk_window_set_resizable( GTK_WINDOW(win), FALSE );
    if( main_box )
      gtk_container_add( GTK_CONTAINER(win), main_box );
  }
}

void                Callback_size_changed(GdkScreen *screen, gpointer   user_data)
{
  int               width, height, curr_w, curr_h;

  SCREEN_WIDTH_HEIGHT( width, height );
  curr_w = width / app->Gui.WatcherSizeX;
  if( ! curr_w ) curr_w = 1;
  curr_h = height / app->Gui.WatcherSizeY;
  if( ! curr_h ) curr_h = 1;
  if( app->Gui.MaxXWatcher != curr_w || app->Gui.MaxYWatcher != curr_h ) {
    app->Gui.MaxXWatcher = curr_w;
    app->Gui.MaxYWatcher = curr_h;
    Update_main_geometry();
  }
}

static void         Init_specific_gtk(GtkWidget *win)
{
  GdkScreen         *screen = gtk_widget_get_screen( win );

#if GTK_MAJOR_VERSION == 3
  GdkVisual *visual = gdk_screen_get_rgba_visual( screen );
  gtk_widget_set_visual( win , visual );

  g_signal_connect( G_OBJECT(win), "draw", G_CALLBACK(Callback_draw_main), 0 );
  g_signal_connect( G_OBJECT(win), "screen-changed", G_CALLBACK(Callback_draw_main), 0 );
#else
  GdkColormap       *colormap;
  if( !( colormap = gdk_screen_get_rgba_colormap( screen ))) {
    ERR_APP( "%s", app->Strs[149] ); /* "Unable to use transparency channel" */
    colormap = gdk_screen_get_rgb_colormap( screen );
    app->Flags |= APP_FLG_NO_ALPHA;
  }
  else
    app->Flags &= ~( APP_FLG_NO_ALPHA );

  gtk_widget_set_colormap( win, colormap );
  g_signal_connect( G_OBJECT(win), "expose-event", G_CALLBACK(Callback_draw_main), 0 );
  g_signal_connect( G_OBJECT(win), "composited-changed", G_CALLBACK(Callback_draw_main), 0 );
#endif

  if( gdk_screen_is_composited( screen ))
    app->Flags &= ~( APP_FLG_NO_COMPOSITE );
  else {
    ERR_APP( "%s", app->Strs[164] ); /* "Screen composite not available (= no transparency)" */
    app->Flags |= APP_FLG_NO_COMPOSITE;
  }

  if( app->Flags & APP_FLG_PHONE )
    g_signal_connect( G_OBJECT(screen), "size-changed", G_CALLBACK(Callback_size_changed), 0 );
}

void                Create_window_main()
{
  GtkWidget         *win, *table;
  GError            *err;
  gint              width, height;

  /* Main window ------------------------------------------------------ */
#if GTK_MAJOR_VERSION == 3
#  ifdef LIB_HANDY
  win = hdy_window_new();
  gtk_application_add_window( GTK_APPLICATION( app->Gui.App ), GTK_WINDOW(win) );
#  else
  win = gtk_application_window_new( GTK_APPLICATION( app->Gui.App ));
#  endif
#else
  win = gtk_window_new( GTK_WINDOW_TOPLEVEL );
#endif

  app->Gui.Main[MW_WIN_MAIN] = win;

  gtk_window_set_title( GTK_WINDOW(win), APP_NAME );
  gtk_window_set_decorated( GTK_WINDOW(win), 0 );
  gtk_widget_set_app_paintable( win, TRUE );

  gtk_window_set_icon_from_file( GTK_WINDOW(win), DEFAULT_APP_RESOURCES"/updates-warner.png", &err );

  if( app->Flags & APP_FLG_PHONE ) {
    SCREEN_WIDTH_HEIGHT( width, height );
    gtk_window_move( GTK_WINDOW(win), 0, 0 );
    gtk_window_set_default_size( GTK_WINDOW(win), width, height );
    app->Gui.MaxXWatcher = width / app->Gui.WatcherSizeX;
    app->Gui.MaxYWatcher = height / app->Gui.WatcherSizeY;
    app->Gui.Flags |= GUI_FLG_LOCK_MAIN;
  }
  else {
    gtk_window_set_default_size( GTK_WINDOW(win), app->Gui.MaxXWatcher * ( app->Gui.WatcherSizeX + 2 ) - 2, app->Gui.MaxYWatcher * ( app->Gui.WatcherSizeY + 2 ) - 2 );
    gtk_window_set_skip_taskbar_hint( GTK_WINDOW(win), 1 );
    if( app->Gui.Flags & GUI_FLG_ON_TOP )
      gtk_window_set_keep_above( GTK_WINDOW(win), 1 );
    else
      gtk_window_set_keep_below( GTK_WINDOW(win), 1 );
    gtk_window_set_resizable( GTK_WINDOW(win), FALSE );
  }

  g_signal_connect( G_OBJECT(win), "delete_event", G_CALLBACK(Callback_on_delete), 0 );
  g_signal_connect( G_OBJECT(win), "destroy", G_CALLBACK(Callback_on_destroy), 0 );
  gtk_widget_add_events( win, GDK_BUTTON1_MOTION_MASK );
  g_signal_connect( G_OBJECT(win), "motion_notify_event", G_CALLBACK(Callback_on_motion), 0 );
  gtk_widget_add_events( win, GDK_BUTTON_PRESS_MASK );
  g_signal_connect( G_OBJECT(win), "button_press_event", G_CALLBACK(Callback_on_press), 0 );
  gtk_widget_add_events( win, GDK_BUTTON_RELEASE_MASK );
  g_signal_connect( G_OBJECT(win), "button_release_event", G_CALLBACK(Callback_on_release), 0 );
  g_signal_connect( G_OBJECT(win), "configure_event", G_CALLBACK(Callback_on_configure), 0 );

  Init_specific_gtk( win );

  /* Watchers table --------------------------------------------------- */
#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 4
  table = gtk_grid_new();
  gtk_grid_set_row_spacing( GTK_GRID(table), 2 );
  gtk_grid_set_column_spacing( GTK_GRID(table), 2 );
#else
  table = gtk_table_new( app->Gui.MaxYWatcher, app->Gui.MaxXWatcher, FALSE );
  gtk_table_set_col_spacings( GTK_TABLE(table), 2 );
  gtk_table_set_row_spacings( GTK_TABLE(table), 2 );
#endif
  gtk_container_add( GTK_CONTAINER(win), table );
  app->Gui.Main[MW_TABLE_MAIN] = table;
}
