/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

void                     Copy_gui_after_line(s_op *operation)
{
  const char             *entry;

  if( ! gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_AFTER_WORD_PATTERN]) )
      || !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_AFTER_WORD_PATTERN]) )))
    return;

  STRDUP( operation->Pattern, entry );
}

unsigned char            Get_gui_after_line(s_op *operation, GtkWidget *lbl_error)
{
  const char             *entry;
  unsigned char          ret = 0;

  if( ! gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_AFTER_LINE_PATTERN]) )) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[211] ); /* "Missing value in the 'After line' block" */
    return( 1 );
  }
  if( !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_AFTER_LINE_PATTERN]) ))) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[158] ); /* "A problem occured with a GTK widget" */
    return( 1 );
  }
  if( ! operation->Pattern || strcmp( entry, operation->Pattern )) {
    ret = 2;
    STRDUP( entry, entry );
    free( (char*) operation->Pattern );
    operation->Pattern = entry;
  }
  return( ret );
}

static void              Callback_line_changed(GtkSpinButton *spinbutton, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return;
  }
  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
}

static gboolean          Callback_pattern_changed(GtkWidget *widget, GdkEvent *event, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return( FALSE );
  }
  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
  return( FALSE );
}

/*
  Purpose: Creating and initialise an 'After line' search operation structure
  - pattern : must be 0 or malloc'ed
*/
s_op                     *New_operation_after_line(const char *pattern)
{
  GtkWidget              *hbox, *item;
  s_op                   *operation;

  CALLOC( operation, 1, sizeof(*operation) );
  operation->Type = OP_TYPE_AFTER_LINE;
  operation->Pattern = pattern;

  MALLOC( operation->GtkTable, OP_WIDGET_AFTER_LINE_COUNT * sizeof(*operation->GtkTable) );

  hbox = APP_GTK_HBOX( FALSE, 0 );
  g_object_ref( G_OBJECT(hbox) );
  operation->GtkTable[OP_WIDGET_AFTER_LINE_HBOX] = hbox;

  Create_del_button( hbox );

  item = gtk_label_new( app->Strs[209] ); /* " After line containing " */
  gtk_widget_set_tooltip_text( item, app->Strs[210] ); /* "official_after_line: Version will be search only after the line containing this pattern" */
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  item = gtk_entry_new();
  gtk_widget_set_tooltip_text( item, app->Strs[210] ); /* "official_after_line: Version will be search only after the line containing this pattern" */
  gtk_entry_set_max_length( GTK_ENTRY(item), 100 );
  if( pattern )
    gtk_entry_set_text( GTK_ENTRY(item), pattern );
  g_signal_connect( G_OBJECT(item), "focus-out-event", G_CALLBACK(Callback_pattern_changed), operation );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_line_changed), operation );
  gtk_box_pack_start( GTK_BOX(hbox), item, TRUE, TRUE, 0 );
  operation->GtkTable[OP_WIDGET_AFTER_LINE_PATTERN] = item;

  return( operation );
}
