/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

void                     Copy_gui_after_word(s_op *operation)
{
  const char             *entry;

  operation->Number = (unsigned int) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_AFTER_WORD_NB]) );
  if( ! gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_AFTER_WORD_PATTERN]) )
      || !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_AFTER_WORD_PATTERN]) )))
    return;

  STRDUP( operation->Pattern, entry );
}

unsigned char            Get_gui_after_word(s_op *operation, GtkWidget *lbl_error)
{
  const char             *entry;
  unsigned short         nb;
  unsigned char          ret = 0;

  nb = (unsigned int) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_AFTER_WORD_NB]) );

  if( ! gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_AFTER_WORD_PATTERN]) )) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[128] ); /* "Missing mandatory element" */
    return( 1 );
  }
  if( !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_AFTER_WORD_PATTERN]) ))) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[158] ); /* "A problem occured with a GTK widget" */
    return( 1 );
  }

  if( ! operation->Pattern || strcmp( entry, operation->Pattern )) {
    ret = 2;
    STRDUP( entry, entry );
    free( (char*) operation->Pattern );
    operation->Pattern = entry;
  }

  if( operation->Number != nb ) {
    ret = 2;
    operation->Number = nb;
  }
  return( ret );
}

static void              Callback_word_changed(GtkSpinButton *spinbutton, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return;
  }

  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
}

static gboolean          Callback_pattern_changed(GtkWidget *widget, GdkEvent *event, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return( FALSE );
  }

  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
  return( FALSE );
}

/*
  Purpose: Creating and initialise an 'After word' search operation structure
  - pattern : must be 0 or malloc'ed
*/
s_op                     *New_operation_after_word(unsigned int nb_word, const char *pattern)
{
  GtkWidget              *hbox_main, *item;
  s_op                   *operation;

  CALLOC( operation, 1, sizeof(s_op) );
  operation->Type = OP_TYPE_AFTER_WORD;
  operation->Number = nb_word;
  operation->Pattern = pattern;

  MALLOC( operation->GtkTable, OP_WIDGET_AFTER_WORD_COUNT * sizeof(*operation->GtkTable) );

  hbox_main = APP_GTK_HBOX( FALSE, 0 );
  g_object_ref( G_OBJECT(hbox_main) );
  operation->GtkTable[OP_WIDGET_AFTER_WORD_HBOX] = hbox_main;

  Create_del_button( hbox_main );

  item = gtk_label_new( app->Strs[239] ); /* " After word " */
  gtk_widget_set_tooltip_text( item, app->Strs[238] ); /* "official_after_word: Version will be search only after this pattern" */
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  item = gtk_entry_new();
  gtk_widget_set_tooltip_text( item, app->Strs[238] ); /* "official_after_word: Version will be search only after this pattern" */
  gtk_entry_set_max_length( GTK_ENTRY(item), 100 );
  if( pattern )
    gtk_entry_set_text( GTK_ENTRY(item), pattern );
  g_signal_connect( G_OBJECT(item), "focus-out-event", G_CALLBACK(Callback_pattern_changed), operation );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_word_changed), operation );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, TRUE, TRUE, 0 );
  operation->GtkTable[OP_WIDGET_AFTER_WORD_PATTERN] = item;

  item = gtk_label_new( app->Strs[240] ); /* " number " */
  gtk_widget_set_tooltip_text( item, app->Strs[238] ); /* "official_after_word: Version will be search only after this pattern" */
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  item = gtk_spin_button_new_with_range( 1, 65535, 1 );
  gtk_widget_set_tooltip_text( item, "1 -> 65535" );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(item), nb_word );
  g_signal_connect( G_OBJECT(item), "value-changed", G_CALLBACK(Callback_word_changed), operation );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  operation->GtkTable[OP_WIDGET_AFTER_WORD_NB] = item;

  return( operation );
}