/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

void                     Copy_gui_in_field(s_op *operation)
{
  const char             *entry;

  operation->Number = (unsigned int) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_FIELD_NB]) );
  if( ! gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_FIELD_CHAR]) )
      || !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_FIELD_CHAR]) )))
    return;
  operation->Delimiter[0] = entry[0];
}

unsigned char            Get_gui_in_field(s_op *operation, GtkWidget *lbl_error)
{
  const char             *entry;
  unsigned short         nb;
  unsigned char          ret = 0;

  nb = (unsigned int) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_FIELD_NB]) );

  if( gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_FIELD_CHAR])) < 1 ) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[189] ); /* "Missing delimiter character" */
    return( 1 );
  }
  if( !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_FIELD_CHAR]) ))) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[158] ); /* "A problem occured with a GTK widget" */
    return( 1 );
  }
  if( entry[0] < ' ' || entry[0] > '~' ) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[121] ); /* "A delimiter specified is not permitted" */
    return( 1 );
  }

  if( operation->Delimiter[0] != entry[0] ) {
    ret = 2;
    operation->Delimiter[0] = entry[0];
  }
  if( operation->Number != nb ) {
    ret = 2;
    operation->Number = nb;
  }
  return( ret );
}

static void              Callback_field_changed(GtkSpinButton *spinbutton, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return;
  }
  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
}

static gboolean          Callback_delimiter_changed(GtkWidget *widget, GdkEvent *event, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return( FALSE );
  }
  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
  return( FALSE );
}

/*
  Purpose: Creating a hbox containing the search block 'In field'
*/
s_op                     *New_operation_in_field(unsigned int nb_field, char split)
{
  GtkWidget              *hbox_main, *item;
  s_op                   *operation;
  char                   delim[2];

  ASSERT( !nb_field, "nb_field = 0" );

  CALLOC( operation, 1, sizeof(*operation) );
  operation->Type = OP_TYPE_IN_FIELD;
  operation->Number = nb_field;
  operation->Delimiter[0] = split;

  MALLOC( operation->GtkTable, OP_WIDGET_IN_FIELD_COUNT * sizeof(*operation->GtkTable) );

  hbox_main = APP_GTK_HBOX( FALSE, 0 );
  g_object_ref( G_OBJECT(hbox_main) );
  operation->GtkTable[OP_WIDGET_IN_FIELD_HBOX] = hbox_main;

  Create_del_button( hbox_main );

  item = gtk_label_new( app->Strs[131] ); /* "In field number" */
  gtk_widget_set_tooltip_text( item, app->Strs[241] ); /* "official_in_field: The current line is split in fields with a chosen character" */
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  item = gtk_spin_button_new_with_range( 1, 65535, 1 );
  gtk_widget_set_tooltip_text( item, "1 -> 65535" );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(item), nb_field );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  g_signal_connect( G_OBJECT(item), "value-changed", G_CALLBACK(Callback_field_changed), operation );
  operation->GtkTable[OP_WIDGET_IN_FIELD_NB] = item;

  item = gtk_label_new( app->Strs[113] ); /* "cut with " */
  gtk_widget_set_tooltip_text( item, app->Strs[241] ); /* "official_in_field: The current line is split in fields with a chosen character" */
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  item = gtk_entry_new();
  gtk_entry_set_max_length( GTK_ENTRY(item), 1 );
  gtk_entry_set_width_chars( GTK_ENTRY(item), 2 );
  gtk_widget_set_tooltip_text( item, app->Strs[242] ); /* "official_in_field: Set the character used to split the line" */
  if( !( split < ' ' || split > '~' )) {
    delim[0] = split;
    delim[1] = '\0';
    gtk_entry_set_text( GTK_ENTRY(item), delim );
  }
  g_signal_connect( G_OBJECT(item), "focus-out-event", G_CALLBACK(Callback_delimiter_changed), operation );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_field_changed), operation );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  operation->GtkTable[OP_WIDGET_IN_FIELD_CHAR] = item;

  return( operation );
}
