/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

void                     Copy_gui_in_line(s_op *operation)
{
  const char             *entry;

  operation->Number = (unsigned int) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_LINE_NB]) );
  operation->Activated = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(operation->GtkTable[OP_WIDGET_IN_LINE_CHECK]) );
  if( ! gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_LINE_PATTERN]) )
      || !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_LINE_PATTERN]) )))
    return;

  STRDUP( operation->Pattern, entry );
}

unsigned char            Get_gui_in_line(s_op *operation, GtkWidget *lbl_error)
{
  const char             *entry;
  unsigned short         nb;
  unsigned char          ret = 0;
  _Bool                  active;

  nb = (unsigned int) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_LINE_NB]) );
  active = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(operation->GtkTable[OP_WIDGET_IN_LINE_CHECK]) );

  if( active ) {
    if( gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_LINE_PATTERN])) < 1 ) {
      gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[188] ); /* "Missing value of the 'In line' field" */
      return( 1 );
    }
    if( !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_LINE_PATTERN]) ))) {
      gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[158] ); /* "A problem occured with a GTK widget" */
      return( 1 );
    }
    if( ! operation->Pattern || strcmp( entry, operation->Pattern )) {
      ret = 2;
      STRDUP( entry, entry );
      free( (char*) operation->Pattern );
      operation->Pattern = entry;
    }
  }

  if( operation->Number != nb ) {
    ret = 2;
    operation->Number = nb;
  }
  if( operation->Activated != active ) {
    ret = 2;
    operation->Activated = active;
  }
  return( ret );
}

static void              Callback_line_changed(GtkSpinButton *spinbutton, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return;
  }

  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
}

static void              Callback_check_changed(GtkWidget *button, gpointer param)
{
  s_op                   *operation = param;

  gtk_widget_set_sensitive( operation->GtkTable[OP_WIDGET_IN_LINE_PATTERN], gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(operation->GtkTable[OP_WIDGET_IN_LINE_CHECK]) ));

  if( operation->Template ) {
    Update_template_result();
    return;
  }

  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
}

static gboolean          Callback_pattern_changed(GtkWidget *widget, GdkEvent *event, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return( FALSE );
  }

  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
  return( FALSE );
}

/*
  Purpose: Creating and initialise an 'In line' search operation structure
  - pattern : must be 0 or malloc'ed
*/
s_op                     *New_operation_in_line(unsigned int nb_line, _Bool containing, const char *pattern)
{
  GtkWidget              *hbox_main, *item;
  s_op                   *operation;

  CALLOC( operation, 1, sizeof(*operation) );
  operation->Type = OP_TYPE_IN_LINE;
  operation->Number = nb_line;
  operation->Activated = containing;
  operation->Pattern = pattern;

  MALLOC( operation->GtkTable, OP_WIDGET_IN_LINE_COUNT * sizeof(*operation->GtkTable) );

  hbox_main = APP_GTK_HBOX( FALSE, 0 );
  g_object_ref( G_OBJECT(hbox_main) );
  operation->GtkTable[OP_WIDGET_IN_LINE_HBOX] = hbox_main;

  Create_del_button( hbox_main );

  item = gtk_label_new( app->Strs[108] ); /* " In line number " */
  gtk_widget_set_tooltip_text( item, app->Strs[129] ); /* "official_in_line: Line count where to retrieve the official version from the web resource" */
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  item = gtk_spin_button_new_with_range( 1, 65535, 1 );
  gtk_widget_set_tooltip_text( item, "1 -> 65535" );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(item), nb_line );
  g_signal_connect( G_OBJECT(item), "value-changed", G_CALLBACK(Callback_line_changed), operation );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  operation->GtkTable[OP_WIDGET_IN_LINE_NB] = item;

  /* - Containing the word - */
  item = gtk_check_button_new_with_label( app->Strs[54] ); /* " containing the word " */
  gtk_widget_set_tooltip_text( item, app->Strs[130] ); /* "official_in_line: Only the lines containing this word will count" */
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(item), containing );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  operation->GtkTable[OP_WIDGET_IN_LINE_CHECK] = item;

  item = gtk_entry_new();
  gtk_widget_set_tooltip_text( item, app->Strs[130] ); /* "official_in_line: Only the lines containing this word will count" */
  gtk_entry_set_max_length( GTK_ENTRY(item), MAX_LENGTH_PATTERN );
  if( pattern )
    gtk_entry_set_text( GTK_ENTRY(item), pattern );
  gtk_widget_set_sensitive( item, containing );
  g_signal_connect( G_OBJECT(item), "focus-out-event", G_CALLBACK(Callback_pattern_changed), operation );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_line_changed), operation );
  g_signal_connect( G_OBJECT(operation->GtkTable[OP_WIDGET_IN_LINE_CHECK]), "clicked", G_CALLBACK(Callback_check_changed), operation );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  operation->GtkTable[OP_WIDGET_IN_LINE_PATTERN] = item;

  return( operation );
}
