/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

void                     Copy_gui_in_section(s_op *operation)
{
  const char             *entry;

  operation->Number = (unsigned int) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_SECTION_NB]) );
  if( ! gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_SECTION_CHARS]) )
      || !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_SECTION_CHARS]) )))
    return;
  operation->Delimiter[0] = entry[0];
  operation->Delimiter[1] = entry[1];
}

unsigned char            Get_gui_in_section(s_op *operation, GtkWidget *lbl_error)
{
  const char             *entry;
  unsigned short         nb;
  unsigned char          ret = 0;

  nb = (unsigned short) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_SECTION_NB]) );

  if( gtk_entry_get_text_length( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_SECTION_CHARS])) < 2 ) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[189] ); /* "Missing delimiter character" */
    return( 1 );
  }
  if( !( entry = gtk_entry_get_text( GTK_ENTRY(operation->GtkTable[OP_WIDGET_IN_SECTION_CHARS]) ))) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[158] ); /* "A problem occured with a GTK widget" */
    return( 1 );
  }
  if( entry[0] < ' ' || entry[0] > '~'
      || entry[1] < ' ' || entry[1] > '~' ) {
    gtk_label_set_label( GTK_LABEL(lbl_error), app->Strs[121] ); /* "A delimiter specified is not permitted" */
    return( 1 );
  }

  if( operation->Number != nb ) {
    ret = 2;
    operation->Number = nb;
  }
  if( operation->Delimiter[0] != entry[0] || operation->Delimiter[1] != entry[1] ) {
    ret = 2;
    operation->Delimiter[0] = entry[0];
    operation->Delimiter[1] = entry[1];
  }
  return( ret );
}


static void              Callback_section_changed(GtkSpinButton *spinbutton, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return;
  }

  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
}

static gboolean          Callback_delimiter_changed(GtkWidget *widget, GdkEvent *event, gpointer param)
{
  s_op                   *operation = param;

  if( operation->Template ) {
    Update_template_result();
    return( FALSE );
  }

  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
  return( FALSE );
}

/*
  Purpose: Creating a hbox containing the search block 'In section'
*/
s_op                     *New_operation_in_section(unsigned int nb_section, char open_section, char close_section)
{
  GtkWidget              *hbox_main, *item;
  s_op                   *operation;
  char                   delim[3];

  ASSERT( !open_section || !close_section, "open_section = %d, close_section = %d", open_section, close_section );

  CALLOC( operation, 1, sizeof(s_op) );
  MALLOC( operation->GtkTable, OP_WIDGET_IN_SECTION_COUNT * sizeof(*operation->GtkTable) );
  operation->Type = OP_TYPE_IN_SECTION;
  operation->Number = nb_section;
  operation->Delimiter[0] = open_section;
  operation->Delimiter[1] = close_section;

  hbox_main = APP_GTK_HBOX( FALSE, 0 );
  g_object_ref( G_OBJECT(hbox_main) );
  operation->GtkTable[OP_WIDGET_IN_SECTION_HBOX] = hbox_main;

  Create_del_button( hbox_main );

  item = gtk_label_new( app->Strs[174] ); /* " In section number " */
  gtk_widget_set_tooltip_text( item, app->Strs[58] ); /* "official_in_section: The version can be retrieved inside the nth section" */
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  item = gtk_spin_button_new_with_range( 1, 65535, 1 );
  gtk_widget_set_tooltip_text( item, "1 -> 65535" );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(item), nb_section );
  g_signal_connect( G_OBJECT(item), "value-changed", G_CALLBACK(Callback_section_changed), operation );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  operation->GtkTable[OP_WIDGET_IN_SECTION_NB] = item;

  item = gtk_label_new( app->Strs[171] ); /* " delimited with " */
  gtk_widget_set_tooltip_text( item, app->Strs[163] ); /* "official_in_section: Set the delimiters opening and closing the sections" */
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  item = gtk_entry_new();
  gtk_entry_set_max_length( GTK_ENTRY(item), 2 );
  gtk_entry_set_width_chars( GTK_ENTRY(item), 3 );
  gtk_widget_set_tooltip_text( item, app->Strs[163] ); /* "official_in_section: Set the delimiters opening and closing the sections" */
  delim[0] = open_section;
  delim[1] = close_section;
  delim[2] = '\0';
  gtk_entry_set_text( GTK_ENTRY(item), delim );
  g_signal_connect( G_OBJECT(item), "focus-out-event", G_CALLBACK(Callback_delimiter_changed), operation );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_section_changed), operation );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  operation->GtkTable[OP_WIDGET_IN_SECTION_CHARS] = item;

  return( operation );
}
