/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

void                      Copy_gui_in_word(s_op *operation)
{
  operation->Number = (unsigned int) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_WORD_NB]) );
  operation->Activated = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(operation->GtkTable[OP_WIDGET_IN_WORD_CHECK]) );
}

unsigned char             Get_gui_in_word(s_op *operation, GtkWidget *lbl_error)
{
  unsigned short          nb;
  unsigned char           ret = 0;
  _Bool                   active;

  nb = (unsigned int) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(operation->GtkTable[OP_WIDGET_IN_WORD_NB]) );
  active = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(operation->GtkTable[OP_WIDGET_IN_WORD_CHECK]) );

  if( operation->Activated != active ) {
    ret = 2;
    operation->Activated = active;
  }
  if( operation->Number != nb ) {
    ret = 2;
    operation->Number = nb;
  }
  return( ret );
}

static void              Callback_word_changed(GtkSpinButton *spinbutton, s_op *operation)
{
  if( operation->Template ) {
    Update_template_result();
    return;
  }

  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
}

static void              Callback_check_changed(GtkWidget *button, s_op *operation)
{
  if( operation->Template ) {
    Update_template_result();
    return;
  }

  if( !( app->Flags & APP_FLG_INIT ))
    Update_official_result();
}

/*
  Purpose: Creating and initialise an 'In word' search operation structure
  - pattern : must be 0 or malloc'ed
*/
s_op                     *New_operation_in_word(unsigned int nb_word, _Bool from_end)
{
  GtkWidget              *hbox_main, *item;
  s_op                   *operation;

  CALLOC( operation, 1, sizeof(*operation) );
  operation->Type = OP_TYPE_IN_WORD;
  operation->Number = nb_word;
  operation->Activated = from_end;

  MALLOC( operation->GtkTable, OP_WIDGET_IN_WORD_COUNT * sizeof(*operation->GtkTable) );

  hbox_main = APP_GTK_HBOX( FALSE, 0 );
  g_object_ref( G_OBJECT(hbox_main) );
  operation->GtkTable[OP_WIDGET_IN_WORD_HBOX] = hbox_main;

  Create_del_button( hbox_main );

  item = gtk_label_new( app->Strs[109] ); /* " In word number " */
  gtk_widget_set_tooltip_text( item, app->Strs[68] ); /* "xxx_in_word: The version can be retrieved inside the nth word of the line" */
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  item = gtk_spin_button_new_with_range( 1, 65535, 1 );
  gtk_widget_set_tooltip_text( item, "1 -> 65535" );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(item), nb_word );
  g_signal_connect( G_OBJECT(item), "value-changed", G_CALLBACK(Callback_word_changed), operation );
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  operation->GtkTable[OP_WIDGET_IN_WORD_NB] = item;

  /* - From the end - */
  item = gtk_check_button_new_with_label( app->Strs[66] ); /* " from the end" */
  gtk_widget_set_tooltip_text( item, app->Strs[110] ); /* "xxx_in_word: Getting the nth word starting from end of line" */
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(item), operation->Activated );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_check_changed), operation );
  operation->GtkTable[OP_WIDGET_IN_WORD_CHECK] = item;

  return( operation );
}
