/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

/*
  Purpose: Removing template operations from gui
*/
static void         Remove_template_operations(s_gui *gui, s_tpl *template)
{
  unsigned int      a;

  if( template->OperationCount ) {
    for( a = 0; a < template->OperationCount; ++a )
      if( template->OperationTable[a]->Temporary )
	Free_struct_operation( template->OperationTable[a], 0 );
      else
	gtk_container_remove( GTK_CONTAINER(gui->TemplateSettings[TS_VBOX_OPERATION]), template->OperationTable[a]->GtkTable[0] );
    template->OperationCount = 0;
    free( template->OperationTable );
    template->OperationTable = 0;
  }
}

/*
  Purpose: Callback to make the 'add template' window appear
*/
static void         Callback_add_template(GtkWidget *unused1, s_gui *gui)
{
  s_tpl             *template = &gui->TmpTemplate;

  gui->Flags |= GUI_FLG_TEMPLATE_MODIFY;

#ifdef LIB_HANDY
  if( app->Flags & APP_FLG_PHONE )
    g_object_set( gui->TemplateSettings[TS_BAR_WIN], "title", app->Strs[259], NULL ); /* "Adding a template" */
#endif
  gtk_window_set_title( GTK_WINDOW(gui->TemplateSettings[TS_WIN_MAIN]), app->Strs[259] ); /* "Adding a template" */

  gtk_entry_set_text( GTK_ENTRY(gui->TemplateSettings[TS_ETY_NAME]), "" );
  gtk_entry_set_text( GTK_ENTRY(gui->TemplateSettings[TS_ETY_OFFI_HEADER]), "" );

  /* Operations ------------------------------------------------------------- */
  Remove_template_operations( gui, template );

  gtk_entry_set_text( GTK_ENTRY(gui->TemplateSettings[TS_ETY_RESULT]), "" );

  /* buttons area ----------------------------------------------------------- */
  gtk_widget_hide( gui->TemplateSettings[TS_LBL_ERROR] );
  gtk_widget_hide( gui->TemplateSettings[TS_BUT_SAVE] );
  gtk_widget_show( gui->TemplateSettings[TS_BUT_ADD] );
  gtk_widget_show( gui->TemplateSettings[TS_WIN_MAIN] );

  gtk_widget_set_sensitive( gui->WatcherSettings[WS_MENU_ITEM_TEMPLATE], 1 );
}

/*
  Purpose: Callback to make the 'Modify template' window appear
*/
void                Callback_edit_template(GtkWidget *unused1, s_gui *gui)
{
  s_tpl             *template;
  s_op              *operation;
  unsigned int      a;

  gui->Flags |= GUI_FLG_TEMPLATE_MODIFY;

  template = Get_current_selected_template( gui );
  ASSERT( !template, "template == 0" );

#ifdef LIB_HANDY
  if( app->Flags & APP_FLG_PHONE )
    g_object_set( gui->TemplateSettings[TS_BAR_WIN], "title", app->Strs[260], NULL ); /* "Editing a template" */
#endif
  gtk_window_set_title( GTK_WINDOW(gui->TemplateSettings[TS_WIN_MAIN]), app->Strs[260] ); /* "Editing a template" */

  gtk_entry_set_text( GTK_ENTRY(gui->TemplateSettings[TS_ETY_NAME]), template->Name );

  /* Operations ------------------------------------------------------------- */
  Remove_template_operations( gui, &gui->TmpTemplate );

  gui->TmpTemplate.OperationCount = template->OperationCount;
  REALLOC( gui->TmpTemplate.OperationTable, gui->TmpTemplate.OperationCount * sizeof(*gui->TmpTemplate.OperationTable) );

  for( a = 0; a < template->OperationCount; ++a ) {
    operation = template->OperationTable[a];
    gui->TmpTemplate.OperationTable[a] = operation;
    gtk_box_pack_start( GTK_BOX(gui->TemplateSettings[TS_VBOX_OPERATION]), operation->GtkTable[0], FALSE, FALSE, 0 );
  }
  gtk_widget_show_all( gui->TemplateSettings[TS_VBOX_OPERATION] );

  gtk_entry_set_text( GTK_ENTRY(gui->TemplateSettings[TS_ETY_RESULT]), "" );
  if( template->HeaderOptions )
    gtk_entry_set_text( GTK_ENTRY(gui->TemplateSettings[TS_ETY_OFFI_HEADER]), template->HeaderOptions );
  else
    gtk_entry_set_text( GTK_ENTRY(gui->TemplateSettings[TS_ETY_OFFI_HEADER]), "" );

  /* buttons area ----------------------------------------------------------- */
  gtk_widget_hide( gui->TemplateSettings[TS_LBL_ERROR] );
  gtk_widget_hide( gui->TemplateSettings[TS_BUT_ADD] );
  gtk_widget_show( gui->TemplateSettings[TS_BUT_SAVE] );
  gtk_widget_show( gui->TemplateSettings[TS_WIN_MAIN] );
}

/*
  Purpose: Callback to move up a template in the list
*/
static void         Callback_move_up(GtkWidget *widget, gpointer data)
{
  /* GtkTreeIter       iter; */
  /* GtkTreeModel      *model; */
  /* gchar             *name; */
  /* unsigned int      a, count; */

  /* if( !app->Gui.CancelPref.Flags ) */
  /*   gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] ); */
  /* app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_WATCHER_ORDER; */

  /* gtk_tree_selection_get_selected( app->Gui.SelectionWatcher, &model, &iter ); */
  /* gtk_tree_model_get( model, &iter, 0, &name, -1 ); */
  /* for( a = 1; a < app->Gui.WatcherCount; ++a ) /\* can't be first one *\/ */
  /*   if( !strcmp( name, app->Gui.WatcherTable[a]->Name )) */
  /*     break; */

  /* gtk_list_store_move_before( app->Gui.ListWatchers, &iter, &app->Gui.WatcherTable[a - 1]->Iter ); */

  /* // Swap_templates */

  /* if( a == 1 ) */
  /*   gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_UP], FALSE ); */
  /* else */
  /*   gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_UP], TRUE ); */
  /* gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_DOWN], TRUE ); */

  /* count = 0; */
  /* for( a = 0; a < app->Gui.WatcherCount; ++a ) */
  /*   if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE )) */
  /*     app->Gui.WatcherTable[a]->VisiblePos = count++; */
}

/*
  Purpose: Callback to move down a template in the list
*/
static void         Callback_move_down(GtkWidget *widget, gpointer data)
{
  /* GtkTreeIter       iter; */
  /* GtkTreeModel      *model; */
  /* gchar             *name; */
  /* unsigned int      a, count; */

  /* if( !app->Gui.CancelPref.Flags ) */
  /*   gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] ); */
  /* app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_WATCHER_ORDER; */

  /* gtk_tree_selection_get_selected( app->Gui.SelectionWatcher, &model, &iter ); */
  /* gtk_tree_model_get( model, &iter, 0, &name, -1 ); */
  /* for( a = 0; a < app->Gui.WatcherCount; ++a ) */
  /*   if( !strcmp( name, app->Gui.WatcherTable[a]->Name )) */
  /*     break; */

  /* gtk_list_store_move_after( app->Gui.ListWatchers, &iter, &app->Gui.WatcherTable[a + 1]->Iter ); */
  /* // Swap_templates */

  /* if( a + 2 < app->Gui.WatcherCount ) */
  /*   gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_DOWN], TRUE ); */
  /* else */
  /*   gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_DOWN], FALSE ); */
  /* gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_UP], TRUE ); */

  /* count = 0; */
  /* for( a = 0; a < app->Gui.WatcherCount; ++a ) */
  /*   if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE )) */
  /*     app->Gui.WatcherTable[a]->VisiblePos = count++; */
}

/*
  Purpose: Callback to make sensitive, or not, the buttons: 'Edit' and 'Remove' for the GUI template table
  - 'select' can be 0
*/
static void         Callback_select_changed(GtkTreeSelection *select, s_gui *gui)
{
  GtkTreeModel      *model;
  GtkTreeIter       iter;

  if( gtk_tree_selection_get_selected( gui->SelectionTemplate, &model, &iter )) {
    /* gtk_tree_model_get( model, &iter, 0, &name, -1 ); */

    /* if( strcmp( name, gui->WatcherTable[0]->Name )) */
    /*   gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_UP], 1 ); */
    /* else */
    /*   gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_UP], 0 ); */

    /* if( strcmp( name, gui->WatcherTable[gui->WatcherCount - 1]->Name )) */
    /*   gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_DOWN], 1 ); */
    /* else */
    /*   gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_DOWN], 0 ); */

    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_EDIT_TPL], 1 );
    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_REMOVE_TPL], 1 );

    if( gui->Flags & GUI_FLG_TEMPLATE_MODIFY )
      Callback_edit_template( gui->PreferenceSettings[PS_BUT_EDIT_TPL], gui );
  }
  else {
    /* gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_UP], 0 ); */
    /* gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_DOWN], 0 ); */
    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_EDIT_TPL], 0 );
    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_REMOVE_TPL], 0 );
  }
}

/*
  Purpose: Callback to remove a template
*/
static void         Callback_remove_template(GtkTreeViewColumn *col, s_gui *gui)
{
  GtkTreeModel      *model;
  gchar             *name, *tpl;
  s_tpl             *template;
  GtkTreeIter       iter;
  unsigned int      a, length;
  unsigned char     move;

  if( ! gtk_tree_selection_get_selected( gui->SelectionTemplate, &model, &iter ))
    return;

  //#LOCK template table

  Remove_template_operations( gui, &gui->TmpTemplate );
  gtk_widget_hide( gui->TemplateSettings[TS_WIN_MAIN] );

  if( ! gui->CancelPref.Flags )
    gtk_widget_show( gui->PreferenceSettings[PS_BUT_SAVE] );

  gtk_tree_model_get( model, &iter, 0, &name, -1 );
  move = 0;
  for( a = 0; a < gui->File.TemplateCount; ++a ) {
    template = gui->File.TemplateTable[a];
    if( move ) {
      gui->File.TemplateTable[a - 1] = template;
      ASSERT( !gui->File.TemplateTable[a - 1], "gui->File.TemplateTable[%u - 1] = 0", a );
    }
    else {
      ASSERT( !template, "gui->File.TemplateTable[%u] = 0", a );
      if( ! strcmp( name, template->Name )) {
	//# LockTable
  	gui->File.TemplateTable[a] = 0;
	length = strlen( template->Name );
	MALLOC( tpl, 9 + length + 1 ); /* "template:" + '\0' */
	memcpy( tpl, "template:", 9 );
	memcpy( &tpl[9], template->Name, length );
	tpl[9 + length] = '\0';
  	Remove_section_from_file( tpl );
	free( tpl );
	Free_struct_template( template, 0 );
  	move = a + 1;
      }
    }
  }
  if( a )
    gui->File.TemplateTable[a - 1] = 0;
  --gui->File.TemplateCount;

  /* Selecting previous of the one removed */
  ASSERT( !move, "move = 0" );
  if( gui->File.TemplateTable[move - 1] )
    gtk_tree_selection_select_iter( gui->SelectionTemplate, &gui->File.TemplateTable[move - 1]->Iter );

  /* Updating related things */
  if( ! gui->File.TemplateCount )
    gtk_widget_set_sensitive( gui->WatcherSettings[WS_MENU_ITEM_TEMPLATE], 0 );
}

/*
  Purpose: Create the template list tab of the settings window
*/
void                  Create_preferences_tab_templates(GtkWidget *tab)
{
  GtkWidget           *vbox, *vbox_main, *hbox, *lview, *item;
  GtkTreeViewColumn   *col;
  s_tpl               *template;
  unsigned int        a;

  vbox_main = APP_GTK_VBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(tab), vbox_main );
  gtk_notebook_set_tab_label_text( GTK_NOTEBOOK(tab), vbox_main, app->Strs[253] ); /* "Templates" */

  /* ----- Templates list */
  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_box_pack_start( GTK_BOX(vbox_main), hbox, TRUE, TRUE, 0 );

  item = gtk_scrolled_window_new( NULL, NULL );
  gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(item), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
  gtk_box_pack_start( GTK_BOX(hbox), item, TRUE, TRUE, 0 );

  app->Gui.ListTemplates = gtk_list_store_new( 1, G_TYPE_STRING );

  lview = gtk_tree_view_new_with_model( GTK_TREE_MODEL(app->Gui.ListTemplates) );
  gtk_container_add( GTK_CONTAINER(item), lview );

  app->Gui.SelectionTemplate = gtk_tree_view_get_selection( GTK_TREE_VIEW(lview) );
  g_signal_connect( G_OBJECT(app->Gui.SelectionTemplate), "changed", G_CALLBACK(Callback_select_changed), &app->Gui );

  /* ----- Name column */
  col = gtk_tree_view_column_new_with_attributes( app->Strs[31], app->CellRender[COL_NAME], "text", 0, NULL ); /* "Name" */
  gtk_tree_view_column_set_clickable( col, TRUE );
  gtk_tree_view_append_column( GTK_TREE_VIEW(lview), col );

  /* ---- Filling table */
  for( a = 0; a < app->Gui.File.TemplateCount; ++a ) {
    template = app->Gui.File.TemplateTable[a];
    gtk_list_store_append ( app->Gui.ListTemplates, &template->Iter );
    gtk_list_store_set( app->Gui.ListTemplates, &template->Iter, 0, template->Name, -1 );
  }

  /* ---- Bottom buttons ------------------------------------------------------------------- */
  vbox = APP_GTK_VBOX( FALSE, 0 );
  gtk_box_pack_start( GTK_BOX(hbox), vbox, FALSE, FALSE, 0 );

  /* Add template */
  item = gtk_button_new_with_mnemonic( app->Strs[33] ); /* "_Add" */
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_add_template), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_ADD_TPL] = item;

  /* Edit template */
  item = gtk_button_new_with_mnemonic( app->Strs[34] ); /* "_Edit" */
  gtk_widget_set_sensitive( item, 0 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_edit_template), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_EDIT_TPL] = item;

  /* Move up template */
  item = gtk_button_new_with_mnemonic( app->Strs[48] ); /* "Move up" */
  gtk_widget_set_sensitive( item, 0 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_move_up), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_MOVE_UP_TPL] = item;

  /* Move down template */
  item = gtk_button_new_with_mnemonic( app->Strs[49] ); /* "Move down " */
  gtk_widget_set_sensitive( item, 0 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_move_down), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_MOVE_DOWN_TPL] = item;

  /* Separator */
  item = APP_GTK_HSEPAR();
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  item = APP_GTK_HSEPAR();
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );

  /* Remove template */
  item = gtk_button_new_with_mnemonic( app->Strs[35] ); /* "Remove" */
  gtk_widget_set_sensitive( item, 0 );
  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(Callback_remove_template), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_REMOVE_TPL] = item;
}
