/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

/*
  Purpose: Emptying operation list in GUI
*/
void                Empty_search_block_list(s_gui *gui)
{
  GList             *list;
  s_op              *operation;
  unsigned int      a;

  /* Official list */
  list = gtk_container_get_children( GTK_CONTAINER(gui->WatcherSettings[WS_VBOX_SEARCH_OFFI]) );
  for(; list != NULL; list = list->next )
    if( list->data != gui->WatcherSettings[WS_ADD_BLOCK_OFFICIAL] )
      gtk_container_remove( GTK_CONTAINER(gui->WatcherSettings[WS_VBOX_SEARCH_OFFI]), list->data );

  if( list )
    g_list_free( list );

  for( a = 0; a < gui->TmpWatcher.OfficialOperationCount; ++a ) {
    operation = gui->TmpWatcher.OfficialOperationTable[a];
    if( operation->Temporary )
      Free_struct_operation( operation, 0 );
  }
  gui->TmpWatcher.OfficialOperationCount = 0;

  /* Local list */
  list = gtk_container_get_children( GTK_CONTAINER(gui->WatcherSettings[WS_VBOX_SEARCH_LOCAL]) );
  for(; list != NULL; list = list->next )
    if( list->data != gui->WatcherSettings[WS_ADD_BLOCK_LOCAL] )
      gtk_container_remove( GTK_CONTAINER(gui->WatcherSettings[WS_VBOX_SEARCH_LOCAL]), list->data );

  if( list )
    g_list_free( list );

  for( a = 0; a < gui->TmpWatcher.LocalOperationCount; ++a ) {
    operation = gui->TmpWatcher.LocalOperationTable[a];
    if( operation->Temporary )
      Free_struct_operation( operation, 0 );
  }
  gui->TmpWatcher.LocalOperationCount = 0;

  /* Template */
  if( gui->TmpWatcher.TemplateName ) {
    free( gui->TmpWatcher.TemplateName );
    gui->TmpWatcher.TemplateName = 0;
  }
}

/*
  Purpose: Callback to make the 'add watcher' window appear
*/
static void         Callback_add_watcher(GtkTreeViewColumn *col, gpointer data)
{
  s_wtr             *watcher = &app->Gui.TmpWatcher;
  char              *tmp;

  app->Gui.Flags |= GUI_FLG_WATCHER_MODIFY | GUI_FLG_NEW_WATCHER;

  if( ! app->Gui.CancelPref.Flags )
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_WATCHER_ORDER;

  /* Clean before bzero watcher --------------------------------------------- */
  Empty_search_block_list( &app->Gui );

  free( watcher->Name );
  bzero( watcher, sizeof(*watcher) );

  gtk_window_set_title( GTK_WINDOW(app->Gui.WatcherSettings[WS_WIN_MAIN]), app->Strs[42] ); /* "Adding a watcher" */
#ifdef LIB_HANDY
  if( app->Flags & APP_FLG_PHONE )
    g_object_set( app->Gui.WatcherSettings[WS_BAR_WIN], "title", app->Strs[42], NULL ); /* "Adding a watcher" */
#endif

  /* The watcher properties ------------------------------------------------- */
  gtk_entry_set_text( GTK_ENTRY(app->Gui.WatcherSettings[WS_ETY_NAME]), "" );
  gtk_image_set_from_pixbuf( GTK_IMAGE(app->Gui.WatcherSettings[WS_IMG_LOGO]), app->Gui.ImgTable[IMG_DEFAULT_WATCHER] ); //#memleak: will this free the potential unused previous pix ?

  /* Official version --------------------------------------------------------- */
  gtk_entry_set_text( GTK_ENTRY(app->Gui.WatcherSettings[WS_ETY_OFFI_URL]), "" );
  gtk_entry_set_text( GTK_ENTRY(app->Gui.WatcherSettings[WS_ETY_OFFI_HEADER]), "" );

#ifdef LIB_HANDY
  APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.WatcherSettings[WS_TOGGLE_HEADER], 0 );
#else
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.WatcherSettings[WS_TOGGLE_HEADER]), 0 );
#endif

  if( watcher->OfficialOperationCount != 3 ) {
    watcher->OfficialOperationCount = 3;
    REALLOC( watcher->OfficialOperationTable, watcher->OfficialOperationCount * sizeof(*watcher->OfficialOperationTable) );
  }

  watcher->OfficialOperationTable[0] = New_operation_after_line( 0 );
  gtk_container_add( GTK_CONTAINER(app->Gui.WatcherSettings[WS_VBOX_SEARCH_OFFI]), watcher->OfficialOperationTable[0]->GtkTable[0] );
  watcher->OfficialOperationTable[0]->Temporary = 1;

  watcher->OfficialOperationTable[1] = New_operation_in_line( 1, 0, 0 );
  gtk_container_add( GTK_CONTAINER(app->Gui.WatcherSettings[WS_VBOX_SEARCH_OFFI]), watcher->OfficialOperationTable[1]->GtkTable[0] );
  watcher->OfficialOperationTable[1]->Temporary = 1;

  watcher->OfficialOperationTable[2] = New_operation_in_section( 1, '<', '>' );
  gtk_container_add( GTK_CONTAINER(app->Gui.WatcherSettings[WS_VBOX_SEARCH_OFFI]), watcher->OfficialOperationTable[2]->GtkTable[0] );
  watcher->OfficialOperationTable[2]->Temporary = 1;

  gtk_entry_set_text( GTK_ENTRY(app->Gui.WatcherSettings[WS_ETY_OFFI_RESULT]), "" );

  tmp = Build_cachefile_name( "tmp_watcher_result", 18 );
  unlink( tmp );
  free( tmp );

#ifdef LIB_HANDY
  hdy_expander_row_set_expanded( (HdyExpanderRow*) app->Gui.WatcherSettings[WS_EXPROW_OFFICIAL], 1 );
#endif

  gtk_widget_show( app->Gui.WatcherSettings[WS_ADD_BLOCK_OFFICIAL] );
  gtk_widget_show_all( app->Gui.WatcherSettings[WS_VBOX_SEARCH_OFFI] );

  /* Local version ---------------------------------------------------------- */
  gtk_entry_set_text( GTK_ENTRY(app->Gui.WatcherSettings[WS_ETY_LOCAL_VERSION]), "" );
  gtk_entry_set_text( GTK_ENTRY(app->Gui.WatcherSettings[WS_ETY_LOCAL_EXEC]), "" );
  gtk_entry_set_text( GTK_ENTRY(app->Gui.WatcherSettings[WS_ETY_ARGUMENT]), "" );
#ifdef LIB_HANDY
  APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.WatcherSettings[WS_TOGGLE_OPTIONNAL], 0 );
  APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.WatcherSettings[WS_TOGGLE_STDERR], 0 );
  hdy_expander_row_set_expanded( (HdyExpanderRow*) app->Gui.WatcherSettings[WS_EXPROW_LOCAL], 1 );
#else
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.WatcherSettings[WS_TOGGLE_OPTIONNAL]), 0 );
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.WatcherSettings[WS_TOGGLE_STDERR]), 0 );
#endif
  gtk_widget_show( app->Gui.WatcherSettings[WS_ADD_BLOCK_LOCAL] );
  gtk_widget_show_all( app->Gui.WatcherSettings[WS_VBOX_SEARCH_LOCAL] );

  /* Other options ---------------------------------------------------------- */
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(app->Gui.WatcherSettings[WS_SPIN_TIMER]), DEFAULT_TIMER );
  gtk_entry_set_text( GTK_ENTRY(app->Gui.WatcherSettings[WS_ETY_TRIGGER_MISMATCH]), "" );
  gtk_entry_set_text( GTK_ENTRY(app->Gui.WatcherSettings[WS_ETY_TRIGGER_ERROR]), "" );
#ifdef LIB_HANDY
  APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.WatcherSettings[WS_TOGGLE_MISMATCH], 0 );
  APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.WatcherSettings[WS_TOGGLE_ERROR], 0 );
#else
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.WatcherSettings[WS_TOGGLE_MISMATCH]), 0 );
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.WatcherSettings[WS_TOGGLE_ERROR]), 0 );
#endif
  gtk_widget_set_sensitive( app->Gui.WatcherSettings[WS_ETY_TRIGGER_MISMATCH], 0 );
  gtk_widget_set_sensitive( app->Gui.WatcherSettings[WS_ETY_TRIGGER_ERROR], 0 );

  /* buttons area ----------------------------------------------------------- */
  gtk_widget_hide( app->Gui.WatcherSettings[WS_LBL_ERROR] );
  gtk_widget_hide( app->Gui.WatcherSettings[WS_BUT_SAVE] );
  gtk_widget_show( app->Gui.WatcherSettings[WS_BUT_ADD] );
  gtk_widget_show( app->Gui.WatcherSettings[WS_WIN_MAIN] );
}

/*
  Purpose: Getting the current selected watcher
*/
s_wtr                    *Get_current_selected_watcher(s_gui *gui)
{
  gchar                  *name = 0;
  GtkTreeModel           *model;
  GtkTreeIter            iter;
  unsigned int           pos;

  if( gtk_tree_selection_get_selected( gui->SelectionWatcher, &model, &iter )) {
    gtk_tree_model_get( model, &iter, COL_NAME, &name, -1 );
    for( pos = 0; pos < gui->WatcherCount; ++pos )
      if( ! strcmp( name, gui->WatcherTable[pos]->Name ))
        return( gui->WatcherTable[pos] );
  }

  FIXME( "Reaching the 'adding in progress' watcher" );
  return( 0 );
}


/*
  Purpose: Callback to make the 'Modify watcher' window appear
*/
void                Callback_edit_watcher(GtkWidget *unused1, s_gui *gui)
{
  s_op              *operation;
  s_wtr             *watcher = 0;
  char              *tmp;
  unsigned int      a;

  gui->Flags |= GUI_FLG_WATCHER_MODIFY;
  gui->Flags &= ~( GUI_FLG_NEW_WATCHER );

  watcher = Get_current_selected_watcher( gui );
  ASSERT( !watcher, "watcher = 0" );

  gtk_window_set_title( GTK_WINDOW(gui->WatcherSettings[WS_WIN_MAIN]), app->Strs[134] ); /* "Editing a watcher" */
#ifdef LIB_HANDY
  if( app->Flags & APP_FLG_PHONE )
    g_object_set( gui->WatcherSettings[WS_BAR_WIN], "title", app->Strs[134], NULL ); /* "Editing a watcher" */
#endif

  free( gui->TmpWatcher.Name );
  bzero( &gui->TmpWatcher, sizeof(gui->TmpWatcher) );

  tmp = Build_cachefile_name( "tmp_watcher_result", 18 );
  unlink( tmp );
  free( tmp );

  /* The watcher properties ------------------------------------------------- */
  gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_NAME]), watcher->Name );

  Set_icon_to_button( gui->WatcherSettings[WS_IMG_LOGO], watcher->CurrentPixbuf );

  /* Official version ------------------------------------------------------- */
  ASSERT( !watcher->OfficialURL, "OfficialURL = 0" );
  gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_OFFI_URL]), watcher->OfficialURL );

  if( watcher->Flags & WTR_FLG_OFFICIAL_HEADER ) {
#ifdef LIB_HANDY
    APP_GTK_TOGGLE_SET_ACTIVE( gui->WatcherSettings[WS_TOGGLE_HEADER], 1 );
#else
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_TOGGLE_HEADER]), 1 );
#endif
    gtk_widget_set_sensitive( gui->WatcherSettings[WS_ETY_OFFI_HEADER], 1 );
  }
  else {
#ifdef LIB_HANDY
    APP_GTK_TOGGLE_SET_ACTIVE( gui->WatcherSettings[WS_TOGGLE_HEADER], 0 );
#else
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_TOGGLE_HEADER]), 0 );
#endif
    gtk_widget_set_sensitive( gui->WatcherSettings[WS_ETY_OFFI_HEADER], 0 );
  }

  if( watcher->OfficialHeader )
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_OFFI_HEADER]), watcher->OfficialHeader );
  else
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_OFFI_HEADER]), "" );

  gtk_spin_button_set_value( GTK_SPIN_BUTTON(gui->WatcherSettings[WS_SPIN_TIMER]), watcher->OfficialTimer );

  Empty_search_block_list( gui );

  if( watcher->TemplateName ) {
    for( a = 0; a < gui->File.TemplateCount; ++a )
      if( ! strcmp( watcher->TemplateName, gui->File.TemplateTable[a]->Name )) {
	gtk_box_pack_start( GTK_BOX(gui->WatcherSettings[WS_VBOX_SEARCH_OFFI]), gui->File.TemplateTable[a]->MainBox, FALSE, FALSE, 0 );
	STRDUP( gui->TmpWatcher.TemplateName, gui->File.TemplateTable[a]->Name );
	gtk_widget_show_all( gui->WatcherSettings[WS_VBOX_SEARCH_OFFI] );
	gtk_widget_hide( gui->WatcherSettings[WS_ADD_BLOCK_OFFICIAL] );
	break;
      }
    if( a != gui->File.TemplateCount )
      gtk_widget_show( gui->WatcherSettings[WS_ADD_BLOCK_OFFICIAL] );
  }
  else {
    /* Official search operation */
    if( gui->TmpWatcher.OfficialOperationCount != watcher->OfficialOperationCount ) {
      gui->TmpWatcher.OfficialOperationCount = watcher->OfficialOperationCount;
      REALLOC( gui->TmpWatcher.OfficialOperationTable, gui->TmpWatcher.OfficialOperationCount * sizeof(*gui->TmpWatcher.OfficialOperationTable) );
    }

    for( a = 0; a < watcher->OfficialOperationCount; ++a ) {
      operation = watcher->OfficialOperationTable[a];
      gui->TmpWatcher.OfficialOperationTable[a] = operation;
      gtk_box_pack_start( GTK_BOX(gui->WatcherSettings[WS_VBOX_SEARCH_OFFI]), operation->GtkTable[0], FALSE, FALSE, 0 );
    }
    if( watcher->OfficialOperationCount < MAX_COUNT_OPERATION )
      gtk_widget_show( gui->WatcherSettings[WS_ADD_BLOCK_OFFICIAL] );
    else
      gtk_widget_hide( gui->WatcherSettings[WS_ADD_BLOCK_OFFICIAL] );
    gtk_widget_show_all( gui->WatcherSettings[WS_VBOX_SEARCH_OFFI] );

    /* Local search operation */
    if( gui->TmpWatcher.LocalOperationCount != watcher->LocalOperationCount ) {
      gui->TmpWatcher.LocalOperationCount = watcher->LocalOperationCount;
      REALLOC( gui->TmpWatcher.LocalOperationTable, gui->TmpWatcher.LocalOperationCount * sizeof(*gui->TmpWatcher.LocalOperationTable) );
    }

    for( a = 0; a < watcher->LocalOperationCount; ++a ) {
      operation = watcher->LocalOperationTable[a];
      gui->TmpWatcher.LocalOperationTable[a] = operation;
      gtk_box_pack_start( GTK_BOX(gui->WatcherSettings[WS_VBOX_SEARCH_LOCAL]), operation->GtkTable[0], FALSE, FALSE, 0 );
    }

    if( watcher->LocalOperationCount < MAX_COUNT_OPERATION )
      gtk_widget_show( gui->WatcherSettings[WS_ADD_BLOCK_LOCAL] );
    else
      gtk_widget_hide( gui->WatcherSettings[WS_ADD_BLOCK_LOCAL] );
    gtk_widget_show_all( gui->WatcherSettings[WS_VBOX_SEARCH_LOCAL] );
  }

  if( watcher->OfficialVersion )
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_OFFI_RESULT]), watcher->OfficialVersion );
  else
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_OFFI_RESULT]), "" );

  /* Local version ---------------------------------------------------------- */
#ifdef LIB_HANDY
  APP_GTK_TOGGLE_SET_ACTIVE( gui->WatcherSettings[WS_TOGGLE_OPTIONNAL], ( watcher->Flags & WTR_FLG_OPTIONNAL ) != 0 );
#else
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_TOGGLE_OPTIONNAL]), ( watcher->Flags & WTR_FLG_OPTIONNAL ) != 0 );
#endif

  if( watcher->LocalVersion )
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_LOCAL_VERSION]), watcher->LocalVersion );
  else
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_LOCAL_VERSION]), "" );

  if( watcher->LocalExec )
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_LOCAL_EXEC]), watcher->LocalExec );
  else
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_LOCAL_EXEC]), "" );

  if( watcher->LocalArg )
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_ARGUMENT]), watcher->LocalArg );
  else
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_ARGUMENT]), "" );

#ifdef LIB_HANDY
  APP_GTK_TOGGLE_SET_ACTIVE( gui->WatcherSettings[WS_TOGGLE_STDERR], ( watcher->Flags & WTR_FLG_STDERR ) != 0 );
#else
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_TOGGLE_STDERR]), ( watcher->Flags & WTR_FLG_STDERR ) != 0 );
#endif

  if( watcher->Flags & WTR_FLG_LOCAL_AUTO )
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_RAD_VERSION_AUTO]), 1 );
  else
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_RAD_VERSION_MANUAL]), 1 );

  /* Other options ---------------------------------------------------------- */
  if( watcher->Trigger_on_mismatch )
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_TRIGGER_MISMATCH]), watcher->Trigger_on_mismatch );
  else
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_TRIGGER_MISMATCH]), "" );

  if( watcher->Flags & WTR_FLG_TRIGGER_MISMATCH ) {
#ifdef LIB_HANDY
    APP_GTK_TOGGLE_SET_ACTIVE( gui->WatcherSettings[WS_TOGGLE_MISMATCH], 1 );
#else
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_TOGGLE_MISMATCH]), 1 );
#endif
    gtk_widget_set_sensitive( gui->WatcherSettings[WS_ETY_TRIGGER_MISMATCH], 1 );
  }
  else {
#ifdef LIB_HANDY
    APP_GTK_TOGGLE_SET_ACTIVE( gui->WatcherSettings[WS_TOGGLE_MISMATCH], 0 );
#else
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_TOGGLE_MISMATCH]), 0 );
#endif
    gtk_widget_set_sensitive( gui->WatcherSettings[WS_ETY_TRIGGER_MISMATCH], 0 );
  }

  if( watcher->Trigger_on_error )
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_TRIGGER_ERROR]), watcher->Trigger_on_error );
  else
    gtk_entry_set_text( GTK_ENTRY(gui->WatcherSettings[WS_ETY_TRIGGER_ERROR]), "" );

  if( watcher->Flags & WTR_FLG_TRIGGER_ERROR ) {
#ifdef LIB_HANDY
    APP_GTK_TOGGLE_SET_ACTIVE( gui->WatcherSettings[WS_TOGGLE_ERROR], 1 );
#else
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_TOGGLE_ERROR]), 1 );
#endif
    gtk_widget_set_sensitive( gui->WatcherSettings[WS_ETY_TRIGGER_ERROR], 1 );
  }
  else {
#ifdef LIB_HANDY
    APP_GTK_TOGGLE_SET_ACTIVE( gui->WatcherSettings[WS_TOGGLE_ERROR], 0 );
#else
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(gui->WatcherSettings[WS_TOGGLE_ERROR]), 0 );
#endif
    gtk_widget_set_sensitive( gui->WatcherSettings[WS_ETY_TRIGGER_ERROR], 0 );
  }

  /* Add watcher ------------------------------------------------------------ */
  gtk_widget_hide( gui->WatcherSettings[WS_BUT_ADD] );
  gtk_widget_hide( gui->WatcherSettings[WS_LBL_ERROR] );
  gtk_widget_show( gui->WatcherSettings[WS_BUT_SAVE] );
  gtk_widget_show( gui->WatcherSettings[WS_WIN_MAIN] );
}

/*
  Purpose: Swaping the position of 2 watcher
*/
static void         Swap_watchers(s_wtr *wtr1, s_wtr *wtr2)
{
  unsigned int      pos;

  ASSERT( wtr1->Pos >= app->Gui.WatcherTableSize, "Watcher '%s': invalid pos (%u / %u)", wtr1->Name, wtr1->Pos, app->Gui.WatcherTableSize );
  ASSERT( wtr2->Pos >= app->Gui.WatcherTableSize, "Watcher '%s': invalid pos (%u / %u)", wtr2->Name, wtr2->Pos, app->Gui.WatcherTableSize );
  app->Gui.WatcherTable[wtr1->Pos] = wtr2;
  app->Gui.WatcherTable[wtr2->Pos] = wtr1;
  pos = wtr1->Pos;
  wtr1->Pos = wtr2->Pos;
  wtr2->Pos = pos;

  app->Gui.Save.Flags |= SAVE_FLG_WATCHER_ORDER;

  Update_main_geometry();
}

/*
  Purpose: Callback to move up a watcher in the list
*/
static void         Callback_move_up(GtkWidget *widget, gpointer data)
{
  GtkTreeIter       iter;
  GtkTreeModel      *model;
  gchar             *name;
  unsigned int      a, count;

  if( !app->Gui.CancelPref.Flags )
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_WATCHER_ORDER;

  gtk_tree_selection_get_selected( app->Gui.SelectionWatcher, &model, &iter );
  gtk_tree_model_get( model, &iter, COL_NAME, &name, -1 );
  for( a = 1; a < app->Gui.WatcherCount; ++a ) /* can't be first one */
    if( !strcmp( name, app->Gui.WatcherTable[a]->Name ))
      break;

  gtk_list_store_move_before( app->Gui.ListWatchers, &iter, &app->Gui.WatcherTable[a - 1]->Iter );

  Swap_watchers( app->Gui.WatcherTable[a], app->Gui.WatcherTable[a - 1] );

  if( a == 1 )
    gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_UP], FALSE );
  else
    gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_UP], TRUE );
  gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_DOWN], TRUE );

  count = 0;
  for( a = 0; a < app->Gui.WatcherCount; ++a )
    if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE ))
      app->Gui.WatcherTable[a]->VisiblePos = count++;
}

/*
  Purpose: Callback to move down a watcher in the list
*/
static void         Callback_move_down(GtkWidget *widget, gpointer data)
{
  GtkTreeIter       iter;
  GtkTreeModel      *model;
  gchar             *name;
  unsigned int      a, count;

  if( !app->Gui.CancelPref.Flags )
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_WATCHER_ORDER;

  gtk_tree_selection_get_selected( app->Gui.SelectionWatcher, &model, &iter );
  gtk_tree_model_get( model, &iter, COL_NAME, &name, -1 );
  for( a = 0; a < app->Gui.WatcherCount; ++a )
    if( !strcmp( name, app->Gui.WatcherTable[a]->Name ))
      break;

  gtk_list_store_move_after( app->Gui.ListWatchers, &iter, &app->Gui.WatcherTable[a + 1]->Iter );
  Swap_watchers( app->Gui.WatcherTable[a], app->Gui.WatcherTable[a + 1] );

  if( a + 2 < app->Gui.WatcherCount )
    gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_DOWN], TRUE );
  else
    gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_DOWN], FALSE );
  gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_MOVE_UP], TRUE );

  count = 0;
  for( a = 0; a < app->Gui.WatcherCount; ++a )
    if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE ))
      app->Gui.WatcherTable[a]->VisiblePos = count++;
}

/*
  Purpose: Callback for toggle cells of the GUI watcher list
*/
static void         Callback_toggle(GtkCellRendererToggle *cell, gchar *path, gpointer data)
{
  gchar             *name;
  s_wtr             *watcher;
  pthread_t         tmp_id;
  GtkTreeIter       iter;
  unsigned int      a, count;

  gtk_tree_model_get_iter_from_string( GTK_TREE_MODEL(app->Gui.ListWatchers), &iter, path );
  gtk_tree_model_get( GTK_TREE_MODEL(app->Gui.ListWatchers), &iter, COL_NAME, &name, -1 );

  for( a = 0; a < app->Gui.WatcherCount; ++a )
    if( ! strcmp( name, app->Gui.WatcherTable[a]->Name ))
      break;

  ASSERT( a == app->Gui.WatcherCount, "a == app->Gui.WatcherCount" );

  watcher = app->Gui.WatcherTable[a];

  if( gtk_cell_renderer_toggle_get_active( cell )) {
    watcher->Flags |= WTR_FLG_DISABLE;
    gtk_list_store_set( GTK_LIST_STORE(app->Gui.ListWatchers), &iter, COL_STATE, FALSE, -1 );
  }
  else {
    watcher->Flags &= ~( WTR_FLG_DISABLE );
    gtk_list_store_set( GTK_LIST_STORE( app->Gui.ListWatchers ), &iter, COL_STATE, TRUE, -1 );
  }
  Update_main_geometry();

  count = 0;
  app->Gui.Flags &= ~( GUI_FLG_NO_WATCHER );
  for( a = 0; a < app->Gui.WatcherCount; ++a )
    if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE ))
      app->Gui.WatcherTable[a]->VisiblePos = count++;
  if( ! count )
    app->Gui.Flags |= GUI_FLG_NO_WATCHER;

  if( !( watcher->Flags & WTR_FLG_DISABLE ))
    if( pthread_create( &tmp_id, &app->Attr, th_Retrieve_both_versions, watcher )) {
      gtk_label_set_label( GTK_LABEL(watcher->LabelLocalVersionResult),
			   app->Strs[179] ); /* "Unable to create a thread" */
      gtk_label_set_label( GTK_LABEL(watcher->LabelOfficialVersionResult),
			   app->Strs[179] ); /* "Unable to create a thread" */
      ERR_SYS( "%s", app->Strs[179] ); /* "Unable to create a thread" */
    }
}

/*
  Purpose: Callback to make sensitive, or not, the buttons: 'Edit' and 'Remove' for the GUI watcher table
*/
static void         Callback_select_changed(GtkTreeSelection *select, s_gui *gui)
{
  GtkTreeModel      *model;
  gchar             *name;
  GtkTreeIter       iter;
  unsigned int      a;

  if( gtk_tree_selection_get_selected( gui->SelectionWatcher, &model, &iter )) {
    gtk_tree_model_get( model, &iter, COL_NAME, &name, -1 );

    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_EDIT], 1 );
    if( strcmp( name, gui->WatcherTable[0]->Name ))
      gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_UP], 1 );
    else
      gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_UP], 0 );

    if( strcmp( name, gui->WatcherTable[gui->WatcherCount - 1]->Name ))
      gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_DOWN], 1 );
    else
      gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_DOWN], 0 );

    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_REMOVE], 1 );

    for( a = 0; a < gui->WatcherCount; ++a )
      if( ! strcmp( name, gui->WatcherTable[a]->Name )) {
	gui->SelectedWatcher = gui->WatcherTable[a];
	break;
      }

    if( gui->Flags & GUI_FLG_WATCHER_MODIFY )
      Callback_edit_watcher( gui->PreferenceSettings[PS_BUT_EDIT], gui );
  }
  else {
    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_EDIT], 0 );
    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_UP], 0 );
    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_MOVE_DOWN], 0 );
    gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_REMOVE], 0 );
    if( gui->Flags & GUI_FLG_WATCHER_MODIFY ) {
      gtk_widget_hide( gui->WatcherSettings[WS_WIN_MAIN] );
      gui->Flags &= ~( GUI_FLG_WATCHER_MODIFY );
    }
  }
}

/*
  Purpose: Callback to remove a watcher
*/
static void         Callback_remove_watcher(GtkTreeViewColumn *col, s_gui *gui)
{
  GtkTreeModel      *model;
  gchar             *name;
  s_wtr             *watcher, *removed = 0;
  GtkTreeIter       iter;
  unsigned int      a, count;
  unsigned char     move;

  if( ! gtk_tree_selection_get_selected( gui->SelectionWatcher, &model, &iter ))
    return;

  gtk_widget_hide( gui->WatcherSettings[WS_WIN_MAIN] );

  if( ! gui->CancelPref.Flags )
    gtk_widget_show( gui->PreferenceSettings[PS_BUT_SAVE] );
  gui->CancelPref.Flags |= GUI_CANCEL_FLG_WATCHER_ORDER;

  /* removing from preference list */
  gtk_tree_model_get( model, &iter, COL_NAME, &name, -1 );

  /* removing from gui table */
  move = 0;
  for( a = 0; a < gui->WatcherCount; ++a ) {
    watcher = gui->WatcherTable[a];
    if( move ) {
      gui->WatcherTable[a - 1] = watcher;
      ASSERT( !gui->WatcherTable[a - 1], "gui->WatcherTable[a - 1] = 0" );
      gui->WatcherTable[a - 1]->Pos = a - 1;
    }
    else {
      ASSERT( !watcher, "gui->WatcherTable[a] = 0" );
      if( !strcmp( name, watcher->Name )) {
	if( !( watcher->Flags & ( WTR_FLG_LOCAL_IN_PROGRESS | WTR_FLG_OFFICIAL_IN_PROGRESS ))) {
	  gui->WatcherTable[a] = 0;
	  Remove_section_from_file( watcher->Name );
	  removed = watcher;
	  move = a + 1;
	}
	else {
	  ERR_APP( "%s", app->Strs[202] ); /* "Watcher cannot be removed while version update is in progress" */
	  return;
	}
      }
    }
  }
  --gui->WatcherCount;
  if( a )
    gui->WatcherTable[a - 1] = 0;

  ASSERT( !removed, "removed = 0" );
  Free_struct_watcher( removed, 1, 0 );

  /* Updating related things */
  Update_main_geometry();

  count = 0;
  for( a = 0; a < gui->WatcherCount; ++a )
    if( !( gui->WatcherTable[a]->Flags & WTR_FLG_DISABLE ))
      gui->WatcherTable[a]->VisiblePos = count++;

  if( ! count )
    gui->Flags |= GUI_FLG_NO_WATCHER;
  else
    gui->Flags &= ~( GUI_FLG_NO_WATCHER );
}

/*
  Purpose: Create the watcher list tab of the settings window
*/
void                  Create_preferences_tab_watchers(GtkWidget *tab)
{
  GtkWidget           *vbox, *vbox_main, *hbox, *lview, *item;
  GtkTreeViewColumn   *col;

  vbox_main = APP_GTK_VBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(tab), vbox_main );
  gtk_notebook_set_tab_label_text( GTK_NOTEBOOK(tab), vbox_main, app->Strs[28] ); /* "Watchers" */

  /* ----- Watchers list */
  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_box_pack_start( GTK_BOX(vbox_main), hbox, TRUE, TRUE, 0 );

  vbox = gtk_scrolled_window_new( NULL, NULL );
  gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(vbox), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
  gtk_box_pack_start( GTK_BOX(hbox), vbox, TRUE, TRUE, 0 );

  app->Gui.ListWatchers = gtk_list_store_new( 3, G_TYPE_BOOLEAN, GDK_TYPE_PIXBUF, G_TYPE_STRING );

  lview = gtk_tree_view_new_with_model( GTK_TREE_MODEL(app->Gui.ListWatchers) );
  gtk_container_add( GTK_CONTAINER(vbox), lview );

  app->Gui.SelectionWatcher = gtk_tree_view_get_selection( GTK_TREE_VIEW(lview) );
  g_signal_connect( G_OBJECT(app->Gui.SelectionWatcher), "changed", G_CALLBACK(Callback_select_changed), &app->Gui );

  app->CellRender[COL_LOGO] = gtk_cell_renderer_pixbuf_new();
  app->CellRender[COL_NAME] = gtk_cell_renderer_text_new();
  app->CellRender[COL_STATE] = gtk_cell_renderer_toggle_new();
  g_signal_connect( G_OBJECT(app->CellRender[COL_STATE]), "toggled", G_CALLBACK(Callback_toggle), &app->Gui );

  /* ----- Enable column */
  col = gtk_tree_view_column_new_with_attributes( app->Strs[32], app->CellRender[COL_STATE], "active", COL_STATE, NULL ); /* "Enable" */
  gtk_tree_view_append_column( GTK_TREE_VIEW(lview), col );

  /* ----- Logo column */
  col = gtk_tree_view_column_new_with_attributes( app->Strs[47], app->CellRender[COL_LOGO], "pixbuf", COL_LOGO, NULL ); /* "Logo" */
  gtk_tree_view_append_column( GTK_TREE_VIEW(lview), col );

  /* ----- Name column */
  col = gtk_tree_view_column_new_with_attributes( app->Strs[31], app->CellRender[COL_NAME], "text", COL_NAME, NULL ); /* "Name" */
  gtk_tree_view_column_set_clickable( col, TRUE );
  gtk_tree_view_append_column( GTK_TREE_VIEW(lview), col );

  /* ---- Bottom buttons ------------------------------------------------------------------- */
  vbox = APP_GTK_VBOX( FALSE, 0 );
  gtk_box_pack_start( GTK_BOX(hbox), vbox, FALSE, FALSE, 0 );

  /* Add watcher */
  item = gtk_button_new_with_mnemonic( app->Strs[33] ); /* "_Add" */
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_add_watcher), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_ADD] = item;

  /* Edit watcher */
  item = gtk_button_new_with_mnemonic( app->Strs[34] ); /* "_Edit" */
  gtk_widget_set_sensitive( item, 0 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_edit_watcher), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_EDIT] = item;

  /* Move up watcher */
  item = gtk_button_new_with_mnemonic( app->Strs[48] ); /* "Move up" */
  gtk_widget_set_sensitive( item, 0 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_move_up), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_MOVE_UP] = item;

  /* Move down watcher */
  item = gtk_button_new_with_mnemonic( app->Strs[49] ); /* "Move down " */
  gtk_widget_set_sensitive( item, 0 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_move_down), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_MOVE_DOWN] = item;

  /* Separator */
  item = APP_GTK_HSEPAR();
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  item = APP_GTK_HSEPAR();
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );

  /* Remove watcher */
  item = gtk_button_new_with_mnemonic( app->Strs[35] ); /* "Remove" */
  gtk_widget_set_sensitive( item, 0 );
  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(Callback_remove_watcher), &app->Gui );
  gtk_box_pack_start( GTK_BOX(vbox), item, FALSE, FALSE, 0 );
  app->Gui.PreferenceSettings[PS_BUT_REMOVE] = item;
}
