/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2015-2016  Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

/*
  Purpose: Moving manually a folder (only files inside)
*/
static void         Move_folder(const char *source, const char *destination)
{
  struct dirent     *dp;
  DIR               *dirp;
  char              *tmp;
  s_dest            dest;
  unsigned int      len_dest, len_file;

  if( ! chdir( source )) {
    ERR_SYS( "%s: chdir( %s )", app->Strs[157], app->CacheFolder ); /* "Unable to move folder" */
    return;
  }

  if( !( dirp = opendir( "." ))) {
    ERR_SYS( "%s: opendir( %s )", app->Strs[157], app->CacheFolder ); /* "Unable to move folder" */
    return;
  }

  len_dest = strlen( destination );
  dest.Length = len_dest + 1; /* '/' */
  dest.Pos = dest.Length;
  if( !( dest.Mem = malloc( dest.Length ))) {
    ERR_SYS( "%s", app->Strs[1] ); /* "Couldn't allocate enough memory" */
    goto error;
  }

  memcpy( dest.Mem, destination, len_dest );
  dest.Mem[ len_dest ] = '/';
  while(( dp = readdir( dirp )) > 0 ) {
    if( dp->d_type == DT_REG ) {
      len_file = strlen( dp->d_name );
      if( dest.Pos + len_file + 1 >= dest.Length ) {
	dest.Length = len_dest + len_file + 2; /* '/' + '\0' */
	if(( tmp = realloc( dest.Mem, dest.Length )) == NULL ) {
	  ERR_SYS( "%s", app->Strs[1] ); /* "Couldn't allocate enough memory" */
	  free( dest.Mem );
	  goto error;
	}
	dest.Mem = tmp;
      }
      memcpy( &dest.Mem[dest.Pos], dp->d_name, len_file );
      dest.Mem[dest.Pos + len_file] = '\0';
      if( ! Copy_file( dp->d_name, dest.Mem ))
	unlink( dp->d_name );
    }
  }

 error:
  closedir( dirp );
}

/*
  Purpose: Moving the cache folder, fallback is creating a folder, copying file and removing folder
*/
static void         Change_cache_folder(const char *new_folder)
{
  char              *tmp_ety;
  int               save_errno;

  if(( tmp_ety = strdup( new_folder )) == NULL ) {
    ERR_SYS( "%s", app->Strs[1] ); /* "Couldn't allocate enough memory" */
    return;
  }

  LOCK_MUTEX( app->LockCache );
  if( rename( app->CacheFolder, tmp_ety )) {
    if( errno == EXDEV /* oldpath and newpath are not on the same mounted filesystem  */
	|| errno == ENOENT ) { /* The link named by oldpath does not exist, or a directory component in newpath does not exist */
      save_errno = errno;
      if( Try_create_folder( tmp_ety, strlen( tmp_ety ))) {
	ERR_SYS( "cache_folder: %s: '%s'", app->Strs[155], tmp_ety ); /* "Unable to create folder" */
	free( tmp_ety );
	return;
      }
      if( save_errno == ENOENT ) {
	rename( app->CacheFolder, tmp_ety );
	//#
      }
      else {
	Move_folder( app->CacheFolder, tmp_ety );

	/* If folder is empty after moving all files, it will delete else not */
	if( ! rmdir( app->CacheFolder )) {
	  WARN_APP( "rmdir( %s ): %s", app->CacheFolder, app->Strs[159] ); /* "Unable to remove old folder" */
	}
      }
    }
    else {
      ERR_SYS( "rename()" );
      ERR_APP( "%s: '%s' -> '%s'", app->Strs[157], app->CacheFolder, tmp_ety ); /* "Unable to move folder" */
      free( tmp_ety );
      return;
    }
  }
  free( app->CacheFolder );
  app->CacheFolder = tmp_ety;
  UNLOCK_MUTEX( app->LockCache );
}

/*
  Purpose: Callback for the 'save' button
*/
static void         Callback_save_preferences(GtkWidget *unused1, gpointer unused2)
{
  const char        *entry;
  pthread_t         tmp_id;
  unsigned int      a;

  gtk_widget_hide( app->Gui.PreferenceSettings[PS_WIN_MAIN] );
  if( app->Gui.HoveredWatcher ) {
    gtk_widget_hide( app->Gui.HoveredWatcher->InfosWindow );
    app->Gui.HoveredWatcher = 0;
  }

  if( !( app->Gui.Flags & GUI_FLG_ON_TOP )) {
    gtk_window_set_keep_above( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 0 );
    gtk_window_set_keep_below( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 1 );
  }

  if( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_MAIN_COLOR ) {
    if( app->Gui.CancelPref.MainColor.red != app->Gui.MainColor.red
	|| app->Gui.CancelPref.MainColor.green != app->Gui.MainColor.green
	|| app->Gui.CancelPref.MainColor.blue != app->Gui.MainColor.blue
#if GTK_MAJOR_VERSION >= 3
	|| app->Gui.CancelPref.MainColor.alpha != app->Gui.MainColor.alpha ) {
#else
	|| app->Gui.CancelPref.MainOpacity != app->Gui.MainOpacity ) {
#endif
      Callback_draw_main( app->Gui.Main[MW_WIN_MAIN], 0, 0 );
      for( a = 0; a < app->Gui.WatcherCount; ++a )
	if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE ))
	  Callback_on_expose_watcher( app->Gui.WatcherTable[a]->DrawArea, 0, app->Gui.WatcherTable[a] );
    }
    else
      app->Gui.CancelPref.Flags &= ~( GUI_CANCEL_FLG_MAIN_COLOR );
  }

  /* Cache folder -------------- */
  if( gtk_entry_get_text_length( GTK_ENTRY(app->Gui.PreferenceSettings[PS_ETY_CACHE]) )
      && ( entry = gtk_entry_get_text( GTK_ENTRY(app->Gui.PreferenceSettings[PS_ETY_CACHE]) ))) {
    if( strcmp( entry, app->CacheFolder ))
      Change_cache_folder( entry );
  }
  else
    gtk_entry_set_text( GTK_ENTRY(app->Gui.PreferenceSettings[PS_ETY_CACHE]), app->CacheFolder );

  /* Saving to config file */
  Search_section2( &app->Gui.File, &app->Gui.File.SectionTarget, "preferences" );

  Generate_preferences_section( &app->Gui, &app->Gui.File.SectionReplace );

  if( pthread_create( &tmp_id, &app->Attr, th_Save_modifications_to_file, &app->Gui.File )) {
    ERR_APP( "%s", app->Strs[179] ); /* "Unable to create a thread" */
    th_Save_modifications_to_file( &app->Gui.File );
  }
  gtk_widget_hide( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
}

/*
  Purpose: Updating geometry of main window, and positions of watchers into it
*/
void                Update_main_geometry()
{
  s_wtr             *watcher;
  unsigned int      a, x, y, count;

  count = 0;
  if( app->Gui.Flags & GUI_FLG_VERTICAL_EXPAND ) {
    for( a = 0; a < app->Gui.WatcherCount; ++a ) {
      watcher = app->Gui.WatcherTable[a];
      if( watcher->Flags & WTR_FLG_ATTACHED ) {
	gtk_container_remove( GTK_CONTAINER(app->Gui.Main[MW_TABLE_MAIN]), watcher->DrawArea );
	watcher->Flags &= ~( WTR_FLG_ATTACHED );
      }
      if( !( watcher->Flags & WTR_FLG_DISABLE )) {
#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 4
	gtk_grid_attach( GTK_GRID(app->Gui.Main[MW_TABLE_MAIN]), watcher->DrawArea,
			 count / app->Gui.MaxYWatcher, count % app->Gui.MaxYWatcher,
			 1, 1 );
#else
	gtk_table_attach_defaults( GTK_TABLE(app->Gui.Main[MW_TABLE_MAIN]), watcher->DrawArea,
				   count / app->Gui.MaxYWatcher, ( count / app->Gui.MaxYWatcher ) + 1,
				   count % app->Gui.MaxYWatcher, ( count % app->Gui.MaxYWatcher ) + 1 );
#endif
	watcher->Flags |= WTR_FLG_ATTACHED;
	++count;
      }
    }

    if( count > app->Gui.MaxYWatcher )
      y = app->Gui.MaxYWatcher * ( app->Gui.WatcherSizeY + 2 );
    else
      y = count * ( app->Gui.WatcherSizeY + 2 );
    if( !count )
      ++count;
    x = ( app->Gui.WatcherSizeX + 2 ) * ((( count - 1 ) / app->Gui.MaxYWatcher ) + 1 );
  }
  else {
    for( a = 0; a < app->Gui.WatcherCount; ++a ) {
      watcher = app->Gui.WatcherTable[a];
      if( watcher->Flags & WTR_FLG_ATTACHED ) {
	gtk_container_remove( GTK_CONTAINER(app->Gui.Main[MW_TABLE_MAIN]), watcher->DrawArea );
	watcher->Flags &= ~( WTR_FLG_ATTACHED );
      }
      if( !( watcher->Flags & WTR_FLG_DISABLE )) {
#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 4
	gtk_grid_attach( GTK_GRID(app->Gui.Main[MW_TABLE_MAIN]), watcher->DrawArea,
			 count % app->Gui.MaxXWatcher, count / app->Gui.MaxXWatcher,
			 1, 1 );
#else
	gtk_table_attach_defaults( GTK_TABLE(app->Gui.Main[MW_TABLE_MAIN]), watcher->DrawArea,
				   count % app->Gui.MaxXWatcher, ( count % app->Gui.MaxXWatcher ) + 1,
				   count / app->Gui.MaxXWatcher, ( count / app->Gui.MaxXWatcher ) + 1 );
#endif
	watcher->Flags |= WTR_FLG_ATTACHED;
	++count;
      }
    }

    if( count > app->Gui.MaxXWatcher )
      x = app->Gui.MaxXWatcher * ( app->Gui.WatcherSizeX + 2 );
    else
      x = count * ( app->Gui.WatcherSizeX + 2 );
    if( !count )
      ++count;
    y = ( app->Gui.WatcherSizeY + 2 ) * ((( count - 1 ) / app->Gui.MaxXWatcher ) + 1 );
  }

  if( x < app->Gui.WatcherSizeX )
    x = app->Gui.WatcherSizeX + 2;
  else if( x > ( app->Gui.WatcherSizeX + 2 ) * app->Gui.MaxXWatcher )
    x = ( app->Gui.WatcherSizeX + 2 ) * app->Gui.MaxXWatcher;

  if( y < app->Gui.WatcherSizeY )
    y = app->Gui.WatcherSizeY + 2;
  else if( y > ( app->Gui.WatcherSizeY + 2 ) * app->Gui.MaxYWatcher )
    y = ( app->Gui.WatcherSizeY + 2 ) * app->Gui.MaxYWatcher;

  gtk_widget_show_all( app->Gui.Main[MW_TABLE_MAIN] );
  if( app->Flags & APP_FLG_PHONE ) {
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(app->Gui.PreferenceSettings[PS_SPIN_MAX_WIDTH]), app->Gui.MaxXWatcher );
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(app->Gui.PreferenceSettings[PS_SPIN_MAX_HEIGHT]), app->Gui.MaxYWatcher );
  }
  else
    gtk_widget_set_size_request( app->Gui.Main[MW_WIN_MAIN], x - 2, y - 2 );
}

/*
  Purpose: Callback for spin widget to set the maximal width
*/
static void         Callback_spin_width(GtkWidget *widget, gpointer data)
{
  if( ! app->Gui.CancelPref.MaxXWatcher ) {
    app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_PREFERENCES;
    app->Gui.CancelPref.MaxXWatcher = app->Gui.MaxXWatcher;
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  }
  app->Gui.MaxXWatcher = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(widget) );
#if GTK_MAJOR_VERSION < 2 || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 4 )
  gtk_table_resize( GTK_TABLE(app->Gui.Main[MW_TABLE_MAIN]), app->Gui.MaxXWatcher, app->Gui.MaxYWatcher );
#endif
  Update_main_geometry();
}

/*
  Purpose: Callback for spin widget to set the maximal height
*/
static void         Callback_spin_height(GtkWidget *widget, gpointer data)
{
  if( ! app->Gui.CancelPref.MaxYWatcher ) {
    app->Gui.CancelPref.MaxYWatcher = app->Gui.MaxYWatcher;
    app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_PREFERENCES;
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  }
  app->Gui.MaxYWatcher = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(widget) );
  gtk_widget_set_size_request( app->Gui.Main[MW_WIN_MAIN], app->Gui.MaxXWatcher * ( app->Gui.WatcherSizeX + 2 ) - 2, app->Gui.MaxYWatcher * ( app->Gui.WatcherSizeY + 2 ) - 2 );
  Update_main_geometry();
}

/*
  Purpose: Callback for checkbox 'vertical expand'
*/
static gboolean     Callback_state_vexpand(GtkWidget *widget, gboolean state, gpointer data)
{
  if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_V_EXPAND )) {
    app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_V_EXPAND | GUI_CANCEL_FLG_PREFERENCES;
    app->Gui.CancelPref.vexpand = ( state == 0 );
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  }

  if( state )
    app->Gui.Flags |= GUI_FLG_VERTICAL_EXPAND;
  else
    app->Gui.Flags &= ~( GUI_FLG_VERTICAL_EXPAND );

  Update_main_geometry();

  return( FALSE );
}

#if GTK_MAJOR_VERSION < 3
static void         Callback_check_vexpand(GtkWidget *widget, gpointer data)
{
  gboolean          is_active;

  is_active = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget));

  Callback_state_vexpand( widget, is_active, data );
}
#endif

static gboolean     Callback_toggle_lock(GtkWidget *widget, gboolean state, gpointer data)
{
  if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_LOCK_MAIN )) {
    app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_LOCK_MAIN | GUI_CANCEL_FLG_PREFERENCES;
    app->Gui.CancelPref.LockPosition = ( state == 0 );
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  }
  if( state )
    app->Gui.Flags |= GUI_FLG_LOCK_MAIN;
  else
    app->Gui.Flags &= ~( GUI_FLG_LOCK_MAIN );
  return( FALSE );
}

#if GTK_MAJOR_VERSION < 3
static void         Callback_check_lock(GtkWidget *widget, gpointer data)
{
  gboolean          is_active;

  is_active = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget));

  Callback_toggle_lock( widget, is_active, data );
}
#endif

static void         Callback_toggle_systray(GtkWidget *widget, gboolean state, gpointer data)
{
  if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_SYSTRAY )) {
    app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_SYSTRAY | GUI_CANCEL_FLG_PREFERENCES;
    app->Gui.CancelPref.Systray = ( app->Flags & APP_FLG_SYSTRAY ) != 0;
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  }

  if( state ) {
    app->Flags |= APP_FLG_SYSTRAY;
    gtk_status_icon_set_visible( app->Systray, 1 );
  }
  else {
    app->Flags &= ~( APP_FLG_SYSTRAY );
    gtk_status_icon_set_visible( app->Systray, 0 );
  }
}

#if GTK_MAJOR_VERSION < 3
static void         Callback_check_systray(GtkWidget *widget, gpointer data)
{
  gboolean          is_active;

  is_active = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_TOGGLE_SYSTRAY]) );

  Callback_toggle_systray( widget, is_active, data );
}
#endif

static gboolean      Callback_entry_cache(GtkEntry *entry, GdkEventKey *event, gpointer data)
{
  if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_CACHE )) {
    app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_CACHE | GUI_CANCEL_FLG_PREFERENCES;
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  }
  return( FALSE );
}

static void          Refresh_preferences()
{
  /* Main window part */
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(app->Gui.PreferenceSettings[PS_SPIN_MAX_WIDTH]), app->Gui.MaxXWatcher );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(app->Gui.PreferenceSettings[PS_SPIN_MAX_HEIGHT]), app->Gui.MaxYWatcher );

  APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.PreferenceSettings[PS_TOGGLE_VEXPAND], ( app->Gui.Flags & GUI_FLG_VERTICAL_EXPAND ) != 0 );
  APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.PreferenceSettings[PS_CHK_LOCK_POS], ( app->Gui.Flags & GUI_FLG_LOCK_MAIN ) != 0 );

  if( app->Gui.Flags & GUI_FLG_ON_TOP )
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_ALWAYS_ON_TOP]), 1 );
  else
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_ALWAYS_ON_BOTTOM]), 1 );

#ifndef LIB_HANDY
#  if GTK_MAJOR_VERSION >= 3
  gtk_color_button_set_alpha( GTK_COLOR_BUTTON(app->Gui.PreferenceSettings[PS_BUT_COLOR]), app->Gui.MainColor.alpha * 65536.0 );
  GdkColor color;
  color.red = app->Gui.MainColor.red * 65536.0;
  color.green = app->Gui.MainColor.green * 65536.0;
  color.blue = app->Gui.MainColor.blue * 65536.0;
  gtk_color_button_set_color( GTK_COLOR_BUTTON(app->Gui.PreferenceSettings[PS_BUT_COLOR]), &color );
#  else
  gtk_color_button_set_alpha( GTK_COLOR_BUTTON(app->Gui.PreferenceSettings[PS_BUT_COLOR]), app->Gui.MainOpacity * 65536.0 );
  gtk_color_button_set_color( GTK_COLOR_BUTTON(app->Gui.PreferenceSettings[PS_BUT_COLOR]), &app->Gui.MainColor );
#  endif
#endif

  /* InfosView expansion part */
#ifdef LIB_HANDY
  switch( app->Gui.Flags & GUI_FLG_INFOS_MASK ) {
  case GUI_FLG_INFOS_TO_TOP: hdy_combo_row_set_selected_index( (HdyComboRow*) app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION], 1 ); break;
  case GUI_FLG_INFOS_TO_LEFT: hdy_combo_row_set_selected_index( (HdyComboRow*) app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION], 2 ); break;
  case GUI_FLG_INFOS_TO_RIGHT: hdy_combo_row_set_selected_index( (HdyComboRow*) app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION], 3 ); break;
  default: hdy_combo_row_set_selected_index( (HdyComboRow*) app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION], 0 ); break;
  }
#else
  switch( app->Gui.Flags & GUI_FLG_INFOS_MASK ) {
  case GUI_FLG_INFOS_TO_TOP: gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_TOP]), 1 ); break;
  case GUI_FLG_INFOS_TO_LEFT: gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_LEFT]), 1 ); break;
  case GUI_FLG_INFOS_TO_RIGHT: gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_RIGHT]), 1 ); break;
  default: gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_BOTTOM]), 1 ); break;
  }
#endif

  /* Others options part */
  if( app->Flags & APP_FLG_SYSTRAY ) {
    APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.PreferenceSettings[PS_TOGGLE_SYSTRAY], 1 );
    //#if GTK_MAJOR_VERSION == 2 || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 14 )
    gtk_status_icon_set_visible( app->Systray, 1 );
  }
  else {
    APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.PreferenceSettings[PS_TOGGLE_SYSTRAY], 0 );
    //#if GTK_MAJOR_VERSION == 2 || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 14 )
    gtk_status_icon_set_visible( app->Systray, 0 );
  }
  gtk_entry_set_text( GTK_ENTRY(app->Gui.PreferenceSettings[PS_ETY_CACHE]), app->CacheFolder );

  gtk_widget_hide( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  bzero( &app->Gui.CancelPref, sizeof(app->Gui.CancelPref) );

  gtk_widget_set_sensitive( app->Gui.WatcherSettings[WS_MENU_ITEM_TEMPLATE], (app->Gui.File.TemplateCount != 0) );
}

/*
  Purpose: Handeling the callback of the button 'Reload configuration' in the preferences window
*/
static gboolean      Callback_reload(GtkButton *button, s_gui *gui)
{
  s_tpl              *template;
  unsigned int       a;

  app->CountReload = gui->WatcherCount;
  gui->WatcherCount = 0;

  bzero( &app->Gui.CancelPref, sizeof(app->Gui.CancelPref) );
  MALLOC( app->ReloadTable, app->CountReload * sizeof(*app->ReloadTable) );

  gtk_widget_set_sensitive( gui->PreferenceSettings[PS_BUT_RELOAD], 0 );

  /* Cleaning watchers */
  for( a = 0; a < app->CountReload; ++a ) {
    app->ReloadTable[a] = gui->WatcherTable[a];
    //#exceptionnal thread race on gui->WatcherTable with th_Updater()
    gui->WatcherTable[a] = 0;
    if( !( app->ReloadTable[a]->Flags & WTR_FLG_DISABLE )) {
      gtk_container_remove( GTK_CONTAINER(gui->Main[MW_TABLE_MAIN]), app->ReloadTable[a]->DrawArea );
      app->ReloadTable[a]->Flags |= WTR_FLG_DISABLE;
    }
  }

  /* Cleaning templates */
  if( gui->File.TemplateCount ) {
    for( a = 0; a < gui->File.TemplateCount; ++a )
      Free_struct_template( gui->File.TemplateTable[a], 0 );
    free( gui->File.TemplateTable );
    gui->File.TemplateTable = 0;
    gui->File.TemplateCount = 0;
  }

  //#thread race on configfile
  /* Cleaning  file */
  free( gui->File.Mem );
  gui->File.Mem = 0;
  gui->File.Length = 0;

  Load_file_config( &gui->File, gui->File.Name );

  if( gui->Flags & GUI_FLG_ON_TOP ) {
    gtk_window_set_keep_below( GTK_WINDOW(gui->Main[MW_WIN_MAIN]), 0 );
    gtk_window_set_keep_above( GTK_WINDOW(gui->Main[MW_WIN_MAIN]), 1 );
  }
  else {
    gtk_window_set_keep_above( GTK_WINDOW(gui->Main[MW_WIN_MAIN]), 0 );
    gtk_window_set_keep_below( GTK_WINDOW(gui->Main[MW_WIN_MAIN]), 1 );
  }

  Refresh_preferences();
  Create_watchers();

  for( a = 0; a < app->CoreCount; ++a )
    app->ThreadFlags[a] |= THD_FLG_RELOAD;

  pthread_cond_broadcast( &app->TimerCond );

  if( a == app->CoreCount - 1 && gui->File.ErrorsCount ) {
    char tmp[4096];
    int length;

    length = snprintf( tmp, 4096, "  %s  \n  %s\n\n", app->Strs[218], gui->File.Name ); /* "Some errors have been found in the configuration file." */
    for( a = 0; a < gui->File.ErrorsCount; ++a )
      if( length > 0 && length < 4096 )
	length += snprintf( &tmp[length], 4096 - length, "- %s \n", gui->File.Error[a] );

    if( length > 0 ) {
      LOCK_GTK();
      gtk_label_set_label( GTK_LABEL(app->Error[ERW_LBL_MESSAGE]), tmp );
      gtk_widget_show( app->Error[ERW_WIN_MAIN] );
      UNLOCK_GTK();
    }
  }

  for( a = 0; a < gui->File.TemplateCount; ++a ) {
    template = gui->File.TemplateTable[a];
    gtk_list_store_append ( gui->ListTemplates, &template->Iter );
    gtk_list_store_set( gui->ListTemplates, &template->Iter, 0, template->Name, -1 );
    Add_template_to_menu( gui->WatcherSettings[WS_MENU_TEMPLATE], template );
  }
  gtk_widget_hide( gui->PreferenceSettings[PS_BUT_SAVE] );
  return( TRUE );
}

static void         Callback_radio_expand_to_bottom(GtkWidget *widget, gpointer data)
{
  if( !( app->Flags & APP_FLG_INIT ))
    if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_INFOS_VIEW )) {
      app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_INFOS_VIEW | GUI_CANCEL_FLG_PREFERENCES;
      if( app->Gui.Flags & GUI_FLG_INFOS_TO_TOP )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_TOP;
      else if( app->Gui.Flags & GUI_FLG_INFOS_TO_LEFT )
      app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_LEFT;
      else if( app->Gui.Flags & GUI_FLG_INFOS_TO_RIGHT )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_RIGHT;
      else
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_BOTTOM;
      gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
    }

  app->Gui.Flags &= ~( GUI_FLG_INFOS_MASK );
  if( !( app->Flags & APP_FLG_INIT )) {
    if( !app->Gui.HoveredWatcher ) {
      unsigned int a;
      for( a = 0; a < app->Gui.WatcherCount; ++a )
	if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE )) {
	  app->Gui.HoveredWatcher = app->Gui.WatcherTable[a];
	  break;
	}
    }
    if( app->Gui.HoveredWatcher ) {
      Update_info_position( app->Gui.HoveredWatcher );
      gtk_widget_show( app->Gui.HoveredWatcher->InfosWindow );
    }
  }
}

static void         Callback_radio_expand_to_top(GtkWidget *widget, gpointer data)
{
  unsigned int      a;

  if( !( app->Flags & APP_FLG_INIT ))
    if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_INFOS_VIEW )) {
      app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_INFOS_VIEW | GUI_CANCEL_FLG_PREFERENCES;
      if( app->Gui.Flags & GUI_FLG_INFOS_TO_TOP )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_TOP;
      else if( app->Gui.Flags & GUI_FLG_INFOS_TO_LEFT )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_LEFT;
      else if( app->Gui.Flags & GUI_FLG_INFOS_TO_RIGHT )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_RIGHT;
      else
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_BOTTOM;
      gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
    }

  app->Gui.Flags &= ~( GUI_FLG_INFOS_MASK );
  app->Gui.Flags |= GUI_FLG_INFOS_TO_TOP;
  if( !( app->Flags & APP_FLG_INIT )) {
    if( !app->Gui.HoveredWatcher ) {
      for( a = 0; a < app->Gui.WatcherCount; ++a )
	if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE )) {
	  app->Gui.HoveredWatcher = app->Gui.WatcherTable[a];
	  break;
	}
    }
    if( app->Gui.HoveredWatcher ) {
      Update_info_position( app->Gui.HoveredWatcher );
      gtk_widget_show( app->Gui.HoveredWatcher->InfosWindow );
    }
  }
}

static void         Callback_radio_expand_to_left(GtkWidget *widget, gpointer data)
{
  unsigned int      a;

  if( !( app->Flags & APP_FLG_INIT ))
    if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_INFOS_VIEW )) {
      app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_INFOS_VIEW | GUI_CANCEL_FLG_PREFERENCES;
      if( app->Gui.Flags & GUI_FLG_INFOS_TO_TOP )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_TOP;
      else if( app->Gui.Flags & GUI_FLG_INFOS_TO_LEFT )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_LEFT;
      else if( app->Gui.Flags & GUI_FLG_INFOS_TO_RIGHT )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_RIGHT;
      else
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_BOTTOM;
      gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
    }

  app->Gui.Flags &= ~( GUI_FLG_INFOS_MASK );
  app->Gui.Flags |= GUI_FLG_INFOS_TO_LEFT;
  if( !( app->Flags & APP_FLG_INIT )) {
    if( ! app->Gui.HoveredWatcher ) {
      for( a = 0; a < app->Gui.WatcherCount; ++a )
	if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE )) {
	  app->Gui.HoveredWatcher = app->Gui.WatcherTable[a];
	  break;
	}
    }
    if( app->Gui.HoveredWatcher ) {
      Update_info_position( app->Gui.HoveredWatcher );
      gtk_widget_show( app->Gui.HoveredWatcher->InfosWindow );
    }
  }
}

static void         Callback_radio_expand_to_right(GtkWidget *widget, gpointer data)
{
  unsigned int      a;

  if( !( app->Flags & APP_FLG_INIT ))
    if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_INFOS_VIEW )) {
      app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_INFOS_VIEW | GUI_CANCEL_FLG_PREFERENCES;
      if( app->Gui.Flags & GUI_FLG_INFOS_TO_TOP )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_TOP;
      else if( app->Gui.Flags & GUI_FLG_INFOS_TO_LEFT )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_LEFT;
      else if( app->Gui.Flags & GUI_FLG_INFOS_TO_RIGHT )
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_RIGHT;
      else
	app->Gui.CancelPref.InfosView = GUI_CANCEL_INFOS_TO_BOTTOM;
      gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
    }

  app->Gui.Flags &= ~( GUI_FLG_INFOS_MASK );
  app->Gui.Flags |= GUI_FLG_INFOS_TO_RIGHT;
  if( !( app->Flags & APP_FLG_INIT )) {
    if( ! app->Gui.HoveredWatcher )
      for( a = 0; a < app->Gui.WatcherCount; ++a )
	if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE )) {
	  app->Gui.HoveredWatcher = app->Gui.WatcherTable[a];
	  break;
	}
    if( app->Gui.HoveredWatcher ) {
      Update_info_position( app->Gui.HoveredWatcher );
      gtk_widget_show( app->Gui.HoveredWatcher->InfosWindow );
    }
  }
}

static void         Callback_radio_always_on_top(GtkWidget *widget, gpointer data)
{
  if( !( app->Gui.Flags & GUI_FLG_ON_TOP )) {
    if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_ALWAYS )) {
      app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_ALWAYS | GUI_CANCEL_FLG_PREFERENCES;
      app->Gui.CancelPref.Always = 0;
      gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
    }
    app->Gui.Flags |= GUI_FLG_ON_TOP;
    gtk_window_set_keep_below( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 0 );
    gtk_window_set_keep_above( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 1 );
  }
}

static void         Callback_radio_always_on_bottom(GtkWidget *widget, gpointer data)
{
  if( app->Gui.Flags & GUI_FLG_ON_TOP ) {
    if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_ALWAYS )) {
      app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_ALWAYS | GUI_CANCEL_FLG_PREFERENCES;
      app->Gui.CancelPref.Always = 1;
      gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
    }
    app->Gui.Flags &= ~( GUI_FLG_ON_TOP );
    gtk_window_set_keep_above( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 0 );
    gtk_window_set_keep_below( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 1 );
  }
}

static void         Update_watchers_geometry()
{
  s_wtr             *watcher;
  GdkPixbuf         *tmp_pix;
  gint              curr_w, curr_h;
  int               a;

  if( app->Flags & APP_FLG_PHONE ) {
    SCREEN_WIDTH_HEIGHT( curr_w, curr_h );
    app->Gui.MaxXWatcher = curr_w / app->Gui.WatcherSizeX;
    app->Gui.MaxYWatcher = curr_h / app->Gui.WatcherSizeY;
  }

  for( a = 0 ; a < app->Gui.WatcherCount; ++a ) {
    watcher = app->Gui.WatcherTable[a];
    gtk_widget_set_size_request( watcher->DrawArea, app->Gui.WatcherSizeX, app->Gui.WatcherSizeY );

    tmp_pix = gdk_pixbuf_scale_simple( watcher->OriginPixbuf, app->Gui.WatcherSizeX, app->Gui.WatcherSizeY, GDK_INTERP_BILINEAR );
    if( !tmp_pix ) {
      ERR_APP( "%s: %s: %s", watcher->Name, app->Strs[228], watcher->ImgPath ); /* "Line" "Unable to resize image" */
    }
    else {
      g_object_unref( watcher->CurrentPixbuf );
      watcher->CurrentPixbuf = tmp_pix;
    }
  }
}

static void         Callback_cancel(GtkWidget *widget, gpointer data)
{
  unsigned int      a;

  if( app->Gui.CancelPref.MaxXWatcher )
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(app->Gui.PreferenceSettings[PS_SPIN_MAX_WIDTH]), app->Gui.CancelPref.MaxXWatcher );

  if( app->Gui.CancelPref.SizeIcon )
    gtk_range_set_value( GTK_RANGE(app->Gui.PreferenceSettings[PS_SCALE_ICON]), app->Gui.CancelPref.SizeIcon );

  if( app->Gui.CancelPref.MaxYWatcher )
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(app->Gui.PreferenceSettings[PS_SPIN_MAX_HEIGHT]), app->Gui.CancelPref.MaxYWatcher );

  if( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_V_EXPAND ) {
    APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.PreferenceSettings[PS_TOGGLE_VEXPAND], app->Gui.CancelPref.vexpand );
  }

  if( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_INFOS_VIEW ) {
    app->Gui.Flags &= ~( GUI_FLG_INFOS_MASK );
    switch( app->Gui.CancelPref.InfosView ) {
    case GUI_CANCEL_INFOS_TO_BOTTOM:
#ifdef LIB_HANDY
      hdy_combo_row_set_selected_index( (HdyComboRow*) app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION], 0 );
#else
      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_BOTTOM]), 1 );
#endif
      break;

    case GUI_CANCEL_INFOS_TO_TOP: app->Gui.Flags |= GUI_FLG_INFOS_TO_TOP;
#ifdef LIB_HANDY
      hdy_combo_row_set_selected_index( (HdyComboRow*) app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION], 1 );
#else
      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_TOP]), 1 );
#endif
      break;

    case GUI_CANCEL_INFOS_TO_LEFT: app->Gui.Flags |= GUI_FLG_INFOS_TO_LEFT;
#ifdef LIB_HANDY
      hdy_combo_row_set_selected_index( (HdyComboRow*) app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION], 2 );
#else
      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_LEFT]), 1 );
#endif
      break;

    case GUI_CANCEL_INFOS_TO_RIGHT: app->Gui.Flags |= GUI_FLG_INFOS_TO_RIGHT;
#ifdef LIB_HANDY
      hdy_combo_row_set_selected_index( (HdyComboRow*) app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION], 3 );
#else
      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_RIGHT]), 1 );
#endif
      break;
    }
    if( app->Gui.HoveredWatcher ) {
      Update_info_position( app->Gui.HoveredWatcher );
      gtk_widget_hide( app->Gui.HoveredWatcher->InfosWindow );
      app->Gui.HoveredWatcher = 0;
    }
  }

  if( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_MAIN_COLOR ) {
    app->Gui.MainColor.red = app->Gui.CancelPref.MainColor.red;
    app->Gui.MainColor.green = app->Gui.CancelPref.MainColor.green;
    app->Gui.MainColor.blue = app->Gui.CancelPref.MainColor.blue;
#if GTK_MAJOR_VERSION >= 3
    app->Gui.MainColor.alpha = app->Gui.CancelPref.MainColor.alpha;
#else
    app->Gui.MainOpacity = app->Gui.CancelPref.MainOpacity;
#endif

    Callback_draw_main( app->Gui.Main[MW_WIN_MAIN], 0, 0 );
    for( a = 0; a < app->Gui.WatcherCount; ++a )
      if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE ))
	Callback_on_expose_watcher( app->Gui.WatcherTable[a]->DrawArea, 0, app->Gui.WatcherTable[a] );

#ifndef LIB_HANDY
  //#if GTK_MAJOR_VERSION == 2 || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 4 )
#  if GTK_MAJOR_VERSION >= 3
  gtk_color_button_set_alpha( GTK_COLOR_BUTTON(app->Gui.PreferenceSettings[PS_BUT_COLOR]), app->Gui.MainColor.alpha * 65536.0 );
  GdkColor color;
  color.red = app->Gui.MainColor.red * 65536.0;
  color.green = app->Gui.MainColor.green * 65536.0;
  color.blue = app->Gui.MainColor.blue * 65536.0;
  gtk_color_button_set_color( GTK_COLOR_BUTTON(app->Gui.PreferenceSettings[PS_BUT_COLOR]), &color );
#  else
  gtk_color_button_set_alpha( GTK_COLOR_BUTTON(app->Gui.PreferenceSettings[PS_BUT_COLOR]), app->Gui.MainOpacity * 65536.0 );
  gtk_color_button_set_color( GTK_COLOR_BUTTON(app->Gui.PreferenceSettings[PS_BUT_COLOR]), &app->Gui.MainColor );
#  endif
#endif
  }

  if( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_LOCK_MAIN )
    APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.PreferenceSettings[PS_CHK_LOCK_POS], app->Gui.CancelPref.LockPosition );

  if( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_SYSTRAY )
    APP_GTK_TOGGLE_SET_ACTIVE( app->Gui.PreferenceSettings[PS_TOGGLE_SYSTRAY], app->Gui.CancelPref.Systray );

  if( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_CACHE )
    gtk_entry_set_text( GTK_ENTRY(app->Gui.PreferenceSettings[PS_ETY_CACHE]), app->CacheFolder );

  if( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_ALWAYS ) {
    if( app->Gui.CancelPref.Always ) {
      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_ALWAYS_ON_TOP]), 1 );
      gtk_window_set_keep_below( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 0 );
      gtk_window_set_keep_above( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 1 );
    }
    else {
      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_ALWAYS_ON_BOTTOM]), 1 );
      gtk_window_set_keep_above( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 0 );
      gtk_window_set_keep_below( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 1 );
    }
  }

  if( !( app->Gui.Flags & GUI_FLG_ON_TOP )) {
    gtk_window_set_keep_above( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 0 );
    gtk_window_set_keep_below( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), 1 );
  }
  gtk_widget_hide( app->Gui.PreferenceSettings[PS_WIN_MAIN] );
  gtk_widget_hide( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  bzero( &app->Gui.CancelPref, sizeof(app->Gui.CancelPref) );
}

static gboolean     Callback_on_tab_change(GtkWidget *widget, gint arg1, gpointer data)
{
  if( app->Gui.HoveredWatcher ) {
    Update_info_position( app->Gui.HoveredWatcher );
    gtk_widget_hide( app->Gui.HoveredWatcher->InfosWindow );
    app->Gui.HoveredWatcher = 0;
  }
  return( TRUE );
}

static gboolean     Callback_on_delete_cancel(GtkWidget *widget, gpointer window)
{
  Callback_cancel( 0, 0 );
  gtk_widget_hide( widget );
  return( TRUE );
}

static void         Create_option_reload_config(GtkWidget *parent)
{
  GtkWidget         *item;
  const char        *label;

  label = app->Strs[180]; /* "Reload configuration" */

#ifdef LIB_HANDY
  GtkWidget         *img, *row;

  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, label );
  gtk_container_add( GTK_CONTAINER(parent), row );

  img = gtk_image_new();
  gtk_image_set_from_pixbuf( GTK_IMAGE(img), app->Gui.ImgTable[IMG_REFRESH] );

  item = gtk_button_new();
  g_object_set( item, "valign", GTK_ALIGN_CENTER, NULL );
  gtk_widget_set_size_request( item, 38, 38 );
  gtk_button_set_image( (GtkButton*) item, img );
  gtk_container_add( GTK_CONTAINER(row), item );
#else
  item = gtk_button_new_with_label( label );
  gtk_box_pack_start( GTK_BOX(parent), item, TRUE, FALSE, 0 );
#endif

  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_reload), &app->Gui );
  app->Gui.PreferenceSettings[PS_BUT_RELOAD] = item;
}

static void         Create_option_systray_icon(GtkWidget *parent)
{
  GtkWidget         *row, *item;
  const char        *label, *tooltip;

  label = app->Strs[38]; /* "Show systray icon" */
  tooltip = app->Strs[92]; /* "systray: show the software icon in the systray system" */

#ifdef LIB_HANDY
  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, label );
  if( !( app->Flags & APP_FLG_PHONE )) {
    hdy_action_row_set_subtitle( (HdyActionRow*) row, tooltip );
    g_object_set( row, "subtitle-lines", 2, NULL );
  }
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_switch_new();
  hdy_action_row_set_activatable_widget( (HdyActionRow*) row, item );
#else
  row = APP_GTK_HBOX( FALSE, 0 );
  gtk_box_pack_start( GTK_BOX(parent), row, TRUE, FALSE, 0 );

# if GTK_MAJOR_VERSION >= 3
  item = gtk_label_new( label );
  gtk_widget_set_tooltip_text( item, tooltip );
  gtk_container_add( GTK_CONTAINER(row), item );

  item = gtk_switch_new();
# else
  item = gtk_check_button_new_with_label( label );
  gtk_widget_set_tooltip_text( item, tooltip );
# endif
#endif

  gtk_container_add( GTK_CONTAINER(row), item );
  if( app->Flags & APP_FLG_SYSTRAY )
    APP_GTK_TOGGLE_SET_ACTIVE( item, 1 );

  app->Gui.PreferenceSettings[PS_HBOX_SYSTRAY] = row;
  app->Gui.PreferenceSettings[PS_TOGGLE_SYSTRAY] = item;

#if GTK_MAJOR_VERSION >= 3
  gtk_widget_set_valign( item, GTK_ALIGN_CENTER );
  g_signal_connect( G_OBJECT(item), "state-set", G_CALLBACK(Callback_toggle_systray), 0 );
#else
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_check_systray), 0 );
#endif
}

static void         Create_option_cache_folder(GtkWidget *parent)
{
  GtkWidget         *row, *item;
  const char        *label, *tooltip;

  label = app->Strs[152]; /* " Cache folder " */
  tooltip = app->Strs[153]; /* "cache_folder: folder containing retrieved web resources" */

#ifdef LIB_HANDY
  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, label );
  if( !( app->Flags & APP_FLG_PHONE )) {
    hdy_action_row_set_subtitle( (HdyActionRow*) row, tooltip );
    g_object_set( row, "subtitle-lines", 2, NULL );
  }
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_entry_new();
  hdy_action_row_set_activatable_widget( (HdyActionRow*) row, item );
  gtk_container_add( GTK_CONTAINER(parent), item );
#else
  row = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_label_new( label );
  gtk_widget_set_tooltip_text( item, tooltip );
  gtk_box_pack_start( GTK_BOX(row), item, FALSE, FALSE, 0);

  item = gtk_entry_new();
  gtk_box_pack_start( GTK_BOX(row), item, TRUE, TRUE, 0 );
#endif

  gtk_entry_set_text( GTK_ENTRY(item), app->CacheFolder );
  gtk_widget_set_tooltip_text( item, tooltip );
  gtk_entry_set_max_length( GTK_ENTRY(item), 2048 );
  gtk_widget_add_events( item, GDK_KEY_PRESS_MASK );
  g_signal_connect( G_OBJECT(item), "key-press-event", G_CALLBACK(Callback_entry_cache), 0 );
  app->Gui.PreferenceSettings[PS_ETY_CACHE] = item;
}

static void         Create_preferences_others(GtkWidget *parent)
{
#ifdef LIB_HANDY
  Create_option_reload_config( parent );
  Create_option_systray_icon( parent );
  Create_option_cache_folder( parent );
#else
  GtkWidget         *frame, *hbox, *group;

  /* Frame ------------------------------------------------------------------ */
  frame = gtk_frame_new( app->Strs[37] ); /* "Other options" */
  gtk_box_pack_start( GTK_BOX(parent), frame, FALSE, FALSE, 0);

  group = APP_GTK_VBOX( TRUE, 0 );
  gtk_container_add( GTK_CONTAINER(frame), group );

  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(group), hbox );
  app->Gui.PreferenceSettings[PS_HBOX_SYSTRAY] = hbox;

  /* - button - reload conf */
  Create_option_reload_config( hbox );

  /* - toggle - systray icon */
  Create_option_systray_icon( hbox );

  /* - entry - cache */
  Create_option_cache_folder( group );
#endif

}

#ifdef LIB_HANDY
typedef struct _HdyValueObject
{
  GObject parent_instance;

  GValue value;
} HdyValueObject;

static gchar        *Callback_infosview_expansion(HdyValueObject *value, gpointer data)
{
  gchar             *str = g_value_dup_string( &value->value );

  if( app->Flags & APP_FLG_INIT )
    return( str );

  switch( hdy_combo_row_get_selected_index( data )) {
  case 0: Callback_radio_expand_to_bottom( 0, 0 ); break;
  case 1: Callback_radio_expand_to_top( 0, 0 ); break;
  case 2: Callback_radio_expand_to_right( 0, 0 ); break;
  case 3: Callback_radio_expand_to_left( 0, 0 ); break;
  }

  return( str );
}
#endif

static void         Create_preferences_infosview_expansion(GtkWidget *parent)
{
#ifdef LIB_HANDY
  GtkWidget         *row;
  GListStore        *list_store;
  HdyValueObject    *obj;

  row = hdy_combo_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, app->Strs[19] ); /* "InfosView expansion" */
  if( !( app->Flags & APP_FLG_PHONE )) {
    hdy_action_row_set_subtitle( (HdyActionRow*) row, app->Strs[29] ); /* "infosview_expansion: position of the information window, relative to the icon" */
    g_object_set( row, "subtitle-lines", 2, NULL );
  }
  gtk_container_add( GTK_CONTAINER(parent), row );
  app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION] = row;

  list_store = g_list_store_new (HDY_TYPE_VALUE_OBJECT);

  obj = hdy_value_object_new_string( app->Strs[20] ); /* "To bottom" */
  g_list_store_insert (list_store, 0, obj);
  g_clear_object (&obj);

  obj = hdy_value_object_new_string( app->Strs[21] ); /* "To top" */
  g_list_store_insert (list_store, 1, obj);
  g_clear_object (&obj);

  obj = hdy_value_object_new_string( app->Strs[22] ); /* "To left" */
  g_list_store_insert (list_store, 2, obj);
  g_clear_object (&obj);

  obj = hdy_value_object_new_string( app->Strs[23] ); /* "To right" */
  g_list_store_insert (list_store, 2, obj);
  g_clear_object (&obj);

  hdy_combo_row_bind_name_model( (HdyComboRow*) row, G_LIST_MODEL (list_store), (HdyComboRowGetNameFunc) Callback_infosview_expansion, row, NULL);

  if( app->Flags & APP_FLG_PHONE )
    gtk_widget_set_sensitive( row, 0 );
#else
  GtkWidget         *frame, *item, *hbox, *table, *vbox_item;
  GtkWidget         *vbox_bott, *vbox_top, *vbox_left, *vbox_right;

  /* Frame ---------------------------------------------------------------- */
  frame = gtk_frame_new( app->Strs[19] ); /* "InfosView expansion" */
  if( !( app->Flags & APP_FLG_PHONE ))
    gtk_box_pack_start( GTK_BOX(parent), frame, FALSE, TRUE, 0);
  app->Gui.PreferenceSettings[PS_COMBOROW_EXPANSION] = frame;

  /* Radio button ---------------------------------------------------------------- */
  /* To bottom */
  vbox_bott = APP_GTK_VBOX( FALSE, 0 );
  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(vbox_bott), hbox );

  vbox_item = APP_GTK_VBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(hbox), vbox_item );

  item = gtk_image_new_from_pixbuf( app->Gui.ImgTable[IMG_INFOS_BOTTOM] );
  gtk_box_pack_start( GTK_BOX(vbox_item), item, FALSE, TRUE, 0);

  item = gtk_radio_button_new_with_label_from_widget( NULL, app->Strs[20] ); /* "To bottom" */
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_radio_expand_to_bottom), 0 );
  gtk_widget_set_tooltip_text( item, app->Strs[93] ); /* "infosview_expansion: the informations view will be shown to the bottom of the watcher" */
  gtk_box_pack_start( GTK_BOX(vbox_item), item, FALSE, TRUE, 0);
  app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_BOTTOM] = item;

  /* To top */
  vbox_top = APP_GTK_VBOX( FALSE, 0 );
  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(vbox_top), hbox );

  vbox_item = APP_GTK_VBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(hbox), vbox_item );

  item = gtk_image_new_from_pixbuf( app->Gui.ImgTable[IMG_INFOS_TOP] );
  gtk_box_pack_start( GTK_BOX(vbox_item), item, FALSE, TRUE, 0);

  item = gtk_radio_button_new_with_label_from_widget( GTK_RADIO_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_BOTTOM]), app->Strs[21] ); /* "To top" */
  if( app->Gui.Flags & GUI_FLG_INFOS_TO_TOP )
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(item), 1 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_radio_expand_to_top), 0 );
  gtk_widget_set_tooltip_text( item, app->Strs[94] ); /* "infosview_expansion: the informations view will be shown to the top of the watcher" */
  gtk_box_pack_start( GTK_BOX(vbox_item), item, FALSE, TRUE, 0);
  app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_TOP] = item;

  /* To left */
  vbox_left = APP_GTK_VBOX( FALSE, 0 );
  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(vbox_left), hbox );

  vbox_item = APP_GTK_VBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(hbox), vbox_item );

  item = gtk_image_new_from_pixbuf( app->Gui.ImgTable[IMG_INFOS_LEFT] );
  gtk_box_pack_start( GTK_BOX(vbox_item), item, FALSE, TRUE, 0);

  item = gtk_radio_button_new_with_label_from_widget( GTK_RADIO_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_TOP]), app->Strs[22] ); /* "To left" */
  app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_LEFT] = item;
  if( app->Gui.Flags & GUI_FLG_INFOS_TO_LEFT )
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(item), 1 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_radio_expand_to_left), 0 );
  gtk_widget_set_tooltip_text( item, app->Strs[95] ); /* "infosview_expansion: the informations view will be shown to the left of the watcher" */
  gtk_box_pack_start( GTK_BOX(vbox_item), item, FALSE, TRUE, 0);

  /* To right */
  vbox_right = APP_GTK_VBOX( FALSE, 0 );
  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(vbox_right), hbox );

  vbox_item = APP_GTK_VBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(hbox), vbox_item );

  item = gtk_image_new_from_pixbuf( app->Gui.ImgTable[IMG_INFOS_RIGHT] );
  gtk_box_pack_start( GTK_BOX(vbox_item), item, FALSE, TRUE, 0);

  item = gtk_radio_button_new_with_label_from_widget( GTK_RADIO_BUTTON(app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_LEFT]), app->Strs[23] ); /* "To right" */
  app->Gui.PreferenceSettings[PS_RAD_INFOS_TO_RIGHT] = item;
  if( app->Gui.Flags & GUI_FLG_INFOS_TO_RIGHT )
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(item), 1 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_radio_expand_to_right), 0 );
  gtk_widget_set_tooltip_text( item, app->Strs[96] ); /* "infosview_expansion: the informations view will be shown to the right of the watcher" */
  gtk_box_pack_start( GTK_BOX(vbox_item), item, FALSE, TRUE, 0);

#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 4
  table = gtk_grid_new();
  gtk_grid_insert_row( GTK_GRID(table), 0 );
  gtk_grid_insert_row( GTK_GRID(table), 1 );
  gtk_grid_insert_column( GTK_GRID(table), 0 );
  gtk_grid_insert_column( GTK_GRID(table), 1 );
  gtk_grid_attach( GTK_GRID(table), vbox_bott, 0, 0, 1, 1 );
  gtk_grid_attach( GTK_GRID(table), vbox_top, 1, 0, 1, 1 );
  gtk_grid_attach( GTK_GRID(table), vbox_left, 0, 1, 1, 1 );
  gtk_grid_attach( GTK_GRID(table), vbox_right, 1, 1, 1, 1 );
#else
  table = gtk_table_new( 2, 2, TRUE );
  gtk_table_attach_defaults( GTK_TABLE(table), vbox_bott, 0, 1, 0, 1 );
  gtk_table_attach_defaults( GTK_TABLE(table), vbox_top, 1, 2, 0, 1 );
  gtk_table_attach_defaults( GTK_TABLE(table), vbox_left, 0, 1, 1, 2 );
  gtk_table_attach_defaults( GTK_TABLE(table), vbox_right, 1, 2, 1, 2 );
#endif
  gtk_container_add( GTK_CONTAINER(frame), table );

  if( app->Flags & APP_FLG_PHONE )
    gtk_widget_set_sensitive( frame, 0 );
#endif
}

static void         Callback_main_color_set(GtkColorButton *widget, gpointer data)
{
  unsigned int      a;

#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 4
  GdkRGBA           color;

  gtk_color_chooser_get_rgba( GTK_COLOR_CHOOSER(widget), &color );

  if( color.red != app->Gui.MainColor.red || color.green != app->Gui.MainColor.green
      || color.blue != app->Gui.MainColor.blue || color.alpha != app->Gui.MainColor.alpha ) {
#else
  GdkColor          color;
  float             opacity;

  gtk_color_button_get_color( widget, &color );
  opacity = gtk_color_button_get_alpha( widget ) / 65536.0;

  if( color.red != app->Gui.MainColor.red || color.green != app->Gui.MainColor.green
      || color.blue != app->Gui.MainColor.blue || opacity != app->Gui.MainOpacity ) {
#endif

    if( !( app->Gui.CancelPref.Flags & GUI_CANCEL_FLG_MAIN_COLOR )) {
      app->Gui.CancelPref.Flags |= GUI_CANCEL_FLG_MAIN_COLOR | GUI_CANCEL_FLG_PREFERENCES;
      app->Gui.CancelPref.MainColor.red = app->Gui.MainColor.red;
      app->Gui.CancelPref.MainColor.green = app->Gui.MainColor.green;
      app->Gui.CancelPref.MainColor.blue = app->Gui.MainColor.blue;
#if GTK_MAJOR_VERSION >= 3
      app->Gui.CancelPref.MainColor.alpha = app->Gui.MainColor.alpha;
#else
      app->Gui.CancelPref.MainOpacity = app->Gui.MainOpacity;
#endif
      gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
    }

    app->Gui.MainColor.red = color.red;
    app->Gui.MainColor.green = color.green;
    app->Gui.MainColor.blue = color.blue;
#if GTK_MAJOR_VERSION >= 3
    app->Gui.MainColor.alpha = color.alpha;
#else
    app->Gui.MainOpacity = opacity;
#endif

    Callback_draw_main( app->Gui.Main[MW_WIN_MAIN], 0, 0 );

    for( a = 0; a < app->Gui.WatcherCount; ++a )
      if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE ))
	Callback_on_expose_watcher( app->Gui.WatcherTable[a]->DrawArea, 0, app->Gui.WatcherTable[a] );
  }
}

static void         Create_option_color(GtkWidget *parent)
{
  GtkWidget         *row, *item;
  const char        *label, *tooltip;

  label = app->Strs[150]; /* " Background color " */
  tooltip = app->Strs[151]; /* "bg_color: to set the background color of the main window" */

#ifdef LIB_HANDY
  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, label );
  if( !( app->Flags & APP_FLG_PHONE )) {
    hdy_action_row_set_subtitle( (HdyActionRow*) row, tooltip );
    g_object_set( row, "subtitle-lines", 2, NULL );
  }
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_color_button_new_with_rgba( &app->Gui.MainColor );
  gtk_container_add( GTK_CONTAINER(row), item );
  hdy_action_row_set_activatable_widget( (HdyActionRow*) row, item );
#else
  row = APP_GTK_HBOX( FALSE, 0 );
  gtk_box_pack_start( GTK_BOX(parent), row, FALSE, TRUE, 0);

  item = gtk_label_new( label );
  gtk_widget_set_tooltip_text( item, tooltip );
  gtk_box_pack_start(GTK_BOX(row), item, FALSE, FALSE, 0);

# if GTK_MAJOR_VERSION == 3
  item = gtk_color_button_new_with_rgba( &app->Gui.MainColor );
# else
  item = gtk_color_button_new();
# endif
  gtk_widget_set_tooltip_text( item, tooltip );
  gtk_box_pack_start( GTK_BOX(row), item, TRUE, TRUE, 0 );
#endif

  gtk_color_button_set_use_alpha( GTK_COLOR_BUTTON(item), 1 );

#if GTK_MAJOR_VERSION >= 3
  GdkColor color;
  color.red = app->Gui.MainColor.red * 65536.0;
  color.green = app->Gui.MainColor.green * 65536.0;
  color.blue = app->Gui.MainColor.blue * 65536.0;
  gtk_color_button_set_color( GTK_COLOR_BUTTON(item), &color );
  gtk_color_button_set_alpha( GTK_COLOR_BUTTON(item), app->Gui.MainColor.alpha * 65536.0 );
#else
  gtk_color_button_set_alpha( GTK_COLOR_BUTTON(item), app->Gui.MainOpacity * 65536.0 );
  gtk_color_button_set_color( GTK_COLOR_BUTTON(item), &app->Gui.MainColor );
#endif

  g_signal_connect( G_OBJECT(item), "color-set", G_CALLBACK(Callback_main_color_set), 0 );
  app->Gui.PreferenceSettings[PS_BUT_COLOR] = item;
}

static void         Create_option_vexpand(GtkWidget *parent)
{
  GtkWidget         *row, *item;

#ifdef LIB_HANDY
  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, app->Strs[18] ); /* "Vertical expand" */
  if( !( app->Flags & APP_FLG_PHONE )) {
    hdy_action_row_set_subtitle( (HdyActionRow*) row, app->Strs[87] ); /* "vertical_expansion, if enable, icons will expand to the bottom then to the right" */
    g_object_set( row, "subtitle-lines", 2, NULL );
  }
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_switch_new();
  gtk_widget_set_valign( item, GTK_ALIGN_CENTER );
  if( app->Gui.Flags & GUI_FLG_VERTICAL_EXPAND )
    gtk_switch_set_state( GTK_SWITCH(item), 1 );
  hdy_action_row_set_activatable_widget( (HdyActionRow*) row, item );
  g_signal_connect( G_OBJECT(item), "state-set", G_CALLBACK(Callback_state_vexpand), 0 );
  gtk_container_add( GTK_CONTAINER(row), item );
#else
# if GTK_MAJOR_VERSION >= 3
  row = APP_GTK_HBOX( TRUE, 0 );
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_label_new( app->Strs[18] ); /* "Vertical expand" */
  gtk_widget_set_tooltip_text( item, app->Strs[87] ); /* "vertical_expansion, if enable, icons will expand to the bottom then to the right" */
  gtk_box_pack_start( GTK_BOX(row), item, FALSE, FALSE, 0 );

  item = gtk_switch_new();
  if( app->Gui.Flags & GUI_FLG_VERTICAL_EXPAND )
    gtk_switch_set_state( GTK_SWITCH(item), 1 );
  g_signal_connect( G_OBJECT(item), "state-set", G_CALLBACK(Callback_state_vexpand), 0 );
  gtk_box_pack_start( GTK_BOX(row), item, TRUE, FALSE, 0 );
# else
  row = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_check_button_new_with_label( app->Strs[18] ); /* "Vertical expand" */
  if( app->Gui.Flags & GUI_FLG_VERTICAL_EXPAND )
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(item), 1 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_check_vexpand), 0 );
  gtk_box_pack_start( GTK_BOX(row), item, FALSE, FALSE, 0 );
  gtk_widget_set_tooltip_text( item, app->Strs[87] ); /* "vertical_expansion, if enable, icons will expand to the bottom then to the right" */
# endif
#endif

  app->Gui.PreferenceSettings[PS_TOGGLE_VEXPAND] = item;
}

static void         Create_option_width_count(GtkWidget *parent)
{
  GtkWidget         *row, *item;
  const char        *label, *tooltip;

  label = app->Strs[24]; /* " Maximal width " */
  tooltip = app->Strs[85]; /* "max_width, to set the maximum count of icon in width" */

#ifdef LIB_HANDY
  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, label );
  if( !( app->Flags & APP_FLG_PHONE )) {
    hdy_action_row_set_subtitle( (HdyActionRow*) row, tooltip );
    g_object_set( row, "subtitle-lines", 2, NULL );
  }
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_spin_button_new_with_range( 1, MAX_COUNT_HEIGHT, 1 );
  hdy_action_row_set_activatable_widget( (HdyActionRow*) row, item );
  gtk_widget_set_valign( item, GTK_ALIGN_CENTER );
  gtk_container_add( GTK_CONTAINER(row), item );
#else
  row = APP_GTK_HBOX( TRUE, 0 );
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_label_new( label );
  gtk_widget_set_tooltip_text( item, tooltip );
  gtk_box_pack_start( GTK_BOX(row), item, FALSE, FALSE, 0 );

  item = gtk_spin_button_new_with_range( 1, MAX_COUNT_WIDTH, 1 );
  gtk_box_pack_start( GTK_BOX(row), item, TRUE, FALSE, 0 );
#endif

  gtk_widget_set_tooltip_text( item, "1 -> 64" );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(item), app->Gui.MaxXWatcher );
  g_signal_connect( G_OBJECT(item), "value-changed", G_CALLBACK(Callback_spin_width), 0 );
  app->Gui.PreferenceSettings[PS_SPIN_MAX_WIDTH] = item;

  if( app->Flags & APP_FLG_PHONE )
    gtk_widget_set_sensitive( row, 0 );
}

static void         Create_option_height_count(GtkWidget *parent)
{
  GtkWidget         *row, *item;
  const char        *label, *tooltip;

  label = app->Strs[25]; /* " Maximal height " */
  tooltip = app->Strs[86]; /* "max_width, to set the maximum count of icon in width" */

#ifdef LIB_HANDY
  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, label );
  if( !( app->Flags & APP_FLG_PHONE )) {
    hdy_action_row_set_subtitle( (HdyActionRow*) row, tooltip );
    g_object_set( row, "subtitle-lines", 2, NULL );
  }
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_spin_button_new_with_range( 1, MAX_COUNT_HEIGHT, 1 );
  hdy_action_row_set_activatable_widget( (HdyActionRow*) row, item );
  gtk_widget_set_valign( item, GTK_ALIGN_CENTER );
  gtk_container_add( GTK_CONTAINER(row), item );
#else
  row = APP_GTK_HBOX( TRUE, 0 );
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_label_new( label );
  gtk_widget_set_tooltip_text( item, tooltip );
  gtk_box_pack_start( GTK_BOX(row), item, FALSE, FALSE, 0 );

  item = gtk_spin_button_new_with_range( 1, MAX_COUNT_HEIGHT, 1 );
  gtk_box_pack_start( GTK_BOX(row), item, TRUE, FALSE, 0 );
#endif

  gtk_widget_set_tooltip_text( item, "1 -> 64" );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(item), app->Gui.MaxYWatcher );
  g_signal_connect( G_OBJECT(item), "value-changed", G_CALLBACK(Callback_spin_height), 0 );
  app->Gui.PreferenceSettings[PS_SPIN_MAX_HEIGHT] = item;

  if( app->Flags & APP_FLG_PHONE )
    gtk_widget_set_sensitive( row, 0 );
}

static void         Callback_icon_size(GtkRange *range, gpointer  user_data)
{
  if( ! app->Gui.CancelPref.SizeIcon ) {
    app->Gui.CancelPref.SizeIcon = app->Gui.WatcherSizeX;
    gtk_widget_show( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
  }

  app->Gui.WatcherSizeX = gtk_range_get_value( range );
  app->Gui.WatcherSizeY = app->Gui.WatcherSizeX;

  Update_watchers_geometry();
  Update_main_geometry();
}

static void         Create_option_icon_size(GtkWidget *parent)
{
  GtkWidget         *row, *item;
  const char        *label;

  label = app->Strs[125]; /* "Icon size" */

#ifdef LIB_HANDY
  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, label );
  gtk_container_add( GTK_CONTAINER(parent), row );
#else
  row = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_label_new( label );
  gtk_box_pack_start( GTK_BOX(row), item, FALSE, FALSE, 0 );
#endif

#if GTK_MAJOR_VERSION == 3
  item = gtk_scale_new_with_range( GTK_ORIENTATION_HORIZONTAL, 26.0, 128.0, 2.0 );
  gtk_widget_set_hexpand( item, TRUE );
#else
  item = gtk_hscale_new_with_range( 26.0, 128.0, 2.0 );
#endif

  gtk_container_add( GTK_CONTAINER(row), item );
  gtk_scale_set_value_pos( GTK_SCALE(item), GTK_POS_LEFT );
  gtk_range_set_value( GTK_RANGE(item), app->Gui.WatcherSizeX );
  g_signal_connect( G_OBJECT(item), "value-changed", G_CALLBACK(Callback_icon_size), 0 );
  app->Gui.PreferenceSettings[PS_SCALE_ICON] = item;
}

static void         Create_option_lock_position(GtkWidget *parent)
{
  GtkWidget         *row, *item;
  const char        *label, *tooltip;

  label = app->Strs[36]; /* "Lock position" */
  tooltip = app->Strs[88]; /* "lock_position: if enable, the main window will not be movable with a simple mouse click" */

#ifdef LIB_HANDY
  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, label );
  if( !( app->Flags & APP_FLG_PHONE )) {
    hdy_action_row_set_subtitle( (HdyActionRow*) row, tooltip );
    g_object_set( row, "subtitle-lines", 2, NULL );
  }
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_switch_new();
  hdy_action_row_set_activatable_widget( (HdyActionRow*) row, item );
  gtk_widget_set_valign( item, GTK_ALIGN_CENTER );
  if( app->Gui.Flags & GUI_FLG_LOCK_MAIN )
    gtk_switch_set_state( GTK_SWITCH(item), 1 );
  g_signal_connect( G_OBJECT(item), "state-set", G_CALLBACK(Callback_toggle_lock), 0 );
  gtk_container_add( GTK_CONTAINER(row), item );
#else
# if GTK_MAJOR_VERSION == 3
  row = APP_GTK_HBOX( TRUE, 0 );
  gtk_container_add( GTK_CONTAINER(parent), row );

  item = gtk_label_new( label );
  gtk_widget_set_tooltip_text( item, tooltip );
  gtk_box_pack_start( GTK_BOX(row), item, FALSE, FALSE, 0 );

  item = gtk_switch_new();
  if( app->Gui.Flags & GUI_FLG_LOCK_MAIN )
    gtk_switch_set_state( GTK_SWITCH(item), 1 );
  g_signal_connect( G_OBJECT(item), "state-set", G_CALLBACK(Callback_toggle_lock), 0 );
  gtk_box_pack_start( GTK_BOX(row), item, TRUE, FALSE, 0 );

# else // GTK_MAJOR_VERSION == 2
  item = gtk_check_button_new_with_label( label );
  if( app->Gui.Flags & GUI_FLG_LOCK_MAIN )
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(item), 1 );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_check_lock), 0 );
  gtk_box_pack_start( GTK_BOX(parent), item, FALSE, FALSE, 0 );
  row = item;
# endif

  gtk_widget_set_tooltip_text( item, tooltip );
#endif

  app->Gui.PreferenceSettings[PS_ROW_LOCK_POS] = row;
  app->Gui.PreferenceSettings[PS_CHK_LOCK_POS] = item;
}

static void          Create_option_always_on(GtkWidget *parent)
{
  GtkWidget         *item, *row;
  const char        *label, *tooltip;

  label = app->Strs[39]; /* " Always on " */
  tooltip = app->Strs[89]; /* "always_on: allow to keep main window on top of or behind other windows" */

#ifdef LIB_HANDY
  row = hdy_action_row_new();
  hdy_preferences_row_set_title( (HdyPreferencesRow*) row, label );
  if( !( app->Flags & APP_FLG_PHONE )) {
    hdy_action_row_set_subtitle( (HdyActionRow*) row, tooltip );
    g_object_set( row, "subtitle-lines", 2, NULL );
  }
  gtk_container_add( GTK_CONTAINER(parent), row );
  app->Gui.PreferenceSettings[PS_HBOX_ALWAYS_ON] = row;

  row = APP_GTK_HBOX( FALSE, 0 );
  gtk_widget_set_valign( row, GTK_ALIGN_CENTER );
  gtk_container_add( GTK_CONTAINER(app->Gui.PreferenceSettings[PS_HBOX_ALWAYS_ON]), row );

  /* - Radio - Always on top */
  item = gtk_radio_button_new_with_label_from_widget( NULL, app->Strs[40] ); /* "top" */
  g_object_set( item, "draw-indicator", FALSE, NULL );
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_radio_always_on_top), 0 );
  gtk_container_add( GTK_CONTAINER(row), item );
  app->Gui.PreferenceSettings[PS_RAD_ALWAYS_ON_TOP] = item;

  /* - Radio - Always on bottom */
  item = gtk_radio_button_new_with_label_from_widget( GTK_RADIO_BUTTON(item), app->Strs[41] ); /* "bottom" */
  g_object_set( item, "draw-indicator", FALSE, NULL );
#else
  /* - Radio - Always on top */
  row = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(parent), row );
  app->Gui.PreferenceSettings[PS_HBOX_ALWAYS_ON] = row;

  item = gtk_label_new( label );
  gtk_widget_set_tooltip_text( item, tooltip );
  gtk_box_pack_start(GTK_BOX(row), item, FALSE, FALSE, 0);

  item = gtk_radio_button_new_with_label_from_widget( NULL, app->Strs[40] ); /* "top" */
  gtk_widget_set_tooltip_text( item, app->Strs[90] ); /* "lock_position: keep main window on top of other windows" */
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_radio_always_on_top), 0 );
  gtk_container_add( GTK_CONTAINER(row), item );
  app->Gui.PreferenceSettings[PS_RAD_ALWAYS_ON_TOP] = item;

  /* - Radio - Always on bottom */
  item = gtk_radio_button_new_with_label_from_widget( GTK_RADIO_BUTTON(item), app->Strs[41] ); /* "bottom" */
  gtk_widget_set_tooltip_text( item, app->Strs[91] ); /* "lock_position: keep main window behind other windows" */
#endif
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_radio_always_on_bottom), 0 );
  gtk_container_add( GTK_CONTAINER(row), item );
  app->Gui.PreferenceSettings[PS_RAD_ALWAYS_ON_BOTTOM] = item;
}

static void         Create_preferences_main_window(GtkWidget *parent)
{
  GtkWidget         *group;

#ifdef LIB_HANDY
  group = parent;
#else
  GtkWidget         *frame;

  frame = gtk_frame_new( app->Strs[26] ); /* "Main window" */
  gtk_box_pack_start( GTK_BOX(parent), frame, TRUE, TRUE, 0);

  group = APP_GTK_VBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(frame), group );
#endif

  /* - scale - icon size  */
  Create_option_icon_size( group );

  /* - Button - Color */
  Create_option_color( group );

  /* - toggle - vertical expand */
  Create_option_vexpand( group );

  /* - spin - width */
  Create_option_width_count( group );

  /* - spin - height */
  Create_option_height_count( group );

  /* - toggle - lock position  */
  Create_option_lock_position( group );

  /* - Radio - Always on */
  Create_option_always_on( group );
}

static void         Create_preferences_tab_globals(GtkWidget *tab)
{
  GtkWidget         *vbox;

  vbox = APP_GTK_VBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(tab), vbox );
  gtk_notebook_set_tab_label_text( GTK_NOTEBOOK(tab), vbox, app->Strs[27] ); /* "Global settings" */

#ifdef LIB_HANDY
  GtkWidget *page, *group;

  page = hdy_preferences_page_new();
  gtk_container_add( GTK_CONTAINER(vbox), page );

  /* Main window */
  group = hdy_preferences_group_new();
  hdy_preferences_group_set_title( (HdyPreferencesGroup*) group, app->Strs[26] ); /* "Main window" */
  gtk_container_add( GTK_CONTAINER(page), group );

  Create_preferences_main_window( group );

  /* InfosView expansion */
  Create_preferences_infosview_expansion( group );

  /* Others options */
  group = hdy_preferences_group_new();
  hdy_preferences_group_set_title( (HdyPreferencesGroup*) group, app->Strs[37] ); /* "Other options" */
  gtk_container_add( GTK_CONTAINER(page), group );

  Create_preferences_others( group );
#else
  GtkWidget         *hbox;

  hbox = APP_GTK_HBOX( TRUE, 0 );
  gtk_box_pack_start( GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  /* Main window */
  Create_preferences_main_window( hbox );

  /* InfosView expansion */
  Create_preferences_infosview_expansion( hbox );

  /* Others options */
  Create_preferences_others( vbox );
#endif
}

void                Create_window_preferences()
{
  GtkWidget         *win, *vbox_main, *item, *align, *line, *hbox, *tab;
  GError            *err;
  gint              width, height;

  /* Window ----------------------------------------------------------------- */
#ifdef LIB_HANDY
  if( app->Flags & APP_FLG_PHONE )
    win = hdy_window_new();
  else
#endif
    win = gtk_window_new( GTK_WINDOW_TOPLEVEL );
  g_signal_connect( G_OBJECT(win), "delete_event", G_CALLBACK(Callback_on_delete_cancel), 0 );
  g_signal_connect( G_OBJECT(win), "destroy", G_CALLBACK(Callback_on_destroy), 0 );

  gtk_window_set_title( GTK_WINDOW(win), app->Strs[15] ); /* "Settings" */
  gtk_window_set_icon_from_file( GTK_WINDOW(win), DEFAULT_APP_RESOURCES"/icon_config_watcher.png", &err );
  app->Gui.PreferenceSettings[PS_WIN_MAIN] = win;

  vbox_main = APP_GTK_VBOX( FALSE, 0 );

#ifdef LIB_HANDY
  GtkWidget *leaf = hdy_leaflet_new();
  gtk_container_add( GTK_CONTAINER(win), leaf );
  gtk_widget_show_all( leaf );

  gtk_container_add( GTK_CONTAINER(leaf), vbox_main );

  if( app->Flags & APP_FLG_PHONE ) {
    GtkWidget *bar = hdy_header_bar_new();
    g_object_set( bar, "show_close_button", TRUE,
		  "title", app->Strs[15], /* "Settings" */
		  "can-focus", FALSE,
		  NULL );
    gtk_container_add( GTK_CONTAINER(vbox_main), bar );
  }
  else {
    SCREEN_WIDTH_HEIGHT( width, height );
    if( width > 500 ) width = 500;
    else if( width > 360 ) width = 360;
    if( height > 700 ) height = 700;
    else if( height > 500 ) height = 500;
    gtk_window_set_default_size( GTK_WINDOW(win), width, height );
    gtk_window_set_resizable( GTK_WINDOW(win), FALSE );
  }
#else
  Set_window_properties( win, vbox_main );

  if( app->Flags & APP_FLG_PHONE ) {
    SCREEN_WIDTH_HEIGHT( width, height );
    gtk_window_set_default_size( GTK_WINDOW(win), width, height );
  }
  else {
    gtk_window_set_default_size( GTK_WINDOW(win), 10, 10 );
    gtk_window_set_resizable( GTK_WINDOW(win), FALSE );
  }
#endif

  /* Tabs ------------------------------------------------------------------- */
  tab = gtk_notebook_new();
  g_signal_connect( G_OBJECT(tab), "switch-page", G_CALLBACK(Callback_on_tab_change), 0 );
  gtk_box_pack_start( GTK_BOX(vbox_main), tab, TRUE, TRUE, 0 );

  Create_preferences_tab_globals( tab );
  Create_preferences_tab_watchers( tab );
  Create_preferences_tab_templates( tab );

  /* Buttons ---------------------------------------------------------------- */
  if( !( app->Flags & APP_FLG_PHONE )) {
    line = APP_GTK_HSEPAR();
    gtk_container_add( GTK_CONTAINER(vbox_main), line );
  }

#if GTK_MAJOR_VERSION < 3 || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 14 )
  align = gtk_alignment_new( 1, 1, 0, 0 );
  gtk_container_add( GTK_CONTAINER(vbox_main), align );
#else
  align = vbox_main;
#endif

  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(align), hbox );
#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 14
  gtk_widget_set_halign( hbox, GTK_ALIGN_END );
#endif

  /* - Save - */
  item = gtk_button_new_with_mnemonic( app->Strs[16] ); /* "_Save" */
  app->Gui.PreferenceSettings[PS_BUT_SAVE] = item;
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_save_preferences), 0 );
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  /* - Cancel - */
  item = gtk_button_new_with_mnemonic( app->Strs[17] ); /* "C_ancel" */
  g_signal_connect( G_OBJECT(item), "clicked", G_CALLBACK(Callback_cancel), 0 );
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  /* End -------------------------------------------------------------------- */
  if( !( app->Flags & APP_FLG_PHONE ))
    if( !( app->Gui.Flags & GUI_FLG_ON_TOP ))
      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(app->Gui.PreferenceSettings[PS_RAD_ALWAYS_ON_BOTTOM]), 1 );
  gtk_widget_show_all( vbox_main );
  gtk_widget_hide( app->Gui.PreferenceSettings[PS_BUT_SAVE] );
}
