/*
 *  'UpdatesWarner' alerts you about system failures on your distant linux hosts
 *  Copyright (C) 2015-2016  Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

void                Systray_stop_blink()
{
#ifndef LIB_HANDY
  if( app->Flags & APP_FLG_PHONE )
#endif
    return;

  if( app->Flags & APP_FLG_BLINK ) {
    app->Flags &= ~( APP_FLG_BLINK );
#if GTK_MAJOR_VERSION < 3
    gtk_status_icon_set_blinking( app->Systray, 0 );
#endif
    //#if ( GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 10 ) || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 14 )
    gtk_status_icon_set_from_pixbuf( app->Systray, app->Gui.ImgTable[IMG_APP] );

    //#if ( GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 18 ) || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 14 )
    gtk_status_icon_set_title( app->Systray, APP_NAME );
  }
}

void                Callback_refresh_systray(GtkWidget *widget, gpointer data)
{
  app->Flags &= ~( APP_FLG_BLINK );
#if GTK_MAJOR_VERSION < 3
  gtk_status_icon_set_blinking( app->Systray, 0 );
#endif
  //#if ( GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 10 ) || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 14 )
  gtk_status_icon_set_from_pixbuf( app->Systray, app->Gui.ImgTable[IMG_APP] );

  //#if ( GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 18 ) || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 14 )
  gtk_status_icon_set_title( app->Systray, APP_NAME );
}

void                Callback_show_preferences(GtkWidget *widget, s_gui *gui)
{
  if( !( app->Gui.Flags & GUI_FLG_ON_TOP )) {
    gtk_window_set_keep_below( GTK_WINDOW(gui->Main[MW_WIN_MAIN]), 0 );
    gtk_window_set_keep_above( GTK_WINDOW(gui->Main[MW_WIN_MAIN]), 1 );
  }

  if( gui->MainX != gui->SaveX || gui->MainY != gui->SaveY )
    gtk_widget_show( gui->PreferenceSettings[PS_BUT_SAVE] );
  gtk_window_deiconify( GTK_WINDOW(gui->PreferenceSettings[PS_WIN_MAIN]) );
  gtk_widget_show( gui->PreferenceSettings[PS_WIN_MAIN] );
}

static void         Callback_left_click(GtkWidget *widget, gpointer data)
{
  if( gtk_widget_get_visible( app->Gui.Main[MW_WIN_MAIN] ) == TRUE ) {
    if( app->Flags & APP_FLG_BLINK )
      Systray_stop_blink();
    else
      gtk_widget_hide( app->Gui.Main[MW_WIN_MAIN] );
  }
  else {
    Systray_stop_blink();
    gtk_window_deiconify( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]) );
    gtk_widget_show( app->Gui.Main[MW_WIN_MAIN] );
    gtk_window_move( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), app->Gui.MainX, app->Gui.MainY );
  }
}

static void         Callback_right_click(GtkStatusIcon *status_icon, guint button, guint32 activate_time, gpointer menu)
{
  Systray_stop_blink();
  gtk_menu_popup( GTK_MENU(menu), NULL, NULL, gtk_status_icon_position_menu, status_icon, button, activate_time );
}

void              Create_systray()
{
  GtkStatusIcon   *tray;
  GtkWidget       *menu, *item;

  /* ICON */
  //# can trigger  Gtk-CRITICAL gtk_widget_get_scale_factor: assertion 'GTK_IS_WIDGET (widget)' failed
  tray = gtk_status_icon_new_from_pixbuf( app->Gui.ImgTable[IMG_APP] );
  app->Systray = tray;

#if GTK_MAJOR_VERSION >= 3
  g_signal_connect( GTK_STATUS_ICON(tray), "activate", G_CALLBACK(Callback_left_click), 0 );
#else
  g_signal_connect( GTK_STATUS_ICON(tray), "activate", GTK_SIGNAL_FUNC(Callback_left_click), 0 );
#endif

#if GTK_MAJOR_VERSION >= 3 || ( GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 18 )
  gtk_status_icon_set_title( tray, APP_NAME );
#endif

#if GTK_MAJOR_VERSION >= 3 || ( GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 20 )
  gtk_status_icon_set_name( tray, APP_NAME );
#endif

  /* MENU */
  menu = gtk_menu_new();
#if GTK_MAJOR_VERSION >= 3
  g_signal_connect( GTK_STATUS_ICON(tray), "popup-menu", G_CALLBACK(Callback_right_click), menu );
#else
  g_signal_connect( GTK_STATUS_ICON(tray), "popup-menu", GTK_SIGNAL_FUNC(Callback_right_click), menu );
#endif

  /* MENU - About */
  GtkWidget *img;
  if( app->Gui.ImgFile[IMG_ABOUT] ) {
    item = gtk_image_menu_item_new_with_mnemonic( app->Strs[205] ); /* "About..." */
    img = gtk_image_new_from_pixbuf( app->Gui.ImgTable[IMG_ABOUT] );
    gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(item), img );
  }
  else
    item = gtk_menu_item_new_with_mnemonic( app->Strs[205] ); /* "About..." */
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Show_window), app->About[ABW_WIN_MAIN] );
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );

  /* MENU - Refresh systray */
  if( app->Gui.ImgTable[IMG_REFRESH] ) {
    item = gtk_image_menu_item_new_with_mnemonic( app->Strs[219] ); /* "_Refresh systray" */
    img = gtk_image_new_from_pixbuf( app->Gui.ImgTable[IMG_REFRESH] );
    gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(item), img );
  }
  else
    item = gtk_menu_item_new_with_mnemonic( app->Strs[219] ); /* "_Refresh systray" */
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_refresh_systray), 0 );
  gtk_widget_set_tooltip_text( item, app->Strs[220] ); /* "'cause some systray systems sux" */
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );

  /* MENU ------------- */
  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );

  /* MENU - Settings */
  if( app->Gui.ImgTable[IMG_SETTINGS] ) {
    item = gtk_image_menu_item_new_with_mnemonic( app->Strs[14] ); /* "_Settings" */
    img = gtk_image_new_from_pixbuf( app->Gui.ImgTable[IMG_SETTINGS] );
    gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(item), img );
  }
  else
    item = gtk_menu_item_new_with_mnemonic( app->Strs[14] ); /* "_Settings" */
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_show_preferences), &app->Gui );
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );

  /* MENU ------------- */
  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );

  /* MENU - Quit */
  img = gtk_image_new_from_file( DEFAULT_APP_RESOURCES"/icon_quit.png" );
  if( img ) {
    item = gtk_image_menu_item_new_with_mnemonic( app->Strs[11] ); /* "_Quit" */
    gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(item), img );
  }
  else
    item = gtk_menu_item_new_with_mnemonic( app->Strs[11] ); /* "_Quit" */

  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Quit_app), 0 );
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );

  gtk_widget_show_all( menu );

  /* TOOLTIP */
#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 16
  gtk_status_icon_set_tooltip( tray, APP_NAME );
#else
  gtk_status_icon_set_tooltip_text( tray, APP_NAME );
#endif

  gtk_status_icon_set_visible( app->Systray, ( app->Flags & APP_FLG_SYSTRAY ) != 0 );
}
