/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

/*
  Purpose: Updating position of the InfoWindow of the watcher
*/
void                Update_info_position(s_wtr *watcher)
{
  int               x, y, width, height;

  gtk_window_get_size( GTK_WINDOW(watcher->InfosWindow), (gint*) &watcher->InfoSizeX, (gint*) &watcher->InfoSizeY );
  gtk_window_get_position( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), (gint*) &x, (gint*) &y );

  if( app->Gui.Flags & GUI_FLG_VERTICAL_EXPAND ) {
    x += ( watcher->VisiblePos / app->Gui.MaxYWatcher ) * ( app->Gui.WatcherSizeX + 2 );
    y += ( watcher->VisiblePos % app->Gui.MaxYWatcher ) * ( app->Gui.WatcherSizeY + 2 );
  }
  else {
    x += ( watcher->VisiblePos % app->Gui.MaxXWatcher ) * ( app->Gui.WatcherSizeX + 2 );
    y += ( watcher->VisiblePos / app->Gui.MaxXWatcher ) * ( app->Gui.WatcherSizeY + 2 );
  }

  if( app->Flags & APP_FLG_PHONE ) {
    SCREEN_WIDTH_HEIGHT( width, height );
    x = ( width - watcher->InfoSizeX ) / 2;
    y = ( height - watcher->InfoSizeY ) / 2;
  }
  else {
    switch( app->Gui.Flags & GUI_FLG_INFOS_MASK ) {
    case 0:
      x += ( app->Gui.WatcherSizeX + 1 ) / 2 - watcher->InfoSizeX / 2;
      y += app->Gui.WatcherSizeY;
      break;

    case GUI_FLG_INFOS_TO_TOP:
      x += ( app->Gui.WatcherSizeX + 1 ) / 2 - watcher->InfoSizeX / 2;
      y -= watcher->InfoSizeY + 2;
      break;

    case GUI_FLG_INFOS_TO_LEFT:
      x -= watcher->InfoSizeX + 2;
      y += ( app->Gui.WatcherSizeY - watcher->InfoSizeY ) / 2;
      break;

    case GUI_FLG_INFOS_TO_RIGHT:
      x += app->Gui.WatcherSizeX + 2;
      y += ( app->Gui.WatcherSizeY - watcher->InfoSizeY ) / 2;
      break;
    }
  }
  gtk_window_move( GTK_WINDOW(watcher->InfosWindow), x, y );
}

/*
  Purpose: Updating the shown/hidden elements in the infosview of a watcher
*/
#define SHOW_LOCAL    (1 << 0)
#define SHOW_OFFICIAL (1 << 1)
#define SHOW_UPDATE   (1 << 2)
void                Update_watcher_infosview(s_wtr *watcher)
{
  unsigned char     flags = 0;

  if( watcher->Flags & ( WTR_FLG_LOCAL_ERROR | WTR_FLG_OFFICIAL_ERROR | WTR_FLG_LOCAL_IN_PROGRESS | WTR_FLG_OFFICIAL_IN_PROGRESS | WTR_FLG_VERSION_MISMATCH ))
    flags = SHOW_LOCAL | SHOW_OFFICIAL;

  if( !( watcher->Flags & ( WTR_FLG_OFFICIAL_ERROR | WTR_FLG_OFFICIAL_IN_PROGRESS ))
      || ( watcher->Flags & WTR_FLG_VERSION_MISMATCH ))
    flags |= SHOW_UPDATE;

  if( flags & SHOW_LOCAL )
    gtk_widget_show( watcher->LabelLocalVersion );
  else
    gtk_widget_hide( watcher->LabelLocalVersion );

  if( flags & SHOW_OFFICIAL ) {
    gtk_widget_show( watcher->LabelOfficialVersion );
    gtk_widget_show( watcher->LabelOfficialVersionResult );
  }
  else {
    gtk_widget_hide( watcher->LabelOfficialVersion );
    gtk_widget_hide( watcher->LabelOfficialVersionResult );
  }

  if( flags & SHOW_UPDATE )
    gtk_widget_show( watcher->BoxNextUpdate );
  else
    gtk_widget_hide( watcher->BoxNextUpdate );

  if( flags & ( SHOW_LOCAL | SHOW_OFFICIAL ))
    gtk_widget_hide( watcher->LabelState );
  else
    gtk_widget_show( watcher->LabelState );

#if GTK_MAJOR_VERSION >= 3
  /* GTK3 the infoview doesn't auto-resize like in GTK2, so forcing it */
  gtk_widget_set_size_request( watcher->InfosWindow, 30, 10 );
#endif
}

/*
  Purpose: Callback for enter event on DrawArea of watcher
*/
static gboolean     Callback_on_enter(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  s_wtr             *watcher = data;

  Systray_stop_blink();

  if( app->Gui.HoveredWatcher != watcher ) {
    if( app->Gui.HoveredWatcher )
      gtk_widget_hide( app->Gui.HoveredWatcher->InfosWindow );
    app->Gui.HoveredWatcher = watcher;
  }

  if( !( app->Flags & APP_FLG_PHONE )) {
    Update_info_position( watcher );
    gtk_widget_show( watcher->InfosWindow );
  }
  return( TRUE );
}

/*
  Purpose: Callback for leave event on DrawArea of watcher
*/
static gboolean     Callback_on_leave(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  s_wtr             *watcher = data;

  app->Gui.HoveredWatcher = 0;
  gtk_widget_hide( watcher->InfosWindow );
  return( TRUE );
}

#if GTK_MAJOR_VERSION < 3
/*
  Purpose: Callback for expose event of the drawable area of the watcher, can also be called to redraw watcher area
  - 'event' can be 0
  - This Function MUST be protected with gdk_threads_enter() and gdk_threads_leave()
*/
gboolean            Callback_on_expose_watcher(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  s_wtr             *watcher = data;
  cairo_t           *cr;
  GdkDrawable       *draw;
  int               x, y;

  ASSERT( !watcher, "watcher = 0" );
  if( watcher->Flags & WTR_FLG_DISABLE ) /* can occur in rare race condition */
    return( TRUE );

  if( !widget->window ) {
    SNO( 1, "%s: widget->window == 0", watcher->Name );
    return( TRUE );
  }

  cr = gdk_cairo_create( widget->window );
  if( app->Flags & APP_FLG_NO_ALPHA )
    cairo_set_source_rgb( cr, app->Gui.MainColor.red / 65536.0, app->Gui.MainColor.green / 65536.0, app->Gui.MainColor.blue / 65536.0 );
  else
    cairo_set_source_rgba( cr, app->Gui.MainColor.red / 65536.0, app->Gui.MainColor.green / 65536.0, app->Gui.MainColor.blue / 65536.0, app->Gui.MainOpacity );
  cairo_set_operator( cr, CAIRO_OPERATOR_SOURCE );
  cairo_paint( cr );
  cairo_destroy( cr );

  draw = gtk_widget_get_window( widget );
  gdk_draw_pixbuf( draw, NULL, watcher->CurrentPixbuf, 0, 0, 0, 0, -1, -1, GDK_RGB_DITHER_NONE, 0, 0 );

  x = app->Gui.WatcherSizeX - 20;
  y = app->Gui.WatcherSizeY - 20;
  ASSERT_NON_FATAL( x < 0 || y < 0, "x = %d, y = %d", x, y );
  if( watcher->Flags & ( WTR_FLG_LOCAL_ERROR | WTR_FLG_OFFICIAL_ERROR ))
    gdk_draw_pixbuf( draw, NULL, app->Gui.ImgTable[IMG_VERSION_ERROR], 0, 0, x, y, -1, -1, GDK_RGB_DITHER_NONE, 0, 0 );
  else if( watcher->Flags & WTR_FLG_VERSION_MISMATCH )
    gdk_draw_pixbuf( draw, NULL, app->Gui.ImgTable[IMG_VERSION_MISMATCH], 0, 0, x, y, -1, -1, GDK_RGB_DITHER_NONE, 0, 0 );
  if( watcher->Flags & ( WTR_FLG_LOCAL_IN_PROGRESS | WTR_FLG_OFFICIAL_IN_PROGRESS ))
    gdk_draw_pixbuf( draw, NULL, app->Gui.ImgTable[IMG_VERSION_IN_PROGRESS], 0, 0, x, y, -1, -1, GDK_RGB_DITHER_NONE, 0, 0 );
  return( TRUE );
}
#else
/*
  Purpose: Callback for draw event of the drawable area of the watcher, can also be called to redraw watcher area
  - 'event' can be 0
  - This Function MUST be protected with gdk_threads_enter() and gdk_threads_leave() if not called from GTK callback
*/
gboolean            Callback_on_expose_watcher(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  s_wtr             *watcher = data;
  GdkWindow         *win;
  cairo_region_t    *region = 0;
  GdkDrawingContext *draw_ctxt;
  cairo_t           *cr, *cr_pix;

  ASSERT( !watcher, "watcher = 0" );
  if( watcher->Flags & WTR_FLG_DISABLE ) /* can occur in rare race condition */
    return( TRUE );

  win = gtk_widget_get_window( widget );
  if( ! win ) {
    SNO( 1, "%s: gtk_widget_get_window() failed", watcher->Name );
    return( TRUE );
  }

  region = cairo_region_create();
  if( ! region ) {
    SNO( 1, "%s: cairo_region_create() failed", watcher->Name );
    goto error;
  }

  draw_ctxt = gdk_window_begin_draw_frame( win, region );
  if( ! draw_ctxt ) {
    SNO( 1, "%s: gdk_window_begin_draw_frame() failed", watcher->Name );
    goto error;
  }

  cr = gdk_drawing_context_get_cairo_context( draw_ctxt );
  if( ! cr ) {
    SNO( 1, "%s: gdk_drawing_context_get_cairo_context() failed", watcher->Name );
    goto error;
  }

  if( app->Flags & APP_FLG_NO_ALPHA )
    cairo_set_source_rgb( cr, app->Gui.MainColor.red, app->Gui.MainColor.green, app->Gui.MainColor.blue );
  else
    cairo_set_source_rgba( cr, app->Gui.MainColor.red, app->Gui.MainColor.green, app->Gui.MainColor.blue, app->Gui.MainColor.alpha );
  cairo_set_operator( cr, CAIRO_OPERATOR_SOURCE );
  cairo_paint( cr );

  cr_pix = gdk_cairo_create( win );
  if( ! cr_pix ) {
    SNO( 1, "%s: gdk_cairo_create() failed", watcher->Name );
    goto error;
  }

  gdk_cairo_set_source_pixbuf( cr_pix, watcher->CurrentPixbuf, 0, 0 );
  cairo_paint( cr_pix );

  if( watcher->Flags & ( WTR_FLG_LOCAL_ERROR | WTR_FLG_OFFICIAL_ERROR )) {
    gdk_cairo_set_source_pixbuf( cr_pix, app->Gui.ImgTable[IMG_VERSION_ERROR], 6, 6 );
    cairo_paint( cr_pix );
  }
  else if( watcher->Flags & WTR_FLG_VERSION_MISMATCH ) {
    gdk_cairo_set_source_pixbuf( cr_pix, app->Gui.ImgTable[IMG_VERSION_MISMATCH], 6, 6 );
    cairo_paint( cr_pix );
  }

  if( watcher->Flags & ( WTR_FLG_LOCAL_IN_PROGRESS | WTR_FLG_OFFICIAL_IN_PROGRESS )) {
    gdk_cairo_set_source_pixbuf( cr_pix, app->Gui.ImgTable[IMG_VERSION_IN_PROGRESS], 6, 6 );
    cairo_paint( cr_pix );
  }

  gdk_window_end_draw_frame( win, draw_ctxt );
  cairo_region_destroy( region );
  cairo_destroy( cr_pix );
  return( TRUE );

 error:
  if( region ) cairo_region_destroy( region );
  //# draw_ctxt free ?
  //# cr free ?
  return( TRUE );
}

#endif

/*
  Purpose: Creating the GUI part of the watcher
  - The core part of 'watcher' must have been set properly
*/
void               Create_new_gui_watcher(s_wtr *watcher)
{
  GtkWidget        *win, *area, *vbox, *hbox_main, *hbox, *item;
  GdkPixbuf        *tmp_pix;
  int              curr_w, curr_h;

  /* Main drawing area ----------------------------------------------------------- */
  area = gtk_drawing_area_new();
  g_object_ref( G_OBJECT(area) ); /* grumpfffff */
  watcher->DrawArea = area;
  gtk_widget_set_size_request( area, app->Gui.WatcherSizeX, app->Gui.WatcherSizeY );
  gtk_widget_add_events( area, GDK_ENTER_NOTIFY_MASK );
  g_signal_connect( G_OBJECT(area), "enter-notify-event", G_CALLBACK(Callback_on_enter), watcher );
  gtk_widget_add_events( area, GDK_LEAVE_NOTIFY_MASK );
  g_signal_connect( G_OBJECT(area), "leave-notify-event", G_CALLBACK(Callback_on_leave), watcher );
  gtk_widget_add_events( area, GDK_LEAVE_NOTIFY_MASK );
#if GTK_MAJOR_VERSION < 3
  g_signal_connect( G_OBJECT(area), "expose-event", G_CALLBACK(Callback_on_expose_watcher), watcher );
#else
  g_signal_connect( G_OBJECT(area), "draw", G_CALLBACK(Callback_on_expose_watcher), watcher );
#endif

  /* Resizing pixbuf ---------------------------------------------------------------- */
  curr_w = gdk_pixbuf_get_width( watcher->OriginPixbuf );
  curr_h = gdk_pixbuf_get_height( watcher->OriginPixbuf );
  if( curr_w != app->Gui.WatcherSizeX || curr_h != app->Gui.WatcherSizeY ) {
    tmp_pix = gdk_pixbuf_scale_simple( watcher->OriginPixbuf, app->Gui.WatcherSizeX, app->Gui.WatcherSizeY, GDK_INTERP_BILINEAR );
    if( ! tmp_pix ) {
      ERR_APP( "%s: %s: %s", watcher->Name, app->Strs[228], watcher->ImgPath ); /* "Line" "Unable to resize image" */
      watcher->CurrentPixbuf = watcher->OriginPixbuf;
    }
    else
      watcher->CurrentPixbuf = tmp_pix;
  }
  else
    watcher->CurrentPixbuf = watcher->OriginPixbuf;

  /* Infos window ---------------------------------------------------------------- */
  win = gtk_window_new( GTK_WINDOW_POPUP );
  watcher->InfosWindow = win;

  //# required in 3.24.24 but not in 3.24.20 ? or maybe a X11/wayland difference ? I don't know what is making it a requirement
#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION > 20
  gtk_window_set_transient_for( GTK_WINDOW(win), GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]) );
#endif

  watcher->InfoSizeY = 150;
  if( app->Flags & APP_FLG_PHONE ) {
    SCREEN_WIDTH_HEIGHT( watcher->InfoSizeX, watcher->InfoSizeY );
    gtk_window_set_default_size( GTK_WINDOW(win), watcher->InfoSizeX, watcher->InfoSizeY );
  }
  else {
    watcher->InfoSizeX = 50;
    gtk_window_set_default_size( GTK_WINDOW(win), watcher->InfoSizeX, watcher->InfoSizeY );
  }
  gtk_window_set_resizable( GTK_WINDOW(win), 0 );

  area = gtk_frame_new( watcher->Name );
  watcher->LabelFrame = area;
  gtk_container_add( GTK_CONTAINER(win), area );

  hbox_main = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(area), hbox_main );

  item = APP_GTK_VSEPAR();
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  vbox = APP_GTK_VBOX( FALSE, 0 );
  gtk_box_pack_start( GTK_BOX(hbox_main), vbox, FALSE, FALSE, 0 );

  item = APP_GTK_VSEPAR();
  gtk_box_pack_start( GTK_BOX(hbox_main), item, FALSE, FALSE, 0 );

  /* - Official version -------------- */
  hbox = APP_GTK_HBOX( FALSE, 0 );
  watcher->LabelOfficialVersion = hbox;
  gtk_container_add( GTK_CONTAINER(vbox), hbox );

  item = gtk_label_new( app->Strs[13] ); /* " Official version : " */
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  item = gtk_label_new( app->Strs[60] ); /* "Not yet retrieved" */
  watcher->LabelOfficialVersionResult = item;
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  /* - State ----------------------- */
  hbox = APP_GTK_HBOX( FALSE, 0 );
  gtk_container_add( GTK_CONTAINER(vbox), hbox );

  item = gtk_label_new( app->Strs[118] ); /* " Up-to-date: " */
  watcher->LabelState = item;
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  /* - Local version -------------- */
  if( watcher->Flags & WTR_FLG_LOCAL_AUTO )
    item = gtk_label_new( app->Strs[133] ); /* " Local version (auto): " */
  else
    item = gtk_label_new( app->Strs[12] ); /* " Local version (manual): " */
  watcher->LabelLocalVersion = item;
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  item = gtk_label_new( app->Strs[60] ); /* "Not yet retrieved" */
  watcher->LabelLocalVersionResult = item;
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  /* - Next update ----------------------- */
  hbox = APP_GTK_HBOX( FALSE, 0 );
  watcher->BoxNextUpdate = hbox;
  gtk_container_add( GTK_CONTAINER(vbox), hbox );

  item = gtk_label_new( app->Strs[178] ); /* " Next update: " */
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  item = gtk_label_new( "--" );
  watcher->LabelNextUpdate = item;
  gtk_box_pack_start( GTK_BOX(hbox), item, FALSE, FALSE, 0 );

  /* Watcher list ----------------------------------------------------------------- */
  gtk_list_store_insert( app->Gui.ListWatchers, &watcher->Iter, watcher->Pos );
  gtk_list_store_set( app->Gui.ListWatchers, &watcher->Iter, COL_LOGO, watcher->CurrentPixbuf, COL_NAME, watcher->Name, COL_STATE, 1, -1 );
  gtk_list_store_set( GTK_LIST_STORE( app->Gui.ListWatchers ), &watcher->Iter, COL_STATE, !( watcher->Flags & WTR_FLG_DISABLE ), -1 );

  /* End -------------------------------------------------------------------------- */
  gtk_widget_show_all( watcher->LabelFrame );
  gtk_widget_hide( win );
  gtk_widget_hide( watcher->LabelState );
  if( !( watcher->Flags & WTR_FLG_DISABLE )) {
    watcher->Flags |= ( WTR_FLG_LOCAL_IN_PROGRESS | WTR_FLG_OFFICIAL_IN_PROGRESS );
    Update_watcher_infosview( watcher );
  }
}

/*
  Purpose: Creating the watchers GUI part
*/
void               Create_watchers()
{
  unsigned int     a, count;

  count = 0;
  for( a = 0; a < app->Gui.WatcherCount; ++a ) {
    if( !( app->Gui.WatcherTable[a]->Flags & WTR_FLG_DISABLE ))
      app->Gui.WatcherTable[a]->VisiblePos = count++;
    Create_new_gui_watcher( app->Gui.WatcherTable[a] );
  }

  if( !count )
    app->Gui.Flags |= GUI_FLG_NO_WATCHER;
  else
    app->Gui.Flags &= ~( GUI_FLG_NO_WATCHER );

  gtk_window_move( GTK_WINDOW(app->Gui.Main[MW_WIN_MAIN]), app->Gui.MainX, app->Gui.MainY );

  Update_main_geometry();
}
