/*
 *  'UpdatesWarner' permits you to survey new versions
 *  Copyright (C) 2017-2021 Vincent Deca. known as Bigcake@ubuntu-fr.org
 *  (user@forum not email)
 *
 *  vincent.deca@neutralite.org
 *
 *  This file is part of 'UpdatesWarner'.
 *
 * 'UpdatesWarner' is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  'UpdatesWarner' is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 'UpdatesWarner'.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../app.h"

/*
  Purpose: Callback from popup menu in watcher area : Going to web site
*/
static gboolean     Callback_goto_web(GtkMenuItem *unused, gpointer param)
{
  s_wtr             *watcher;
  GError            *error = 0;

  ASSERT( !app->Gui.SelectedWatcher, "app->Gui.SelectedWatcher = 0" );

  watcher = app->Gui.SelectedWatcher;
  VERBOSE( "%s: %s: '%s'\n", watcher->Name, app->Strs[137], watcher->OfficialURL ); /* "Go to web site..." */
  if( !g_app_info_launch_default_for_uri( watcher->OfficialURL, NULL, &error ) ) {
    pid_t pid;
    if(( pid = fork()) < 0 ) {
      ERR_SYS( "%s", app->Strs[62] ); /* "Unable to launch command, fork() failed" */
    }
    else if( !pid ) {
      char *tab[3];
      tab[0] = "/usr/bin/xdg-open";
      tab[1] = watcher->OfficialURL;
      tab[2] = 0;
      execv( tab[0], tab );
      ERR_SYS( "%s: %s", app->Strs[148], watcher->OfficialURL ); /* "Unable to launch application associated with this kind of URL" */
      exit( EXIT_FAILURE );
    }
  }

  return( 0 );
}

/*
  Purpose: Callback from popup menu in watcher area : Launch without argument
*/
static gboolean     Callback_launch(GtkMenuItem *unused, gpointer param)
{
  s_gui             *gui = param;
  s_wtr             *watcher;
  pid_t             pid;

  ASSERT( !gui->SelectedWatcher, "gui->SelectedWatcher = 0" );

  watcher = gui->SelectedWatcher;
  VERBOSE( "%s: %s: %s\n", watcher->Name, app->Strs[182], watcher->LocalExec ); /* "Launch l'application" */
  if(( pid = fork()) < 0 ) {
    ERR_SYS( "%s", app->Strs[62] ); /* "Unable to launch command, fork() failed" */
  }
  else if( !pid ) {
    char *tab[2];

    tab[0] = watcher->LocalExec;
    tab[1] = 0;

    close( 0 );
    execv( tab[0], tab );
    ERR_SYS( "'%s': %s", tab[0], app->Strs[63] ); /* "Unable to launch a command, execv() failed" */
    exit( EXIT_FAILURE );
  }

  return( TRUE );
}

/*
  Purpose: Updating all officials version in a thread
*/
static void         *th_Retrieve_all_official_versions(void *param)
{
  s_wtr             *watcher;
  unsigned int      a;

  LOCK_GTK();
  gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_RELOAD], 0 );
  gtk_widget_set_sensitive( app->Gui.Main[MW_ROW_ALL_OFFI_UPDATE], 0 );
  UNLOCK_GTK();

  for( a = 0; a < app->Gui.WatcherCount; ++a ) {
    watcher = app->Gui.WatcherTable[a];
    if( watcher && !( watcher->Flags & ( WTR_FLG_DISABLE | WTR_FLG_OFFICIAL_IN_PROGRESS ))) {
      watcher->Flags &= ~( WTR_FLG_NO_AUTO_UPDATE | WTR_FLG_OFFICIAL_ERROR );
      watcher->Flags |= WTR_FLG_OFFICIAL_IN_PROGRESS | WTR_FLG_OFFICIAL_UPDATE;
      Update_watcher_area( watcher, 1 );
      Refresh_official_version( watcher );
    }
  }

  LOCK_GTK();
  gtk_widget_set_sensitive( app->Gui.PreferenceSettings[PS_BUT_RELOAD], 1 );
  gtk_widget_set_sensitive( app->Gui.Main[MW_ROW_ALL_OFFI_UPDATE], 1 );
  UNLOCK_GTK();
  return( 0 );
}

/*
  Purpose: Callback from popup menu in watcher area : Updating official version
*/
static gboolean     Callback_update_official_version(GtkMenuItem *unused, gpointer param)
{
  s_gui             *gui = param;
  pthread_t         tmp;

  ASSERT( !gui->SelectedWatcher, "gui->SelectedWatcher = 0" );
  gui->SelectedWatcher->Flags |= WTR_FLG_OFFICIAL_UPDATE;
  if( pthread_create( &tmp, &app->Attr, th_Refresh_official_version, gui->SelectedWatcher )) {
    gtk_label_set_label( GTK_LABEL(gui->SelectedWatcher->LabelOfficialVersionResult),
			 app->Strs[179] ); /* "Unable to create a thread" */
    ERR_SYS( "%s", app->Strs[179] ); /* "Unable to create a thread" */
  }
  return( TRUE );
}

/*
  Purpose: Callback from popup menu in watcher area : Updating all official versions
*/
static gboolean     Callback_update_all_official_versions(GtkMenuItem *unused, gpointer param)
{
  pthread_t         tmp;

  if( pthread_create( &tmp, &app->Attr, th_Retrieve_all_official_versions, 0 )) {
    ERR_SYS( "%s", app->Strs[179] ); /* "Unable to create a thread" */
  }
  return( TRUE );
}

/*
  Purpose: Updating local version in a thread
*/
void                *th_Retrieve_local_version(void *param)
{
  s_wtr             *watcher = param;
  unsigned int      a;

  ASSERT( !watcher, "watcher = 0" );

  if( !( watcher->Flags & WTR_FLG_LOCAL_IN_PROGRESS )) {
    watcher->Flags &= ~( WTR_FLG_NO_AUTO_UPDATE | WTR_FLG_LOCAL_ERROR );
    watcher->Flags |= WTR_FLG_LOCAL_IN_PROGRESS;
    Update_watcher_area( watcher, 1 );
    Retrieve_local_version( watcher );

    if( watcher->LocalVersion && watcher->OfficialVersion ) {
      bzero( watcher->Trigger_xor, SIZE_BUFFER_XOR );
      for( a = 0; watcher->LocalVersion[a]; ++a )
	watcher->Trigger_xor[a % SIZE_BUFFER_XOR] = watcher->LocalVersion[a];
      for( a = 0; watcher->OfficialVersion[a]; ++a )
	watcher->Trigger_xor[a % SIZE_BUFFER_XOR] ^= watcher->OfficialVersion[a];
    }

    Update_watcher_area( watcher, 1 );
  }

  return( 0 );
}

/*
  Purpose: Callback from popup menu in watcher area : "Updating local version"
  - app->SelectedWatcher must be set to valid watcher
*/
static gboolean     Callback_update_local_version(GtkMenuItem *unused, gpointer param)
{
  s_gui             *gui = param;
  pthread_t         tmp_id;
  char              *version;

  ASSERT( !gui->SelectedWatcher, "gui->SelectedWatcher = 0" );

  if(( gui->SelectedWatcher->Flags & WTR_FLG_VERSION_MISMATCH ) && !( gui->SelectedWatcher->Flags & WTR_FLG_LOCAL_AUTO )) {
    if(( version = strdup( gui->SelectedWatcher->OfficialVersion )) == NULL ) {
      ERR_SYS( "%s", app->Strs[2] ); /* "Couldn't allocate enough memory" */
      return( TRUE );
    }
    free( gui->SelectedWatcher->LocalVersion );
    gui->SelectedWatcher->LocalVersion = version;
    gtk_label_set_label( GTK_LABEL(gui->SelectedWatcher->LabelLocalVersionResult), version );
    gtk_widget_hide( gui->Main[MW_ROW_LOCAL_UPDATE] );
    gui->SelectedWatcher->Flags &= ~( WTR_FLG_VERSION_MISMATCH );
    Update_watcher_area( gui->SelectedWatcher, 0 );

    Search_section2( &gui->File, &gui->File.SectionTarget, gui->SelectedWatcher->Name );
    Generate_watcher_section( gui->SelectedWatcher, &gui->File.SectionReplace );

    if( pthread_create( &tmp_id, &app->Attr, th_Save_modifications_to_file, &gui->File )) {
      ERR_APP( "%s", app->Strs[179] ); /* "Unable to create a thread" */
      th_Save_modifications_to_file( &gui->File );
    }
  }
  else if( pthread_create( &tmp_id, &app->Attr, th_Retrieve_local_version, gui->SelectedWatcher )) {
    gtk_label_set_label( GTK_LABEL(gui->SelectedWatcher->LabelLocalVersionResult),
			 app->Strs[179] ); /* "Unable to create a thread" */
    ERR_SYS( "%s", app->Strs[179] ); /* "Unable to create a thread" */
  }
  return( TRUE );
}

/*
  Purpose: Callback from popup menu in watcher area : "Edit watcher"
  - app->SelectedWatcher must be set to valid watcher
*/
static gboolean     Callback_show_edit(GtkMenuItem *unused1, gpointer param)
{
  s_gui             *gui = param;

  ASSERT( !gui->SelectedWatcher, "gui->SelectedWatcher = 0" );

  gtk_tree_selection_select_iter( gui->SelectionWatcher, &gui->SelectedWatcher->Iter );
  Callback_edit_watcher( 0, gui );

  return( TRUE );
}

void                Create_menu_watcher()
{
  GtkWidget         *menu, *item, *img;

  menu = gtk_menu_new();
  app->Gui.Main[MW_MENU_WATCHER] = menu;

#ifdef LIB_HANDY
  item = gtk_menu_item_new_with_label( "" );
#else
  //#if GTK_MAJOR_VERSION == 2 || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 10 )
  item = gtk_image_menu_item_new_with_label( "" );
#endif
  gtk_widget_set_sensitive( item, 0 );
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );
  app->Gui.Main[MW_ROW_NAME] = item;

  //#if GTK_MAJOR_VERSION == 2 || ( GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 10 )
#  define APP_NEW_MENU_ITEM_FILE( file, label )				\
  item = gtk_image_menu_item_new_with_mnemonic( label );		\
  img = gtk_image_new_from_file( file );				\
  gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(item), img )
#  define APP_NEW_MENU_ITEM_IMG( pix, label )				\
  item = gtk_image_menu_item_new_with_mnemonic( label );		\
  img = gtk_image_new_from_pixbuf( pix );				\
  gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(item), img )

  APP_NEW_MENU_ITEM_FILE( DEFAULT_APP_RESOURCES"/icon_update_local.png", app->Strs[136] ); /* "Update local version" */
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_update_local_version), &app->Gui );
  app->Gui.Main[MW_ROW_LOCAL_UPDATE] = item;

  APP_NEW_MENU_ITEM_FILE( DEFAULT_APP_RESOURCES"/icon_update_official.png", app->Strs[138] ); /* "Update official version" */
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_update_official_version), &app->Gui );
  app->Gui.Main[MW_ROW_OFFI_UPDATE] = item;

  APP_NEW_MENU_ITEM_FILE( DEFAULT_APP_RESOURCES"/icon_update_all_official.png", app->Strs[201] ); /* "Update all official versions" */
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_update_all_official_versions), &app->Gui );
  app->Gui.Main[MW_ROW_ALL_OFFI_UPDATE] = item;

  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );

  APP_NEW_MENU_ITEM_FILE( DEFAULT_APP_RESOURCES"/icon_launch.png", app->Strs[182] ); /* "Launch l'application" */
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_launch), &app->Gui );
  app->Gui.Main[MW_ROW_LAUNCH] = item;

  APP_NEW_MENU_ITEM_FILE( DEFAULT_APP_RESOURCES"/icon_goto_web.png", app->Strs[137] ); /* "Go to web site..." */
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_goto_web), &app->Gui );
  app->Gui.Main[MW_ROW_GOTO_WEB] = item;

  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );

  APP_NEW_MENU_ITEM_IMG( app->Gui.ImgTable[IMG_MODIFY], app->Strs[208] ); /* "_Edit watcher" */
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_show_edit), &app->Gui );
  app->Gui.Main[MW_ROW_EDIT] = item;

  APP_NEW_MENU_ITEM_IMG( app->Gui.ImgTable[IMG_SETTINGS], app->Strs[14] ); /* "Settings" */
  gtk_menu_shell_append( GTK_MENU_SHELL(menu), item );
  g_signal_connect( G_OBJECT(item), "activate", G_CALLBACK(Callback_show_preferences), &app->Gui );
  app->Gui.Main[MW_ROW_SETTINGS] = item;

  gtk_widget_show_all( menu );
}
