#!/bin/bash

OLDIFS=$IFS
IFS=$(echo -en "\n\b")

#***** Title
function Title {
    clear
    echo -e "*************** UpdatesWarner ****************\n"
}

#***** Initialization
SHNAME="updates-warner"
CORE=$(grep -c "model name" /proc/cpuinfo)
missing=""

flg_gtk2=0;
flg_gtk3=0;

STR_OK="\e[1;32mok\e[0;39m"
STR_NOK="\e[1;31mnok\e[0;39m"
function Init {
    which sudo > /dev/null
    if [[ $? == 1 ]] ; then
	SUDO=""
	if [[ $(whoami) != "root" ]] ; then
	    [ $lang -eq 1 ] && echo -e "\n$ERROR 'sudo' n'est pas présent, le script doit être lancé avec des droits root\n"
	    [ $lang -eq 2 ] && echo -e "\n$ERROR 'sudo' is not available, script must be started with root rights\n"
	    exit
	fi
    else
	SUDO="sudo"
    fi
}

#***** Langue
function Choose_language {
    Title
    ERROR="\e[1;31mError\e[0;39m:"
    echo -e "\t1) Français"
    echo -e "\t2) English\n"
    while true; do
	read -p "Language ? [1/2] " ans
	case $ans in
            1 ) lang=1
		ERROR="\e[1;31mErreur\e[0;39m:"
		break;;
            2 ) lang=2
		break;;
	    * ) echo -e "\n$ERROR Invalide answer\n"
	esac
    done
}

#***** GTK
function Choose_gtk {
    Title
    echo -e "\t2) GTK 2"
    echo -e "\t3) GTK 3\n"
    while true; do
	read -p "GTK ? [2/3] " ans
	case $ans in
            2 ) flg_gtk2=1
		break;;
            3 ) flg_gtk3=1
		break;;
	    * ) echo -e "\n$ERROR Invalide answer\n"
	esac
    done
}

#***** Licence
function Agree_license {
    Title
    if [ $lang -eq 1 ] ; then
	echo -e "Ce logiciel est distribué sous licence GPL"
	echo -e "Cette licence est disponible dans le fichier 'GPL_license'\n"
	ask="Avez-vous lu, compris, accepté cette licence ? [oui/non] "
    elif [ $lang -eq 2 ] ; then
	echo -e "This software is distributed under the GNU GPL License"
	echo -e "This license is available in the 'GPL_license' file\n"
	ask="Did you read, understood, accepted it ? [yes/no] "
    fi
    while true; do
	read -p "$ask" ans
	case $ans in
            [yoYO]* )
		break;;
            * ) exit;
	esac
    done
}

#*********** Requiered ***********
function diy_install {
    if [ $lang -eq 1 ] ; then
	echo -e "\nDes paquets sont manquant pour la compilation :$name (ou superieur)"
	echo "Impossible d'utiliser /usr/bin/apt-get ou /usr/bin/yum pour les installer"
	echo "Il faut que vous installiez manuellement les pré-requis qui ont eu un 'nok'"
    elif [ $lang -eq 2 ] ; then
	echo -e "\nFailed to install the missing packages :$name (ou superieur)"
	echo "Failed to use /usr/bin/apt-get or /usr/bin/yum to install them"
	echo "You have to install manually the prerequisite which got 'nok'"
    fi
    exit
}

#***** Requiered debian
function Requiered_apt_get {
    #======= pre-req compilation
    which make > /dev/null
    if [ $? -ne 0  ] ; then
	echo -e "- make : $STR_NOK"
	missing="build-essential"
    else
	echo -e "- make : $STR_OK"
    fi

    which pkg-config > /dev/null
    if [ $? -ne 0 ] ; then
	echo -e "- pkg-config : $STR_NOK"
	missing="$missing pkg-config"
    else
	echo -e "- pkg-config : $STR_OK"
    fi

    #======= gtk
    if [ $flg_gtk2 -eq 1 ] ; then
	dpkg -l libgtk2.0-dev  > /dev/null
	if [ $? -ne 0 ] ; then
	    echo -e "- libgtk2.0-dev : $STR_NOK"
	    missing="$missing libgtk2.0-dev"
	else
	    echo -e "- libgtk2.0-dev : $STR_OK"
	fi
    fi
    if [ $flg_gtk3 -eq 1 ] ; then
	dpkg -l libgtk-3-dev  > /dev/null
	if [ $? -ne 0 ] ; then
	    echo -e "- libgtk-3-dev : $STR_NOK"
	    missing="$missing libgtk-3-dev"
	else
	    echo -e "- libgtk-3-dev : $STR_OK"
	fi
    fi

    #====== Result
    if [ "$missing" = "" ] ; then
	[ $lang -eq 1 ] && echo -e "\nTout semble présent pour la compilation\n"
	[ $lang -eq 2 ] && echo -e "\nEverything seems to be present for the compilation\n"
	return
    fi

    which apt-get > /dev/null
    if [ $? -ne 0 ] ; then
	[ $lang -eq 1 ] && echo -e "\n=> Installation des paquets manquants :$missing"
	[ $lang -eq 2 ] && echo -e "\n=> Missing packages installation :$missing"

	eval "$SUDO apt-get install $missing"
	if [ $? -ne 0 ] ; then
	    [ $lang -eq 1 ] && echo -e "\n$ERROR Quelque chose s'est mal passé lors de l'installation des pré-requis :$missing"
	    [ $lang -eq 2 ] && echo -e "\nERROR Something went wrong when installing missing packages :$missing"
	    diy_install
	fi
    else
	diy_install
    fi
}


function Requiered_dnf {
    Title

    #======= pre-req compilation
    is_64b=$(cat /proc/cpuinfo | grep flags | grep ' lm')
    if [ "$is_64b" == "" ] ; then
	arch="i686"
    else
	arch="x86_64"
    fi
    #======= pre-req compilation
    which make &> /dev/null
    if [ $? -ne 0  ] ; then
	echo -e "- make : $STR_NOK"
	missing=$(dnf search make | grep -m 1 "^make\.$arch" | cut -f1 -d ' ')
    else
	echo -e "- make : $STR_OK"
    fi

    which gcc > /dev/null
    if [ $? -ne 0 ] ; then
	echo -e "- gcc : $STR_NOK"
	missing=$(dnf search gcc | grep -m 1 "^gcc\.$arch" | cut -f1 -d ' ')
    else
	echo -e "- gcc : $STR_OK"
    fi

    #======= gtk
    export PKG_CONFIG_PATH="/usr/lib/pkgconfig/"

    if [ $flg_gtk2 -eq 1 ] ; then
	if [ ! -e /usr/include/gtk-2.0/gtk/gtk.h ] ; then
	    echo -e "- gtk2-devel : $STR_NOK"
	    [ $lang -eq 1 ] && echo "Recherche du package de developpement de gtk2..."
	    [ $lang -eq 2 ] && echo "Searching gtk2 developpement package..."
	    packet=$(dnf search gtk2 | grep -m 1 "^gtk2-devel\.$arch" | cut -f1 -d ' ')
	    missing="$missing $packet"
	else
	    echo -e "- gtk2-devel : $STR_OK"
	fi
    fi

    if [ $flg_gtk3 -eq 1 ] ; then
	if [ ! -e /usr/include/gtk-3.0/gtk/gtk.h ] ; then
	    echo -e "- gtk2-devel : $STR_NOK"
	    [ $lang -eq 1 ] && echo "Recherche du package de developpement de gtk3..."
	    [ $lang -eq 2 ] && echo "Searching gtk3 developpement package..."
	    packet=$(dnf search gtk3 | grep -m 1 "^gtk3-devel\.$arch" | cut -f1 -d ' ')
	    missing="$missing $packet"
	else
	    echo -e "- gtk3-devel : $STR_OK"
	fi
    fi

    #====== Result
    if [[ "$missing" = "" ]] ; then
	[ $lang -eq 1 ] && echo -e "\nTout semble présent pour la compilation\n"
	[ $lang -eq 2 ] && echo -e "\nEverything seems to be present for the compilation\n"
    else
	[ $lang -eq 1 ] && echo -e "\nInstallation des paquets manquants: '$missing'\n"
	[ $lang -eq 2 ] && echo -e "\nInstalling missing packages: '$missing'\n"
	eval "$SUDO dnf install $missing"
	if [ ! $? = 0 ] ; then
	    diy_install
	fi
    fi
}

#***** Pre-requisites
function Pre_requisites {
    Title
    [ $lang -eq 1 ] && echo -e "=> Vérification des pré-requis:"
    [ $lang -eq 2 ] && echo -e "=> Checking prerequisite:"

    which apt-get &> /dev/null
    if [ $? = 0 ] ; then
	echo -e "\n- Using 'apt-get'\n"
	Requiered_apt_get
    else
	which dnf &> /dev/null
	if [ $? = 0 ] ; then
	    echo -e "\n- Using 'dnf'\n"
	    Requiered_dnf
	else
	    [ $lang -eq 1 ] && echo -e "$ERROR: Distribution non determiné, il va falloir manuellement installer les packets de développement de GTK2 (ou GTK3)et compiler"
	    [ $lang -eq 2 ] && echo -e "$ERROR: Distribution not determinated, you will have to manually install GTK2 (or GTK3) developpement packets and compile"
	    exit
	fi
    fi
}

#***** Compilation
function Compile {
    if [ -e $SHNAME ] ; then
	[ $lang -eq 1 ] && echo -e "$SHNAME : Exécutable déjà créé, je saute la compilation !\n"
	[ $lang -eq 2 ] && echo -e "$SHNAME : Executable file already created, skipping compilation !\n"
    else
	[ $lang -eq 1 ] && echo -e "=> Création de l'exécutable '$SHNAME' :\n"
	[ $lang -eq 2 ] && echo -e "=> Creating the executable file '$SHNAME' :\n"

	[ $flg_gtk2 -eq 1 ] && cp Makefile.gtk2 Makefile
	[ $flg_gtk3 -eq 1 ] && cp Makefile.gtk3 Makefile

	rm -rf /tmp/updates-warner
	make start
	if [ -e $SHNAME ] ; then
	    [ $lang -eq 1 ] && echo -e "\n=> \e[1;32mCompilation de $SHNAME terminé avec succès\e[0;39m\n"
	    [ $lang -eq 2 ] && echo -e "\n=> \e[1;32m$SHNAME compilation finished with success\e[0;39m\n"
	else
	    [ $lang -eq 1 ] && echo -e "\n$ERROR Problème lors de la compilation\n=> Arrêt de l'installation\n"
	    [ $lang -eq 2 ] && echo -e "\n$ERROR A problème occured while compiling\n=> Stopping installation\n"
	    exit
	fi
    fi
}

#***** Copy executable & ressources
function verify_copy {
    if [ $? = 1 ] ; then
	[ $lang -eq 1 ] && echo -e "\nProblème lors de la copie... \nArrêt de l'installation"
	[ $lang -eq 2 ] && echo -e "\nProblem when copying... \nStopping installation"
	exit
    fi
}

function Copy_files {
    [ $lang -eq 1 ] && echo -e "=> Copie des fichiers dans le système"
    [ $lang -eq 2 ] && echo -e "=> Copying files into system"

    if [ -e $SHNAME ] ; then
	if [ ! -e "/usr/share/$SHNAME" ] ; then
	    eval "$SUDO mkdir /usr/share/$SHNAME"
	    verify_copy
	fi
        eval "$SUDO cp -R share/* /usr/share/$SHNAME/"
	verify_copy
	eval "$SUDO chmod -R 644 /usr/share/$SHNAME/*"
        eval "$SUDO cp -R languages /usr/share/$SHNAME/"
	verify_copy
	eval "$SUDO chmod -R 655 /usr/share/$SHNAME/languages"

	pid=$(ps aux | grep updates-warner | grep -v grep | awk '{ print $2 }')
	[ "${pid}" != "" ] && kill ${pid}

	eval "$SUDO cp -R $SHNAME /usr/bin/"
	verify_copy

	[ "${pid}" != "" ] && updates-warner >/dev/null &
    fi

    [ $lang -eq 1 ] && echo -e "=> Copie des pages 'man' dans '/usr/share/man/'\n"
    [ $lang -eq 2 ] && echo -e "=> Copying man pages into '/usr/share/man/'"

    eval "$SUDO cp $SHNAME.en.gz /usr/share/man/man1/$SHNAME.1.gz"
    verify_copy
    eval "$SUDO cp $SHNAME.fr.gz /usr/share/man/fr/man1/$SHNAME.1.gz"
    verify_copy
}

function Copy_config {
    [ -e ~/.config/updates-warner.conf ] && return
    [ $lang -eq 1 ] && ask="Copie d'un fichier d'exemple de configuration dans ~/.config ? [oui/non] "
    [ $lang -eq 2 ] && ask="Copy an example of a configuration file into ~/.config ? [yes/no] "

    while true; do
	read -p "$ask" ans
	case $ans in
            [yoYO]* )
		mkdir -p ~/.config
		cp updates-warner.conf ~/.config/
		sed -i "s#resources/user#$HOME/.cache/updates-warner/img#g" ~/.config/updates-warner.conf
		mkdir -p ~/.cache/updates-warner/img
		cp -R user/* ~/.cache/updates-warner/img/
		echo "- ~/.config/updates-warner.conf"
		echo -e "- ~/.cache/updates-warner/img\n"
		break;;
            * ) break;;
	esac
    done
}

#***** Main ****
Choose_language
Init
Choose_gtk
Agree_license
Pre_requisites
Compile
Copy_config
Copy_files

if [ $lang -eq 1 ] ; then
    echo -e "=> Installation terminée, tu peux maintenant lancer '$SHNAME' ou 'man $SHNAME'\n"
    echo "- Jettes un coup d'oeil au fichier changes-fr.log lors d'une mise à jour"
    echo "  Ca donne pas mal d'infos sur les fonctionnalités ajoutées"
    echo "- N'hésite pas à me contacter pour toute information / correction / idée / aide"
elif [ $lang -eq 2 ] ; then
    echo -e "=> Installation done, you can now run '$SHNAME' or 'man $SHNAME'\n"
    echo "Don't hesitate to read the changes-en.log file when updating"
    echo "It brings a lots of informations about added fonctionnalities"
    echo -e "You can contact me for some information / correction / idea / help\n"
fi

IFS=$OLDIFS
